@@ -1,8 +1,12 @@
-(* camlp5r q_MLast.cmo *)
-(* $Id: compile.ml,v 1.29 2010/02/19 09:06:34 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: compile.ml,v 6.4 2011-01-26 12:31:48 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "q_MLast.cmo";
+#load "pa_macro.cmo";
+
 open Gramext;
+open Versdep;
 
 value strict_parsing = ref False;
 value keywords = ref [];
@@ -28,9 +32,9 @@ value rec name_of_symbol entry =
 value rec name_of_symbol_failed entry =
   fun
   [ Slist0 s -> name_of_symbol_failed entry s
-  | Slist0sep s _ -> name_of_symbol_failed entry s
+  | Slist0sep s _ _ -> name_of_symbol_failed entry s
   | Slist1 s -> name_of_symbol_failed entry s
-  | Slist1sep s _ -> name_of_symbol_failed entry s
+  | Slist1sep s _ _ -> name_of_symbol_failed entry s
   | Sopt s -> name_of_symbol_failed entry s
   | Sflag s -> name_of_symbol_failed entry s
   | Stree t -> name_of_tree_failed entry t
@@ -64,10 +68,10 @@ value tree_failed entry prev_symb tree =
     | Slist1 s ->
         let txt1 = name_of_symbol_failed entry s in
         ("", txt1 ^ " or " ^ txt)
-    | Slist0sep s sep ->
+    | Slist0sep s sep _ ->
         let txt1 = name_of_symbol_failed entry s in
         ("", txt1 ^ " or " ^ txt)
-    | Slist1sep s sep ->
+    | Slist1sep s sep _ ->
         let txt1 = name_of_symbol_failed entry s in
         ("", txt1 ^ " or " ^ txt)
     | Sopt _ | Sflag _ | Stree _ -> ("", txt)
@@ -294,7 +298,7 @@ and parse_symbol entry nlevn s rkont fkont ending_act =
   | s ->
       let e = symbol_parser entry nlevn s in
       match s with
-      [ Slist0 _ | Slist0sep _ _ | Sopt _ | Sflag _ ->
+      [ Slist0 _ | Slist0sep _ _ _ | Sopt _ | Sflag _ ->
           parse_symbol_no_failure e rkont fkont ending_act
       | s ->
           parse_standard_symbol e rkont fkont ending_act ] ]
@@ -302,14 +306,16 @@ and symbol_parser entry nlevn =
   fun
   [ Slist0 s -> <:expr< P.list0 $symbol_parser entry nlevn s$ >>
   | Slist1 s -> <:expr< P.list1 $symbol_parser entry nlevn s$ >>
-  | Slist0sep s sep ->
+  | Slist0sep s sep b ->
+      let b = if b then <:expr< True >> else <:expr< False >> in
       <:expr<
         P.list0sep $symbol_parser entry nlevn s$
-          $symbol_parser entry nlevn sep$ >>
-  | Slist1sep s sep ->
+          $symbol_parser entry nlevn sep$ $b$ >>
+  | Slist1sep s sep b ->
+      let b = if b then <:expr< True >> else <:expr< False >> in
       <:expr<
          P.list1sep $symbol_parser entry nlevn s$
-           $symbol_parser entry nlevn sep$ >>
+           $symbol_parser entry nlevn sep$ $b$ >>
   | Sopt s -> <:expr< P.option $symbol_parser entry nlevn s$ >>
   | Sflag s -> <:expr< P.bool $symbol_parser entry nlevn s$ >>
   | Stree tree ->
@@ -537,9 +543,9 @@ and scan_symbol list =
   | Snterm e -> scan_entry list e
   | Snterml e l -> scan_entry list e
   | Slist0 s -> scan_symbol list s
-  | Slist0sep s sep -> scan_symbol (scan_symbol list s) sep
+  | Slist0sep s sep _ -> scan_symbol (scan_symbol list s) sep
   | Slist1 s -> scan_symbol list s
-  | Slist1sep s sep -> scan_symbol (scan_symbol list s) sep
+  | Slist1sep s sep _ -> scan_symbol (scan_symbol list s) sep
   | Sopt s -> scan_symbol list s
   | Sflag s -> scan_symbol list s
   | Stree t -> scan_tree list t
@@ -576,12 +582,22 @@ value rec expr_list =
   | [x :: l] -> <:expr< [$str:String.escaped x$ :: $expr_list l$] >> ]
 ;
 
+value list_filter =
+  IFDEF OCAML_VERSION <= OCAML_2_01 THEN
+    fun f ->
+      loop [] where rec loop accu =
+        fun
+        [ [x :: l] -> loop (if f x then [x :: accu] else accu) l
+        | [] -> List.rev accu ]
+  ELSE List.filter END
+;
+
 value compile () =
   let _ = keywords.val := [] in
   let list = List.fold_left all_entries_in_graph [] entries.val in
   let list =
-    List.filter (fun e -> List.memq e list) entries.val @
-    List.filter (fun e -> not (List.memq e entries.val)) list
+    list_filter (fun e -> List.memq e list) entries.val @
+    list_filter (fun e -> not (List.memq e entries.val)) list
   in
   let list =
     let set = ref [] in
@@ -595,14 +611,15 @@ value compile () =
   let pel = List.flatten pell in
   let si1 = <:str_item< value rec $list:pel$ >> in
   let si2 =
-    let list = List.sort compare keywords.val in
+    let list = list_sort compare keywords.val in
     <:str_item<
-      List.iter (fun kw -> P.lexer.Plexing.tok_using ("", kw))
+      let lexer = Lazy.force P.lexer in
+      List.iter (fun kw -> lexer.Plexing.tok_using ("", kw))
         $expr_list list$
     >>
   in
   let loc = Ploc.dummy in
-  ([(si1, loc); (si2, loc)], False)
+  ([(si1, loc); (si2, loc)], Some loc)
 ;
 
 Pcaml.parse_implem.val := fun _ -> compile ();
