@@ -1,7 +1,10 @@
-(* camlp5r q_MLast.cmo pa_extend.cmo *)
-(* $Id: pa_extfun.ml,v 1.10 2010/02/19 09:06:35 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pa_extfun.ml,v 6.1 2010-09-15 16:00:21 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "q_MLast.cmo";
+#load "pa_extend.cmo";
+
 open Pcaml;
 
 value not_impl name x = do {
