@@ -1,7 +1,11 @@
-(* camlp5r pa_extend.cmo pa_fstream.cmo q_MLast.cmo *)
-(* $Id: pa_extprint.ml,v 1.49 2010/02/19 09:06:35 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pa_extprint.ml,v 6.1 2010-09-15 16:00:21 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_extend.cmo";
+#load "pa_fstream.cmo";
+#load "q_MLast.cmo";
+
 open Pcaml;
 
 (** Types and Functions for [EXTEND_PRINTER] statement *)
@@ -125,8 +129,11 @@ value text_of_extprint loc el =
                 in
                 let rules =
                   List.fold_right
-                    (fun pe rules ->
-                       let rule = conv loc pe in
+                    (fun (p, wo, e) rules ->
+                       let loc =
+                         Ploc.encl (MLast.loc_of_patt p) (MLast.loc_of_expr e)
+                       in
+                       let rule = conv loc (p, wo, e) in
                        <:expr< [$rule$ :: $rules$] >>)
                     rules <:expr< [] >>
                 in
