@@ -1,7 +1,9 @@
-(* camlp5r q_MLast.cmo *)
-(* $Id: pa_lefteval.ml,v 1.15 2010/02/19 09:06:35 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pa_lefteval.ml,v 6.1 2010-09-15 16:00:21 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "q_MLast.cmo";
+
 value not_impl name x =
   let desc =
     if Obj.is_block (Obj.repr x) then
