@@ -1,6 +1,9 @@
-(* camlp5r pa_extend.cmo q_MLast.cmo *)
-(* $Id: pa_lex.ml 1804 2007-12-27 10:30:24Z deraugla $ *)
-(* Copyright (c) INRIA 2007-2008 *)
+(* camlp5r *)
+(* $Id: pa_lex.ml,v 6.1 2010-09-15 16:00:21 deraugla Exp $ *)
+(* Copyright (c) INRIA 2007-2010 *)
+
+#load "pa_extend.cmo";
+#load "q_MLast.cmo";
 
 (* Simplified syntax of parsers of characters streams *)
 
