@@ -1,5 +1,5 @@
 ;; camlp5 ./pa_lispr.cmo pa_extend.cmo q_MLast.cmo pr_dump.cmo
-;; $Id: pa_lisp.ml,v 1.25 2010/02/19 09:06:35 deraugla Exp $
+;; $Id: pa_lisp.ml,v 6.7 2010-11-13 07:35:45 deraugla Exp $
 ;; Copyright (c) INRIA 2007-2010
 
 (open Pcaml)
@@ -23,9 +23,10 @@
 ;; Lexer
 
 (value rec skip_to_eol
-       (parser
-        (((` (or '\n' '\r'))) ())
-        (((` _) s) (skip_to_eol s))))
+       (lambda len
+        (parser
+         (((` (as (or '\n' '\r') c))) (Buff.store len c))
+         (((` c) s) (skip_to_eol (Buff.store len c) s)))))
 
 (value no_ident (list '(' ')' ' ' '\t' '\n' '\r' ';'))
 
@@ -74,11 +75,9 @@
         (((` x) s) (char_or_quote_id x s))))
 
 (value rec
- lexer
+ next_token_after_spaces
  (lambda kwt
    (parser bp
-           (((` (or ' ' '\t' '\n' '\r')) s) (lexer kwt s))
-           (((` ';') (a (semi kwt bp))) a)
            (((` '(')) (, (, "" "(") (, bp (+ bp 1))))
            (((` ')')) (, (, "" ")") (, bp (+ bp 1))))
            (((` '"') (s (string 0))) ep (, (, "STRING" s) (, bp ep)))
@@ -94,16 +93,11 @@
                                     ((_) "LIDENT"))))))
               (, (, con s) (, bp ep))))
            (() (, (, "EOI" "") (, bp (+ bp 1))))))
- semi
- (lambda (kwt bp)
-   (parser
-    (((` ';') (_ skip_to_eol) s) (lexer kwt s))
-    (() ep (, (, "" ";") (, bp ep)))))
  less
  (parser
   (((` ':') (lab (label 0)) (? (` '<') "'<' expected") (q (quotation 0)))
    (, "QUOT" (^ lab (^ ":" q))))
-  (()  (, "LIDENT" "<")))
+  (() (, "LIDENT" "<")))
  label
  (lambda len
    (parser
@@ -122,6 +116,21 @@
     (((` '>')) (Buff.get len))
     (((a (quotation (Buff.store len '>')))) a))))
 
+(value get_buff (lambda (len _) (Buff.get len)))
+
+(value rec
+ lexer
+ (lambda (len kwt)
+  (parser bp
+   (((` (as (or ' ' '\t' '\n' '\r') c)) s) (lexer (Buff.store len c) kwt s))
+   (((` ';') (a (semi (Buff.store len ';') kwt bp))) a)
+   (((comm  (get_buff len)) (a (next_token_after_spaces kwt))) (, comm a))))
+ semi
+ (lambda (len kwt bp)
+   (parser
+    (((` ';') (len (skip_to_eol (Buff.store len ';'))) s) (lexer len kwt s))
+    (() ep (, (Buff.get len) (, (, "" ";") (, bp ep)))))))
+
 (value lexer_using
        (lambda (kwt (, con prm))
          (match con
@@ -146,8 +155,9 @@
          (let ((kwt (Hashtbl.create 89))
                (lexer2
                 (lambda (kwt (, s _ _))
-                  (let (((, t loc) (lexer kwt s)))
-                    (, t (Ploc.make_unlined loc))))))
+                  (let (((, comm (, t loc)) (lexer 0 kwt s)))
+                    (, t
+                       (Ploc.make_loc Plexing.input_file.val 0 0 loc comm))))))
            ({}
             (Plexing.tok_func (Plexing.lexer_func_of_parser (lexer2 kwt)))
             (Plexing.tok_using (lexer_using kwt))
@@ -541,18 +551,19 @@
                  ((se)
                   (error se "type declaration")))))
      (let* ((empty (list))
-            (td ({} (MLast.tdNam (, loc1 <:vala< n1 >>))
+            (n (, loc1 <:vala< n1 >>))
+            (td ({} (MLast.tdNam <:vala< n >>)
                     (MLast.tdPrm <:vala< tpl >>) (MLast.tdPrv <:vala< False >>)
                     (MLast.tdDef (ctyp_se se2))
                     (MLast.tdCon <:vala< empty >>))))
           (list td :: (type_declaration_list_se sel)))))
   ((list) (list))
-  ((list se :: _) (error se "type_declaration")))
+  ((list se :: _) (error se "type_decl")))
  type_parameter_se
  (lambda_match
   ((Satom _ Alid s) (&& (>= (String.length s) 2) (= ([] s 0) '''))
    (let ((s (String.sub s 1 (- (String.length s) 1))))
-      (, <:vala< s >> (, False False))))
+      (, <:vala< (Some s) >> None)))
   (se
    (error se "type_parameter")))
  ctyp_se
@@ -590,7 +601,7 @@
  constructor_declaration_se
  (lambda_match
   ((Sexpr loc (list (Satom _ Auid ci) :: sel))
-   (, loc <:vala< ci >> <:vala< (List.map ctyp_se sel) >>))
+   (, loc <:vala< ci >> <:vala< (List.map ctyp_se sel) >> None))
   (se
    (error se "constructor_declaration"))))
 
@@ -642,7 +653,8 @@
 
 EXTEND
   implem :
-    [ [ st = LIST0 [ s = str_item -> (, s loc) ]; EOI -> (, st False) ] ]
+    [ [ st = LIST0 [ s = str_item -> (, s loc) ];
+        eoi_loc = [ EOI -> loc ] -> (, st (Some eoi_loc)) ] ]
   ;
   top_phrase :
     [ [ se = sexpr -> (Some (top_phrase_se se))
