@@ -1,7 +1,10 @@
-(* camlp5r pa_extend.cmo q_MLast.cmo *)
-(* $Id: pa_o.ml,v 1.84 2010/02/19 09:06:35 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pa_o.ml,v 6.33 2010-11-16 16:48:21 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_extend.cmo";
+#load "q_MLast.cmo";
+
 open Pcaml;
 
 Pcaml.syntax_name.val := "OCaml";
@@ -20,11 +23,13 @@ do {
   Grammar.Unsafe.clear_entry module_expr;
   Grammar.Unsafe.clear_entry sig_item;
   Grammar.Unsafe.clear_entry str_item;
+  Grammar.Unsafe.clear_entry signature;
+  Grammar.Unsafe.clear_entry structure;
   Grammar.Unsafe.clear_entry expr;
   Grammar.Unsafe.clear_entry patt;
   Grammar.Unsafe.clear_entry ctyp;
   Grammar.Unsafe.clear_entry let_binding;
-  Grammar.Unsafe.clear_entry type_declaration;
+  Grammar.Unsafe.clear_entry type_decl;
   Grammar.Unsafe.clear_entry constructor_declaration;
   Grammar.Unsafe.clear_entry label_declaration;
   Grammar.Unsafe.clear_entry match_case;
@@ -39,20 +44,6 @@ do {
 Pcaml.parse_interf.val := Grammar.Entry.parse interf;
 Pcaml.parse_implem.val := Grammar.Entry.parse implem;
 
-value neg_string n =
-  let len = String.length n in
-  if len > 0 && n.[0] = '-' then String.sub n 1 (len - 1) else "-" ^ n
-;
-
-value mkumin loc f arg =
-  match arg with
-  [ <:expr< $int:n$ >> -> <:expr< $int:neg_string n$ >>
-  | <:expr< $flo:n$ >> -> <:expr< $flo:neg_string n$ >>
-  | _ ->
-      let f = "~" ^ f in
-      <:expr< $lid:f$ $arg$ >> ]
-;
-
 value mklistexp loc last =
   loop True where rec loop top =
     fun
@@ -321,7 +312,9 @@ value get_seq =
   | e -> [e] ]
 ;
 
-value mem_tvar s tpl = List.exists (fun (t, _) -> Pcaml.unvala t = s) tpl;
+value mem_tvar s tpl =
+  List.exists (fun (t, _) -> Pcaml.unvala t = Some s) tpl
+;
 
 value choose_tvar tpl =
   let rec find_alpha v =
@@ -339,23 +332,54 @@ value choose_tvar tpl =
   | None -> make_n 1 ]
 ;
 
+value quotation_content s = do {
+  loop 0 where rec loop i =
+    if i = String.length s then ("", s)
+    else if s.[i] = ':' || s.[i] = '@' then
+      let i = i + 1 in
+      (String.sub s 0 i, String.sub s i (String.length s - i))
+    else loop (i + 1)
+};
+
+value concat_comm loc e =
+  let loc =
+    Ploc.with_comment loc
+      (Ploc.comment loc ^ Ploc.comment (MLast.loc_of_expr e))
+  in
+  let floc =
+    let first = ref True in
+    fun loc1 ->
+      if first.val then do {first.val := False; loc}
+      else loc1
+  in
+  Reloc.expr floc 0 e
+;
+
 EXTEND
-  GLOBAL: sig_item str_item ctyp patt expr module_type module_expr class_type
-    class_expr class_sig_item class_str_item let_binding type_declaration
-    constructor_declaration label_declaration match_case with_constr
-    poly_variant;
+  GLOBAL: sig_item str_item ctyp patt expr module_type module_expr
+    signature structure class_type class_expr class_sig_item class_str_item
+    let_binding type_decl constructor_declaration label_declaration
+    match_case with_constr poly_variant;
   module_expr:
     [ [ "functor"; "("; i = V UIDENT "uid" ""; ":"; t = module_type; ")";
         "->"; me = SELF ->
           <:module_expr< functor ( $_uid:i$ : $t$ ) -> $me$ >>
-      | "struct"; st = V (LIST0 [ s = str_item; OPT ";;" -> s ]); "end" ->
+      | "struct"; st = structure; "end" ->
           <:module_expr< struct $_list:st$ end >> ]
-    | [ me1 = SELF; me2 = SELF -> <:module_expr< $me1$ $me2$ >> ]
+    | [ me1 = SELF; "."; me2 = SELF -> <:module_expr< $me1$ . $me2$ >> ]
+    | [ me1 = SELF; "("; me2 = SELF; ")" -> <:module_expr< $me1$ $me2$ >> ]
     | [ i = mod_expr_ident -> i
+      | "("; "val"; e = expr; ":"; mt = module_type; ")" ->
+         <:module_expr< (value $e$ : $mt$) >>
+      | "("; "val"; e = expr; ")" ->
+         <:module_expr< (value $e$) >>
       | "("; me = SELF; ":"; mt = module_type; ")" ->
           <:module_expr< ( $me$ : $mt$ ) >>
       | "("; me = SELF; ")" -> <:module_expr< $me$ >> ] ]
   ;
+  structure:
+    [ [ st = V (LIST0 [ s = str_item; OPT ";;" -> s ]) -> st ] ]
+  ;
   mod_expr_ident:
     [ LEFTA
       [ i = SELF; "."; j = SELF -> <:module_expr< $i$ . $j$ >> ]
@@ -363,7 +387,8 @@ EXTEND
   ;
   str_item:
     [ "top"
-      [ "exception"; (_, c, tl) = constructor_declaration; b = rebind_exn ->
+      [ "exception"; (_, c, tl, _) = constructor_declaration;
+        b = rebind_exn ->
           <:str_item< exception $_uid:c$ of $_list:tl$ = $_list:b$ >>
       | "external"; i = V LIDENT "lid" ""; ":"; t = ctyp; "=";
         pd = V (LIST1 STRING) ->
@@ -378,7 +403,7 @@ EXTEND
           <:str_item< module type $_uid:i$ = $mt$ >>
       | "open"; i = V mod_ident "list" "" ->
           <:str_item< open $_:i$ >>
-      | "type"; tdl = V (LIST1 type_declaration SEP "and") ->
+      | "type"; tdl = V (LIST1 type_decl SEP "and") ->
           <:str_item< type $_list:tdl$ >>
       | "let"; r = V (FLAG "rec"); l = V (LIST1 let_binding SEP "and"); "in";
         x = expr ->
@@ -417,11 +442,16 @@ EXTEND
           <:module_type< functor ( $_uid:i$ : $t$ ) -> $mt$ >> ]
     | [ mt = SELF; "with"; wcl = V (LIST1 with_constr SEP "and") ->
           <:module_type< $mt$ with $_list:wcl$ >> ]
-    | [ "sig"; sg = V (LIST0 [ s = sig_item; OPT ";;" -> s ]); "end" ->
+    | [ "sig"; sg = signature; "end" ->
           <:module_type< sig $_list:sg$ end >>
+      | "module"; "type"; "of"; me = module_expr ->
+          <:module_type< module type of $me$ >>
       | i = mod_type_ident -> i
       | "("; mt = SELF; ")" -> <:module_type< $mt$ >> ] ]
   ;
+  signature:
+    [ [ sg = V (LIST0 [ s = sig_item; OPT ";;" -> s ]) -> sg ] ]
+  ;
   mod_type_ident:
     [ LEFTA
       [ m1 = SELF; "."; m2 = SELF -> <:module_type< $m1$ . $m2$ >>
@@ -431,7 +461,7 @@ EXTEND
   ;
   sig_item:
     [ "top"
-      [ "exception"; (_, c, tl) = constructor_declaration ->
+      [ "exception"; (_, c, tl, _) = constructor_declaration ->
           <:sig_item< exception $_uid:c$ of $_list:tl$ >>
       | "external"; i = V LIDENT "lid" ""; ":"; t = ctyp; "=";
         pd = V (LIST1 STRING) ->
@@ -450,7 +480,7 @@ EXTEND
           <:sig_item< module type $_uid:i$ = 'abstract >>
       | "open"; i = V mod_ident "list" "" ->
           <:sig_item< open $_:i$ >>
-      | "type"; tdl = V (LIST1 type_declaration SEP "and") ->
+      | "type"; tdl = V (LIST1 type_decl SEP "and") ->
           <:sig_item< type $_list:tdl$ >>
       | "val"; i = V LIDENT "lid" ""; ":"; t = ctyp ->
           <:sig_item< value $_lid:i$ : $t$ >>
@@ -472,8 +502,13 @@ EXTEND
     [ [ "type"; tpl = V type_parameters "list"; i = V mod_ident ""; "=";
         pf = V (FLAG "private"); t = ctyp ->
           <:with_constr< type $_:i$ $_list:tpl$ = $_flag:pf$ $t$ >>
+      | "type"; tpl = V type_parameters "list"; i = V mod_ident ""; ":=";
+        t = ctyp ->
+          <:with_constr< type $_:i$ $_list:tpl$ := $t$ >>
       | "module"; i = V mod_ident ""; "="; me = module_expr ->
-          <:with_constr< module $_:i$ = $me$ >> ] ]
+          <:with_constr< module $_:i$ = $me$ >>
+      | "module"; i = V mod_ident ""; ":="; me = module_expr ->
+          <:with_constr< module $_:i$ := $me$ >> ] ]
   ;
   (* Core expressions *)
   expr:
@@ -491,8 +526,8 @@ EXTEND
           <:expr< let module $_uid:m$ = $mb$ in $e$ >>
       | "function"; OPT "|"; l = V (LIST1 match_case SEP "|") ->
           <:expr< fun [ $_list:l$ ] >>
-      | "fun"; p = patt LEVEL "simple"; e = fun_def ->
-          <:expr< fun [$p$ -> $e$] >>
+      | "fun"; p = patt LEVEL "simple"; (eo, e) = fun_def ->
+          <:expr< fun [$p$ $opt:eo$ -> $e$] >>
       | "match"; e = SELF; "with"; OPT "|";
         l = V (LIST1 match_case SEP "|") ->
           <:expr< match $e$ with [ $_list:l$ ] >>
@@ -558,8 +593,8 @@ EXTEND
       | e1 = SELF; "lsr"; e2 = SELF -> <:expr< $e1$ lsr $e2$ >>
       | e1 = SELF; op = infixop4; e2 = SELF -> <:expr< $lid:op$ $e1$ $e2$ >> ]
     | "unary minus" NONA
-      [ "-"; e = SELF -> <:expr< $mkumin loc "-" e$ >>
-      | "-."; e = SELF -> <:expr< $mkumin loc "-." e$ >> ]
+      [ "-"; e = SELF -> <:expr< - $e$ >>
+      | "-."; e = SELF -> <:expr< -. $e$ >> ]
     | "apply" LEFTA
       [ e1 = SELF; e2 = SELF ->
           let (e1, e2) =
@@ -579,13 +614,22 @@ EXTEND
       | "assert"; e = SELF -> <:expr< assert $e$ >>
       | "lazy"; e = SELF -> <:expr< lazy ($e$) >> ]
     | "." LEFTA
-      [ e1 = SELF; "."; "("; e2 = SELF; ")" -> <:expr< $e1$ .( $e2$ ) >>
+      [ e1 = SELF; "."; "("; op = operator_rparen ->
+          <:expr< $e1$ .( $lid:op$ ) >>
+      | e1 = SELF; "."; "("; e2 = SELF; ")" ->
+          <:expr< $e1$ .( $e2$ ) >>
       | e1 = SELF; "."; "["; e2 = SELF; "]" -> <:expr< $e1$ .[ $e2$ ] >>
       | e = SELF; "."; "{"; el = V (LIST1 expr LEVEL "+" SEP ","); "}" ->
           <:expr< $e$ .{ $_list:el$ } >>
-      | e1 = SELF; "."; e2 = SELF -> <:expr< $e1$ . $e2$ >> ]
+      | e1 = SELF; "."; e2 = SELF ->
+          let rec loop m =
+            fun
+            [ <:expr< $x$ . $y$ >> -> loop <:expr< $m$ . $x$ >> y
+            | e -> <:expr< $m$ . $e$ >> ]
+          in
+          loop e1 e2 ]
     | "~-" NONA
-      [ "!"; e = SELF -> <:expr< $e$ . val>>
+      [ "!"; e = SELF -> <:expr< $e$ . val >>
       | "~-"; e = SELF -> <:expr< ~- $e$ >>
       | "~-."; e = SELF -> <:expr< ~-. $e$ >>
       | f = prefixop; e = SELF -> <:expr< $lid:f$ $e$ >> ]
@@ -597,8 +641,8 @@ EXTEND
       | s = V FLOAT -> <:expr< $_flo:s$ >>
       | s = V STRING -> <:expr< $_str:s$ >>
       | c = V CHAR -> <:expr< $_chr:c$ >>
-      | UIDENT "True" -> <:expr< $uid:" True"$ >>
-      | UIDENT "False" -> <:expr< $uid:" False"$ >>
+      | UIDENT "True" -> <:expr< True_ >>
+      | UIDENT "False" -> <:expr< False_ >>
       | i = expr_ident -> i
       | "false" -> <:expr< False >>
       | "true" -> <:expr< True >>
@@ -612,26 +656,25 @@ EXTEND
       | "{"; e = expr LEVEL "."; "with"; lel = V lbl_expr_list "list"; "}" ->
           <:expr< { ($e$) with $_list:lel$ } >>
       | "("; ")" -> <:expr< () >>
+      | "("; "module"; me = module_expr; ":"; mt = module_type; ")" ->
+          <:expr< (module $me$ : $mt$) >>
+      | "("; "module"; me = module_expr; ")" ->
+          <:expr< (module $me$) >>
       | "("; op = operator_rparen -> <:expr< $lid:op$ >>
       | "("; el = V e_phony "list"; ")" -> <:expr< ($_list:el$) >>
       | "("; e = SELF; ":"; t = ctyp; ")" -> <:expr< ($e$ : $t$) >>
-      | "("; e = SELF; ")" -> <:expr< $e$ >>
-      | "begin"; e = SELF; "end" -> <:expr< $e$ >>
+      | "("; e = SELF; ")" -> concat_comm loc <:expr< $e$ >>
+      | "begin"; e = SELF; "end" -> concat_comm loc <:expr< $e$ >>
       | "begin"; "end" -> <:expr< () >>
       | x = QUOTATION ->
-          let x =
-            try
-              let i = String.index x ':' in
-              (String.sub x 0 i,
-               String.sub x (i + 1) (String.length x - i - 1))
-            with
-            [ Not_found -> ("", x) ]
-          in
-          Pcaml.handle_expr_quotation loc x ] ]
+          let con = quotation_content x in
+          Pcaml.handle_expr_quotation loc con ] ]
   ;
   let_binding:
     [ [ p = val_ident; e = fun_binding -> (p, e)
-      | p = patt; "="; e = expr -> (p, e) ] ]
+      | p = patt; "="; e = expr -> (p, e)
+      | p = patt; ":"; t = poly_type; "="; e = expr ->
+          (<:patt< ($p$ : $t$) >>, e) ] ]
   ;
   val_ident:
     [ [ check_not_part_of_patt; s = LIDENT -> <:patt< $lid:s$ >>
@@ -641,7 +684,7 @@ EXTEND
     [ RIGHTA
       [ p = patt LEVEL "simple"; e = SELF -> <:expr< fun $p$ -> $e$ >>
       | "="; e = expr -> <:expr< $e$ >>
-      | ":"; t = ctyp; "="; e = expr -> <:expr< ($e$ : $t$) >> ] ]
+      | ":"; t = poly_type; "="; e = expr -> <:expr< ($e$ : $t$) >> ] ]
   ;
   match_case:
     [ [ x1 = patt; w = V (OPT [ "when"; e = expr -> e ]); "->"; x2 = expr ->
@@ -656,14 +699,14 @@ EXTEND
     [ [ i = patt_label_ident; "="; e = expr LEVEL "expr1" -> (i, e) ] ]
   ;
   expr1_semi_list:
-    [ [ e = expr LEVEL "expr1"; ";"; el = SELF -> [e :: el]
-      | e = expr LEVEL "expr1"; ";" -> [e]
-      | e = expr LEVEL "expr1" -> [e] ] ]
+    [ [ el = LIST1 (expr LEVEL "expr1") SEP ";" OPT_SEP -> el ] ]
   ;
   fun_def:
     [ RIGHTA
-      [ p = patt LEVEL "simple"; e = SELF -> <:expr< fun $p$ -> $e$ >>
-      | "->"; e = expr -> <:expr< $e$ >> ] ]
+      [ p = patt LEVEL "simple"; (eo, e) = SELF ->
+          (None, <:expr< fun [ $p$ $opt:eo$ -> $e$ ] >>)
+      | eo = OPT [ "when"; e = expr -> e ]; "->"; e = expr ->
+          (eo, <:expr< $e$ >>) ] ]
   ;
   expr_ident:
     [ RIGHTA
@@ -730,8 +773,8 @@ EXTEND
       | s = V FLOAT -> <:patt< $_flo:s$ >>
       | s = V STRING -> <:patt< $_str:s$ >>
       | s = V CHAR -> <:patt< $_chr:s$ >>
-      | UIDENT "True" -> <:patt< $uid:" True"$ >>
-      | UIDENT "False" -> <:patt< $uid:" False"$ >>
+      | UIDENT "True" -> <:patt< True_ >>
+      | UIDENT "False" -> <:patt< False_ >>
       | "false" -> <:patt< False >>
       | "true" -> <:patt< True >>
       | "["; "]" -> <:patt< [] >>
@@ -746,17 +789,15 @@ EXTEND
       | "("; pl = V p_phony "list"; ")" -> <:patt< ($_list:pl$) >>
       | "("; p = SELF; ":"; t = ctyp; ")" -> <:patt< ($p$ : $t$) >>
       | "("; p = SELF; ")" -> <:patt< $p$ >>
+      | "("; "type"; s = V LIDENT; ")" -> <:patt< (type $_lid:s$) >>
+      | "("; "module"; s = V UIDENT; ":"; mt = module_type; ")" ->
+          <:patt< (module $_uid:s$ : $mt$) >>
+      | "("; "module"; s = V UIDENT; ")" ->
+          <:patt< (module $_uid:s$) >>
       | "_" -> <:patt< _ >>
       | x = QUOTATION ->
-          let x =
-            try
-              let i = String.index x ':' in
-              (String.sub x 0 i,
-               String.sub x (i + 1) (String.length x - i - 1))
-            with
-            [ Not_found -> ("", x) ]
-          in
-          Pcaml.handle_patt_quotation loc x ] ]
+          let con = quotation_content x in
+          Pcaml.handle_patt_quotation loc con ] ]
   ;
   patt_semi_list:
     [ [ p = patt; ";"; pl = SELF -> [p :: pl]
@@ -779,15 +820,13 @@ EXTEND
       | i = LIDENT -> <:patt< $lid:i$ >> ] ]
   ;
   (* Type declaration *)
-  type_declaration:
-    [ [ tpl = type_parameters; n = type_patt; "="; pf = V (FLAG "private");
+  type_decl:
+    [ [ tpl = type_parameters; n = V type_patt; "="; pf = V (FLAG "private");
         tk = type_kind; cl = V (LIST0 constrain) ->
-          {MLast.tdNam = n; MLast.tdPrm = <:vala< tpl >>;
-           MLast.tdPrv = pf; MLast.tdDef = tk; MLast.tdCon = cl}
-      | tpl = type_parameters; n = type_patt; cl = V (LIST0 constrain) ->
-          {MLast.tdNam = n; MLast.tdPrm = <:vala< tpl >>;
-           MLast.tdPrv = <:vala< False >>;
-           MLast.tdDef = <:ctyp< '$choose_tvar tpl$ >>; MLast.tdCon = cl} ] ]
+          <:type_decl< $_tp:n$ $list:tpl$ = $_priv:pf$ $tk$ $_list:cl$ >>
+      | tpl = type_parameters; n = V type_patt; cl = V (LIST0 constrain) ->
+          let tk = <:ctyp< '$choose_tvar tpl$ >> in
+          <:type_decl< $_tp:n$ $list:tpl$ = $tk$ $_list:cl$ >> ] ]
   ;
   type_patt:
     [ [ n = V LIDENT -> (loc, n) ] ]
@@ -801,10 +840,12 @@ EXTEND
           <:ctyp< [ $list:cdl$ ] >>
       | t = ctyp ->
           <:ctyp< $t$ >>
-      | t = ctyp; "="; "{"; ldl = V label_declarations "list"; "}" ->
-          <:ctyp< $t$ == { $_list:ldl$ } >>
-      | t = ctyp; "="; OPT "|"; cdl = LIST1 constructor_declaration SEP "|" ->
-          <:ctyp< $t$ == [ $list:cdl$ ] >>
+      | t = ctyp; "="; pf = FLAG "private"; "{";
+        ldl = V label_declarations "list"; "}" ->
+          <:ctyp< $t$ == $priv:pf$ { $_list:ldl$ } >>
+      | t = ctyp; "="; pf = FLAG "private"; OPT "|";
+        cdl = LIST1 constructor_declaration SEP "|" ->
+          <:ctyp< $t$ == $priv:pf$ [ $list:cdl$ ] >>
       | "{"; ldl = V label_declarations "list"; "}" ->
           <:ctyp< { $_list:ldl$ } >> ] ]
   ;
@@ -814,19 +855,34 @@ EXTEND
       | "("; tpl = LIST1 type_parameter SEP ","; ")" -> tpl ] ]
   ;
   type_parameter:
-    [ [ "'"; i = V ident "" -> (i, (False, False))
-      | "+"; "'"; i = V ident "" -> (i, (True, False))
-      | "-"; "'"; i = V ident "" -> (i, (False, True)) ] ]
+    [ [ "+"; p = V simple_type_parameter -> (p, Some True)
+      | "-"; p = V simple_type_parameter -> (p, Some False)
+      | p = V simple_type_parameter -> (p, None) ] ]
+  ;
+  simple_type_parameter:
+    [ [ "'"; i = ident -> Some i
+      | "_" -> None ] ]
   ;
   constructor_declaration:
     [ [ ci = cons_ident; "of"; cal = V (LIST1 (ctyp LEVEL "apply") SEP "*") ->
-          (loc, ci, cal)
-      | ci = cons_ident -> (loc, ci, <:vala< [] >>) ] ]
+          (loc, ci, cal, None)
+      | ci = cons_ident; ":"; cal = V (LIST1 (ctyp LEVEL "apply") SEP "*");
+        "->"; t = ctyp ->
+          (loc, ci, cal, Some t)
+      | ci = cons_ident; ":"; cal = V (LIST1 (ctyp LEVEL "apply") SEP "*") ->
+          let t =
+            match cal with
+            [ <:vala< [t] >> -> t
+            | <:vala< [t :: tl] >> -> <:ctyp< ($list:[t :: tl]$) >>
+            | _ -> assert False ]
+          in
+          (loc, ci, <:vala< [] >>, Some t)
+      | ci = cons_ident -> (loc, ci, <:vala< [] >>, None) ] ]
   ;
   cons_ident:
     [ [ i = V UIDENT "uid" "" -> i
-      | UIDENT "True" -> <:vala< " True" >>
-      | UIDENT "False" -> <:vala< " False" >> ] ]
+      | UIDENT "True" -> <:vala< "True_" >>
+      | UIDENT "False" -> <:vala< "False_" >> ] ]
   ;
   label_declarations:
     [ [ ld = label_declaration; ";"; ldl = SELF -> [ld :: ldl]
@@ -855,6 +911,7 @@ EXTEND
       | "_" -> <:ctyp< _ >>
       | i = V LIDENT -> <:ctyp< $_lid:i$ >>
       | i = V UIDENT -> <:ctyp< $_uid:i$ >>
+      | "("; "module"; mt = module_type; ")" -> <:ctyp< module $mt$ >>
       | "("; t = SELF; ","; tl = LIST1 ctyp SEP ","; ")";
         i = ctyp LEVEL "ctyp2" ->
           List.fold_left (fun c a -> <:ctyp< $c$ $a$ >>) i [t :: tl]
@@ -925,9 +982,9 @@ EXTEND
     | "simple"
       [ "["; ct = ctyp; ","; ctcl = LIST1 ctyp SEP ","; "]";
         ci = class_longident ->
-          <:class_expr< $list:ci$ [ $list:[ct :: ctcl]$ ] >>
+          <:class_expr< [ $list:[ct :: ctcl]$ ] $list:ci$ >>
       | "["; ct = ctyp; "]"; ci = class_longident ->
-          <:class_expr< $list:ci$ [ $ct$ ] >>
+          <:class_expr< [ $ct$ ] $list:ci$ >>
       | ci = class_longident -> <:class_expr< $list:ci$ >>
       | "object"; cspo = V (OPT class_self_patt);
         cf = V class_structure "list"; "end" ->
@@ -946,26 +1003,38 @@ EXTEND
   class_str_item:
     [ [ "inherit"; ce = class_expr; pb = V (OPT [ "as"; i = LIDENT -> i ]) ->
           <:class_str_item< inherit $ce$ $_opt:pb$ >>
-      | "val"; mf = V (FLAG "mutable"); lab = V label "lid" "";
-        e = cvalue_binding ->
-          <:class_str_item< value $_flag:mf$ $_lid:lab$ = $e$ >>
-      | "method"; "private"; "virtual"; l = V label "lid" ""; ":";
+      | "val"; ov = V (FLAG "!") "!"; mf = V (FLAG "mutable");
+        lab = V LIDENT "lid" ""; e = cvalue_binding ->
+          <:class_str_item< value $_!:ov$ $_flag:mf$ $_lid:lab$ = $e$ >>
+      | "val"; ov = V (FLAG "!") "!"; mf = V (FLAG "mutable");
+        "virtual"; lab = V LIDENT "lid" ""; ":"; t = ctyp ->
+          if Pcaml.unvala ov then
+            Ploc.raise loc (Stream.Error "virtual value cannot override")
+          else
+            <:class_str_item< value virtual $_flag:mf$ $_lid:lab$ : $t$ >>
+      | "val"; "virtual"; mf = V (FLAG "mutable"); lab = V LIDENT "lid" "";
+        ":"; t = ctyp ->
+          <:class_str_item< value virtual $_flag:mf$ $_lid:lab$ : $t$ >>
+      | "method"; "private"; "virtual"; l = V LIDENT "lid" ""; ":";
         t = poly_type ->
           <:class_str_item< method virtual private $_lid:l$ : $t$ >>
-      | "method"; "virtual"; "private"; l = V label "lid" ""; ":";
+      | "method"; "virtual"; "private"; l = V LIDENT "lid" ""; ":";
         t = poly_type ->
           <:class_str_item< method virtual private $_lid:l$ : $t$ >>
-      | "method"; "virtual"; l = V label "lid" ""; ":"; t = poly_type ->
+      | "method"; "virtual"; l = V LIDENT "lid" ""; ":"; t = poly_type ->
           <:class_str_item< method virtual $_lid:l$ : $t$ >>
-      | "method"; "private"; l = V label "lid" ""; ":"; t = poly_type; "=";
-        e = expr ->
-          <:class_str_item< method private $_lid:l$ : $t$ = $e$ >>
-      | "method"; "private"; l = V label "lid" ""; sb = fun_binding ->
-          <:class_str_item< method private $_lid:l$ = $sb$ >>
-      | "method"; l = V label "lid" ""; ":"; t = poly_type; "="; e = expr ->
-          <:class_str_item< method $_lid:l$ : $t$ = $e$ >>
-      | "method"; l = V label "lid" ""; sb = fun_binding ->
-          <:class_str_item< method $_lid:l$ = $sb$ >>
+      | "method"; ov = V (FLAG "!") "!"; "private"; l = V LIDENT "lid" "";
+        ":"; t = poly_type; "="; e = expr ->
+          <:class_str_item< method $_!:ov$ private $_lid:l$ : $t$ = $e$ >>
+      | "method"; ov = V (FLAG "!") "!"; "private"; l = V LIDENT "lid" "";
+        sb = fun_binding ->
+          <:class_str_item< method $_!:ov$ private $_lid:l$ = $sb$ >>
+      | "method"; ov = V (FLAG "!") "!"; l = V LIDENT "lid" ""; ":";
+        t = poly_type; "="; e = expr ->
+          <:class_str_item< method $_!:ov$ $_lid:l$ : $t$ = $e$ >>
+      | "method"; ov = V (FLAG "!") "!"; l = V LIDENT "lid" "";
+        sb = fun_binding ->
+          <:class_str_item< method $_!:ov$ $_lid:l$ = $sb$ >>
       | "constraint"; t1 = ctyp; "="; t2 = ctyp ->
           <:class_str_item< type $t1$ = $t2$ >>
       | "initializer"; se = expr -> <:class_str_item< initializer $se$ >> ] ]
@@ -988,12 +1057,15 @@ EXTEND
       | cs = class_signature -> cs ] ]
   ;
   class_signature:
-    [ [ "["; tl = LIST1 ctyp SEP ","; "]"; id = clty_longident ->
-          <:class_type< $list:id$ [ $list:tl$ ] >>
-      | id = clty_longident -> <:class_type< $list:id$ >>
+    [ [ "["; tl = LIST1 ctyp SEP ","; "]"; id = SELF ->
+          <:class_type< $id$ [ $list:tl$ ] >>
       | "object"; cst = V (OPT class_self_type);
         csf = V (LIST0 class_sig_item); "end" ->
-          <:class_type< object $_opt:cst$ $_list:csf$ end >> ] ]
+          <:class_type< object $_opt:cst$ $_list:csf$ end >> ]
+    | [ ct1 = SELF; "."; ct2 = SELF -> <:class_type< $ct1$ . $ct2$ >>
+      | ct1 = SELF; "("; ct2 = SELF; ")" -> <:class_type< $ct1$ $ct2$ >> ]
+    | [ i = V LIDENT -> <:class_type< $_id: i$ >>
+      | i = V UIDENT -> <:class_type< $_id: i$ >> ] ]
   ;
   class_self_type:
     [ [ "("; t = ctyp; ")" -> t ] ]
@@ -1001,19 +1073,19 @@ EXTEND
   class_sig_item:
     [ [ "inherit"; cs = class_signature ->
           <:class_sig_item< inherit $cs$ >>
-      | "val"; mf = V (FLAG "mutable"); l = V label "lid" ""; ":"; t = ctyp ->
+      | "val"; mf = V (FLAG "mutable"); l = V LIDENT "lid" ""; ":"; t = ctyp ->
           <:class_sig_item< value $_flag:mf$ $_lid:l$ : $t$ >>
-      | "method"; "private"; "virtual"; l = V label "lid" ""; ":";
+      | "method"; "private"; "virtual"; l = V LIDENT "lid" ""; ":";
         t = poly_type ->
           <:class_sig_item< method virtual private $_lid:l$ : $t$ >>
-      | "method"; "virtual"; "private"; l = V label "lid" ""; ":";
+      | "method"; "virtual"; "private"; l = V LIDENT "lid" ""; ":";
         t = poly_type ->
           <:class_sig_item< method virtual private $_lid:l$ : $t$ >>
-      | "method"; "virtual"; l = V label "lid" ""; ":"; t = poly_type ->
+      | "method"; "virtual"; l = V LIDENT "lid" ""; ":"; t = poly_type ->
           <:class_sig_item< method virtual $_lid:l$ : $t$ >>
-      | "method"; "private"; l = V label "lid" ""; ":"; t = poly_type ->
+      | "method"; "private"; l = V LIDENT "lid" ""; ":"; t = poly_type ->
           <:class_sig_item< method private $_lid:l$ : $t$ >>
-      | "method"; l = V label "lid" ""; ":"; t = poly_type ->
+      | "method"; l = V LIDENT "lid" ""; ":"; t = poly_type ->
           <:class_sig_item< method $_lid:l$ : $t$ >>
       | "constraint"; t1 = ctyp; "="; t2 = ctyp ->
           <:class_sig_item< type $t1$ = $t2$ >> ] ]
@@ -1039,7 +1111,7 @@ EXTEND
           <:expr< object $_opt:cspo$ $_list:cf$ end >> ] ]
   ;
   expr: LEVEL "."
-    [ [ e = SELF; "#"; lab = V label "lid" -> <:expr< $e$ # $_lid:lab$ >> ] ]
+    [ [ e = SELF; "#"; lab = V LIDENT "lid" -> <:expr< $e$ # $_lid:lab$ >> ] ]
   ;
   expr: LEVEL "simple"
     [ [ "("; e = SELF; ":"; t = ctyp; ":>"; t2 = ctyp; ")" ->
@@ -1079,15 +1151,13 @@ EXTEND
     [ [ "'"; i = ident -> i ] ]
   ;
   poly_type:
-    [ [ test_typevar_list_dot; tpl = LIST1 typevar; "."; t2 = ctyp ->
+    [ [ "type"; nt = LIST1 LIDENT; "."; ct = ctyp ->
+          <:ctyp< type $list:nt$ . $ct$ >>
+      | test_typevar_list_dot; tpl = LIST1 typevar; "."; t2 = ctyp ->
           <:ctyp< ! $list:tpl$ . $t2$ >>
       | t = ctyp -> t ] ]
   ;
   (* Identifiers *)
-  clty_longident:
-    [ [ m = UIDENT; "."; l = SELF -> [m :: l]
-      | i = LIDENT -> [i] ] ]
-  ;
   class_longident:
     [ [ m = UIDENT; "."; l = SELF -> [m :: l]
       | i = LIDENT -> [i] ] ]
@@ -1116,26 +1186,28 @@ EXTEND
       | "`"; i = V ident ""; "of"; ao = V (FLAG "&");
         l = V (LIST1 ctyp SEP "&") ->
           <:poly_variant< `$_:i$ of $_flag:ao$ $_list:l$ >>
-      | t = ctyp -> MLast.PvInh t ] ]
+      | t = ctyp -> <:poly_variant< $t$ >> ] ]
   ;
   name_tag:
     [ [ "`"; i = ident -> i ] ]
   ;
   expr: LEVEL "expr1"
-    [ [ "fun"; p = labeled_patt; e = fun_def -> <:expr< fun $p$ -> $e$ >> ] ]
+    [ [ "fun"; p = labeled_patt; (eo, e) = fun_def ->
+          <:expr< fun [ $p$ $opt:eo$ -> $e$ ] >> ] ]
   ;
   expr: AFTER "apply"
     [ "label"
-      [ i = V TILDEIDENTCOLON; e = SELF -> <:expr< ~$_:i$: $e$ >>
-      | i = V TILDEIDENT -> <:expr< ~$_:i$ >>
-      | i = V QUESTIONIDENTCOLON; e = SELF -> <:expr< ?$_:i$: $e$ >>
-      | i = V QUESTIONIDENT -> <:expr< ?$_:i$ >> ] ]
+      [ i = V TILDEIDENTCOLON; e = SELF -> <:expr< ~{$_:i$ = $e$} >>
+      | i = V TILDEIDENT -> <:expr< ~{$_:i$} >>
+      | i = V QUESTIONIDENTCOLON; e = SELF -> <:expr< ?{$_:i$ = $e$} >>
+      | i = V QUESTIONIDENT -> <:expr< ?{$_:i$} >> ] ]
   ;
   expr: LEVEL "simple"
     [ [ "`"; s = V ident "" -> <:expr< ` $_:s$ >> ] ]
   ;
   fun_def:
-    [ [ p = labeled_patt; e = SELF -> <:expr< fun $p$ -> $e$ >> ] ]
+    [ [ p = labeled_patt; (eo, e) = SELF ->
+          (None, <:expr< fun [ $p$ $opt:eo$ -> $e$ ] >>) ] ]
   ;
   fun_binding:
     [ [ p = labeled_patt; e = SELF -> <:expr< fun $p$ -> $e$ >> ] ]
@@ -1147,33 +1219,35 @@ EXTEND
   ;
   labeled_patt:
     [ [ i = V TILDEIDENTCOLON; p = patt LEVEL "simple" ->
-           <:patt< ~$_:i$: $p$ >>
+           <:patt< ~{$_:i$ = $p$} >>
       | i = V TILDEIDENT ->
-           <:patt< ~$_:i$ >>
+           <:patt< ~{$_:i$} >>
       | "~"; "("; i = LIDENT; ")" ->
-           <:patt< ~$i$ >>
+           <:patt< ~{$lid:i$} >>
       | "~"; "("; i = LIDENT; ":"; t = ctyp; ")" ->
-           <:patt< ~$i$: ($lid:i$ : $t$) >>
+           <:patt< ~{$lid:i$ : $t$} >>
       | i = V QUESTIONIDENTCOLON; j = LIDENT ->
-           <:patt< ?$_:i$: ($lid:j$) >>
+           <:patt< ?{$_:i$ = ?{$lid:j$}} >>
+      | i = V QUESTIONIDENTCOLON; "_" ->
+           <:patt< ?{$_:i$} >>
       | i = V QUESTIONIDENTCOLON; "("; p = patt; "="; e = expr; ")" ->
-          <:patt< ?$_:i$: ( $p$ = $e$ ) >>
+          <:patt< ?{$_:i$ = ?{$p$ = $e$}} >>
       | i = V QUESTIONIDENTCOLON; "("; p = patt; ":"; t = ctyp; ")" ->
-          <:patt< ?$_:i$: ( $p$ : $t$ ) >>
+          <:patt< ?{$_:i$ = ?{$p$ : $t$}} >>
       | i = V QUESTIONIDENTCOLON; "("; p = patt; ":"; t = ctyp; "=";
         e = expr; ")" ->
-          <:patt< ?$_:i$: ( $p$ : $t$ = $e$ ) >>
+          <:patt< ?{$_:i$ = ?{$p$ : $t$ = $e$}} >>
       | i = V QUESTIONIDENTCOLON; "("; p = patt; ")" ->
-          <:patt< ?$_:i$: ( $p$ ) >>
-      | i = V QUESTIONIDENT -> <:patt< ?$_:i$ >>
+          <:patt< ?{$_:i$ = ?{$p$}} >>
+      | i = V QUESTIONIDENT -> <:patt< ?{$_:i$} >>
       | "?"; "("; i = LIDENT; "="; e = expr; ")" ->
-          <:patt< ? ( $lid:i$ = $e$ ) >>
+          <:patt< ?{$lid:i$ = $e$} >>
       | "?"; "("; i = LIDENT; ":"; t = ctyp; "="; e = expr; ")" ->
-          <:patt< ? ( $lid:i$ : $t$ = $e$ ) >>
+          <:patt< ?{$lid:i$ : $t$ = $e$} >>
       | "?"; "("; i = LIDENT; ")" ->
-          <:patt< ?$i$ >>
+          <:patt< ?{$lid:i$} >>
       | "?"; "("; i = LIDENT; ":"; t = ctyp; ")" ->
-          <:patt< ? ( $lid:i$ : $t$ ) >> ] ]
+          <:patt< ?{$lid:i$ : $t$} >> ] ]
   ;
   class_type:
     [ [ i = LIDENT; ":"; t = ctyp LEVEL "apply"; "->"; ct = SELF ->
@@ -1201,8 +1275,8 @@ EXTEND
   interf:
     [ [ si = sig_item_semi; (sil, stopped) = SELF -> ([si :: sil], stopped)
       | "#"; n = LIDENT; dp = OPT expr; ";;" ->
-          ([(<:sig_item< # $lid:n$ $opt:dp$ >>, loc)], True)
-      | EOI -> ([], False) ] ]
+          ([(<:sig_item< # $lid:n$ $opt:dp$ >>, loc)], None)
+      | EOI -> ([], Some loc) ] ]
   ;
   sig_item_semi:
     [ [ si = sig_item; OPT ";;" -> (si, loc) ] ]
@@ -1210,8 +1284,8 @@ EXTEND
   implem:
     [ [ si = str_item_semi; (sil, stopped) = SELF -> ([si :: sil], stopped)
       | "#"; n = LIDENT; dp = OPT expr; ";;" ->
-          ([(<:str_item< # $lid:n$ $opt:dp$ >>, loc)], True)
-      | EOI -> ([], False) ] ]
+          ([(<:str_item< # $lid:n$ $opt:dp$ >>, loc)], None)
+      | EOI -> ([], Some loc) ] ]
   ;
   str_item_semi:
     [ [ si = str_item; OPT ";;" -> (si, loc) ] ]
