@@ -1,7 +1,10 @@
-(* camlp5r pa_extend.cmo q_MLast.cmo *)
-(* $Id: pa_oop.ml,v 1.9 2010/02/19 09:06:35 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pa_oop.ml,v 6.1 2010-09-15 16:00:21 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_extend.cmo";
+#load "q_MLast.cmo";
+
 open Pcaml;
 
 type spat_comp =
