@@ -1,11 +1,16 @@
-(* camlp5r pa_macro.cmo q_MLast.cmo -qmod ctyp,Type *)
-(* $Id: pa_pragma.ml,v 1.63 2010/02/19 09:06:35 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pa_pragma.ml,v 6.4 2010-09-30 20:41:54 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_macro.cmo";
+#load "q_MLast.cmo";
+#qmod "ctyp,Type";
+
 (* expressions evaluated in the context of the preprocessor *)
 (* syntax at toplevel: #pragma <expr> *)
 
 open Printf;
+open Versdep;
 
 value string_of_obj_tag x =
   if Obj.is_block (Obj.repr x) then
@@ -454,18 +459,18 @@ value val_tab = do {
         let t = ty_var () in
         {ctyp =
            <:ctyp<
-             Gramext.g_symbol $t$ -> Gramext.g_symbol $t$ ->
+             Gramext.g_symbol $t$ -> Gramext.g_symbol $t$ -> bool ->
                Gramext.g_symbol $t$ >>;
-         expr = Obj.repr (fun s1 s2 -> Gramext.Slist0sep s1 s2);
+         expr = Obj.repr (fun s1 s2 b -> Gramext.Slist0sep s1 s2 b);
          patt = no_patt loc});
      ("Gramext.Slist1sep",
       fun loc ->
         let t = ty_var () in
         {ctyp =
            <:ctyp<
-             Gramext.g_symbol $t$ -> Gramext.g_symbol $t$ ->
+             Gramext.g_symbol $t$ -> Gramext.g_symbol $t$ -> bool ->
                Gramext.g_symbol $t$ >>;
-         expr = Obj.repr (fun s1 s2 -> Gramext.Slist1sep s1 s2);
+         expr = Obj.repr (fun s1 s2 b -> Gramext.Slist1sep s1 s2 b);
          patt = no_patt loc});
      ("Gramext.Snext",
       fun loc ->
@@ -1006,20 +1011,24 @@ value val_tab = do {
         {ctyp = <:ctyp< bool >>;
          expr = Obj.repr True;
          patt = std_patt True});
-     ("type_declaration",
+     ("type_decl",
       fun loc ->
         {ctyp = <:ctyp< Grammar.Entry.e MLast.type_decl >>;
-         expr = Obj.repr Pcaml.type_declaration;
+         expr = Obj.repr Pcaml.type_decl;
          patt = no_patt loc})];
   ht
 };
 
+IFDEF OCAML_VERSION <= OCAML_1_07 THEN
+  value with_ctyp e t = {ctyp = t; expr = e.expr; patt = e.patt};
+END;
+
 value rec eval_expr env e =
   let loc = MLast.loc_of_expr e in
   match e with
   [ <:expr< fun [ $list:pel$ ] >> ->
       eval_fun loc env pel
-  | <:expr< let $opt:rf$ $list:pel$ in $e$ >> ->
+  | <:expr< let $flag:rf$ $list:pel$ in $e$ >> ->
       eval_let loc env rf pel e
   | <:expr< match $e$ with [ $list:pel$ ] >> ->
       eval_match loc env e pel
@@ -1108,7 +1117,7 @@ and eval_match loc env e pel =
   | None ->
       raise
         (Match_failure
-           (Pcaml.input_file.val, Ploc.line_nb loc,
+           (Ploc.file_name loc, Ploc.line_nb loc,
             Ploc.first_pos loc - Ploc.bol_pos loc)) ]
 
 and eval_try loc env e pel =
@@ -1136,7 +1145,7 @@ and eval_let loc env rf pel e =
             loop extra_env pel
         | [] -> extra_env ]
     in
-    let new_env = List.rev_append extra_env env in
+    let new_env = list_rev_append extra_env env in
     List.iter
       (fun (s, bv) -> bv.valu := eval_expr new_env (Obj.magic bv.valu))
       extra_env;
@@ -1190,7 +1199,7 @@ and eval_fun loc env pel =
     | None ->
         raise
           (Match_failure
-             (Pcaml.input_file.val, Ploc.line_nb loc,
+             (Ploc.file_name loc, Ploc.line_nb loc,
               Ploc.first_pos loc - Ploc.bol_pos loc)) ]
   in
   {ctyp = t; expr = Obj.repr e; patt = no_patt loc}
