@@ -1,5 +1,5 @@
 ; camlp5 ./pa_schemer.cmo pa_extend.cmo q_MLast.cmo pr_dump.cmo
-; $Id: pa_scheme.ml,v 1.94 2010/02/19 09:06:35 deraugla Exp $
+; $Id: pa_scheme.ml,v 6.7 2010-11-13 07:35:45 deraugla Exp $
 ; Copyright (c) INRIA 2007-2010
 
 (open Pcaml)
@@ -35,10 +35,10 @@
 
 ; Lexer
 
-(definerec skip_to_eol
+(definerec (skip_to_eol len)
  (parser
-  (((` (or '\n' '\r'))) ())
-  (((` _) (a skip_to_eol) !) a)))
+  (((` (as (or '\n' '\r') c))) (Buff.store len c))
+  (((` c) (a (skip_to_eol (Buff.store len c))) !) a)))
 
 (define no_ident ['(' ')' '[' ']' '{' '}' ' ' '\t' '\n' '\r' ';' '.'])
 
@@ -161,7 +161,7 @@
    (Ploc.raise (Ploc.make_unlined (values bp ep))
     (Failure "antiquotation not terminated")))))
 
-(define antiloc (Printf.sprintf "%d,%d:%s"))
+(define (antiloc d1 d2 s) (Printf.sprintf "%d,%d:%s" d1 d2 s))
 
 (definerec (antiquot_loc bp len)
  (parser
@@ -178,15 +178,8 @@
     (Failure "antiquotation not terminated")))))
 
 (definerec*
- ((lexer kwt)
+ ((next_token_after_spaces kwt)
   (parser bp
-   (((` (or '\t' '\r')) (a (lexer kwt)) !) a)
-   (((` ' ') (a (after_space kwt)) !) a)
-   (((` ';') (_ skip_to_eol) (a (lexer kwt)) !) a)
-   (((` '\n') s)
-    (if Sys.interactive.val
-     (values (values "NL" "") (values bp (+ bp 1)))
-     (lexer kwt s)))
    (((` '(')) (values (values "" "(") (values bp (+ bp 1))))
    (((` ')')) (values (values "" ")") (values bp (+ bp 1))))
    (((` '[')) (values (values "" "[") (values bp (+ bp 1))))
@@ -208,10 +201,6 @@
    (((` c) (len (ident (Buff.store 0 c)))) ep
     (values (identifier kwt (Buff.get len)) (values bp ep)))
    (() (values (values "EOI" "") (values bp (+ bp 1))))))
- ((after_space kwt)
-  (parser
-   (((` '.')) ep (values (values "SPACEDOT" "") (values (- ep 1) ep)))
-   (((a (lexer kwt))) a)))
  ((dollar bp kwt strm)
   (if Plexer.force_antiquot_loc.val
    (values "ANTIQUOT_LOC" (antiquot_loc bp 0 strm))
@@ -250,6 +239,29 @@
    (((` '>')) (Buff.get len))
    (((a (quotation (Buff.store len '>')))) a))))
 
+(define (get_buff len _) (Buff.get len))
+
+(definerec*
+ ((lexer len kwt)
+  (parser bp
+   (((` (as (or '\t' '\r') c)) (a (lexer (Buff.store len c) kwt)) !) a)
+   (((` ' ') (a (after_space (Buff.store len ' ') kwt)) !) a)
+   (((` ';') (len (skip_to_eol (Buff.store len ';'))) (a (lexer len kwt)) !)
+    a)
+   (((` '\n') s)
+    (let ((len (Buff.store len '\n')))
+     (if Sys.interactive.val
+      (values (Buff.get len) (values (values "NL" "") (values bp (+ bp 1))))
+      (lexer len kwt s))))
+   (((comm (get_buff len)) (a (next_token_after_spaces kwt)))
+    (values comm a))))
+ ((after_space len kwt)
+  (parser
+   (((` '.')) ep
+    (values (Buff.get len)
+     (values (values "SPACEDOT" "") (values (- ep 1) ep))))
+   (((a (lexer len kwt))) a))))
+
 (define (lexer_using kwt (values con prm))
  (match con
   ((or "CHAR" "DOT" "EOI" "INT" "INT_l" "INT_L" "INT_n" "FLOAT" "LIDENT" "NL"
@@ -271,8 +283,8 @@
  (let
   ((kwt (Hashtbl.create 89))
    ((lexer2 kwt (values s _ _))
-    (let (((values t loc) (lexer kwt s)))
-     (values t (Ploc.make_unlined loc)))))
+    (let (((values comm (values t loc)) (lexer 0 kwt s)))
+     (values t (Ploc.make_loc Plexing.input_file.val 1 0 loc comm)))))
   {(Plexing.tok_func (Plexing.lexer_func_of_parser (lexer2 kwt)))
    (Plexing.tok_using (lexer_using kwt)) (Plexing.tok_removing (lambda))
    (Plexing.tok_match Plexing.default_match) (Plexing.tok_text lexer_text)
@@ -585,15 +597,15 @@
    ((Schar loc s) <:expr< $_chr:s$ >>)
    ((Sstring loc s) <:expr< $_str:s$ >>)
    ((Sexpr loc [(Slid _ "~") se])
-    (let ((s (anti_lid_or_error se))) <:expr< ~$_:s$ >>))
+    (let ((s (anti_lid_or_error se))) <:expr< ~{$_:s$} >>))
    ((Sexpr loc [(Slid _ "~") se1 se2])
     (let* ((s (anti_lid_or_error se1)) (e (expr_se se2)))
-     <:expr< ~$_:s$: $e$ >>))
+     <:expr< ~{$_:s$ = $e$} >>))
    ((Sexpr loc [(Slid _ "?") se1 se2])
     (let* ((s (anti_lid_or_error se1)) (e (expr_se se2)))
-     <:expr< ?$_:s$: $e$ >>))
+     <:expr< ?{$_:s$ = $e$} >>))
    ((Sexpr loc [(Slid _ "?") se])
-    (let ((s (anti_lid_or_error se))) <:expr< ?$_:s$ >>))
+    (let ((s (anti_lid_or_error se))) <:expr< ?{$_:s$} >>))
    ((Sexpr loc []) <:expr< () >>)
    ((when (Sexpr loc [(Slid _ s) e1 . (as [_ . _] sel)])
      (List.mem s assoc_left_parsed_op_list))
@@ -901,23 +913,23 @@
    ((Schar loc s) <:patt< $_chr:s$ >>)
    ((Sstring loc s) <:patt< $_str:s$ >>)
    ((Sexpr loc [(Slid _ "~") se])
-    (let ((s (anti_lid_or_error se))) <:patt< ~$_:s$ >>))
+    (let ((s (anti_lid_or_error se))) <:patt< ~{$_:s$} >>))
    ((Sexpr loc [(Slid _ "~") se1 se2])
     (let ((s (anti_lid_or_error se1)) (p (patt_se se2)))
-     <:patt< ~$_:s$: $p$ >>))
+     <:patt< ~{$_:s$ = $p$} >>))
    ((Sexpr loc [(Slid _ "?") se])
     (match se
      ((Sexpr _ [se1 se2])
-      (let ((s (anti_lid_or_error se1)) (p (patt_se se2)))
-       <:patt< ?$_:s$: ($p$) >>))
-     (se (let ((s (anti_lid_or_error se))) <:patt< ?$_:s$ >>))))
+      (let ((s (anti_lid_or_error se1)) (e (expr_se se2)))
+       <:patt< ?{$_:s$ = $e$} >>))
+     (se (let ((s (anti_lid_or_error se))) <:patt< ?{$_:s$} >>))))
    ((Sexpr loc [(Slid _ "?") se1 se2])
     (let ((e (expr_se se2)))
      (match se1
       ((Sexpr _ [se1 se2])
        (let ((s (anti_lid_or_error se1)) (p (patt_se se2)))
-        <:patt< ?$_:s$: ($p$ = $e$) >>))
-      (se (let ((s (anti_lid_or_error se))) <:patt< ? ($_:s$ = $e$) >>)))))
+        <:patt< ?{$_:s$ = ?{$p$ = $e$}} >>))
+      (se (let ((s (anti_lid_or_error se))) <:patt< ?{$_:s$ = $e$} >>)))))
    ((Srec loc sel)
     (let ((lpl (anti_list_map (label_patt_se loc) sel)))
      <:patt< { $_list:lpl$ } >>))
@@ -963,24 +975,24 @@
   (lambda_match
    ((Slid loc "_") (Left <:patt< _ >>))
    ((Slid loc s) (Left <:patt< $lid:(rename_id s)$ >>))
-   ((Sexpr loc [(Slid _ "~") (Slid _ s)]) (Left <:patt< ~$s$ >>))
+   ((Sexpr loc [(Slid _ "~") (Slid _ s)]) (Left <:patt< ~{$lid:s$} >>))
    ((Sexpr loc [(Slid _ "~") (Slid _ s) se])
-    (let ((p (patt_se se))) (Left <:patt< ~$s$: $p$ >>)))
+    (let ((p (patt_se se))) (Left <:patt< ~{$lid:s$ = $p$} >>)))
    ((Sexpr loc [(Slid _ "?") se])
     (match se
-     ((Sexpr _ [se1 se2])
-      (let ((s (anti_lid_or_error se1)) (p (patt_se se2)))
-       (Left <:patt< ?$_:s$: ($p$) >>)))
-     (se (let ((s (anti_lid_or_error se))) (Left <:patt< ?$_:s$ >>)))))
+     ((Sexpr _ [se1 (Slid _ p)])
+      (let ((s (anti_lid_or_error se1)))
+       (Left <:patt< ?{$_:s$ = $lid:p$} >>)))
+     (se (let ((s (anti_lid_or_error se))) (Left <:patt< ?{$_:s$} >>)))))
    ((Sexpr loc [(Slid _ "?") se1 se2])
     (let ((e (expr_se se2)))
      (match se1
-      ((Sexpr _ [se1 se2])
-       (let ((s (anti_lid_or_error se1)) (p (patt_se se2)))
-        (Left <:patt< ?$_:s$: ($p$ = $e$) >>)))
+      ((Sexpr _ [se1 (Slid _ p)])
+       (let ((s (anti_lid_or_error se1)))
+        (Left <:patt< ?{$_:s$ = ?{$lid:p$ = $e$}} >>)))
       (se
        (let ((s (anti_lid_or_error se)))
-        (Left <:patt< ? ($_:s$ = $e$) >>))))))
+        (Left <:patt< ?{$_:s$ = $e$} >>))))))
    ((Sexpr loc [(Slid _ ":") se1 se2])
     (let* ((p (ipatt_se se1)) (t (ctyp_se se2)))
      (Left <:patt< ($p$ : $t$) >>)))
@@ -998,17 +1010,18 @@
  (type_declaration_se
   (lambda_match
    ((Sexpr loc [se1 se2])
-    (let
+    (let*
      (((values n1 loc1 tpl)
        (match se1
         ((Sexpr _ [(Slid loc n) . sel])
          (values (rename_id n) loc (List.map type_param_se sel)))
         ((Slid loc n) (values (rename_id n) loc []))
-        (se (error se "type declaration")))))
-     {(MLast.tdNam (values loc1 <:vala< n1 >>)) (MLast.tdPrm <:vala< tpl >>)
+        (se (error se "type declaration"))))
+      (n (values loc1 <:vala< n1 >>)))
+     {(MLast.tdNam <:vala< n >>) (MLast.tdPrm <:vala< tpl >>)
       (MLast.tdPrv <:vala< False >>) (MLast.tdDef (ctyp_se se2))
       (MLast.tdCon <:vala< [] >>)}))
-   (se (error se "type_declaration"))))
+   (se (error se "type_decl"))))
  (type_declaration_list_se
   (lambda_match
    ([se1 se2 . sel]
@@ -1019,26 +1032,26 @@
          (values (rename_id n) loc (List.map type_param_se sel)))
         ((Slid loc n) (values (rename_id n) loc []))
         (se (error se "type declaration"))))
+      (n (values loc1 <:vala< n1 >>))
       (td
-       {(MLast.tdNam (values loc1 <:vala< n1 >>)) (MLast.tdPrm <:vala< tpl >>)
+       {(MLast.tdNam <:vala< n >>) (MLast.tdPrm <:vala< tpl >>)
         (MLast.tdPrv <:vala< False >>) (MLast.tdDef (ctyp_se se2))
         (MLast.tdCon <:vala< [] >>)}))
      [td . (type_declaration_list_se sel)]))
    ([] [])
-   ([se . _] (error se "type_declaration"))))
+   ([se . _] (error se "type_decl"))))
  ((type_param_se se)
   (match se
    ((when (Slid _ s) (&& (>= (String.length s) 2) (= s.[0] ''')))
     (let ((s (String.sub s 1 (- (String.length s) 1))))
-     (values <:vala< s >> (values False False))))
+     (values <:vala< (Some s) >> None)))
    ((when (Slid _ s) (&& (>= (String.length s) 3) (= s.[1] ''')))
     (let
      ((vara
-       (if (= s.[0] '+')
-        (values True False)
-        (if (= s.[0] '-') (values False True) (error se "type_param"))))
+       (if (= s.[0] '+') (Some True)
+        (if (= s.[0] '-') (Some False) (error se "type_param"))))
       (s (String.sub s 2 (- (String.length s) 2))))
-     (values <:vala< s >> vara)))
+     (values <:vala< (Some s) >> vara)))
    (se (error se "type_param"))))
  (ctyp_se
   (lambda_match
@@ -1111,7 +1124,8 @@
  (constructor_declaration_se
   (lambda_match
    ((Sexpr loc [(Suid _ ci) . sel])
-    (values loc <:vala< (rename_id ci) >> <:vala< (List.map ctyp_se sel) >>))
+    (values loc <:vala< (rename_id ci) >> <:vala< (List.map ctyp_se sel) >>
+       None))
    (se (error se "constructor_declaration"))))
  (variant_declaration_se
   (lambda_match
@@ -1124,7 +1138,8 @@
         (sel (values False sel))))
       (tl (List.map ctyp_se sel)))
      <:poly_variant< ` $s$ of $flag:a$ $list:tl$ >>))
-   (se (let ((t (ctyp_se se))) <:poly_variant< $t$ >>))))
+   (se (let* ((t (ctyp_se se)) (loc (loc_of_sexpr se)))
+       <:poly_variant< $t$ >>))))
  (label_declaration_se
   (lambda_match
    ((Sexpr loc [(Slid _ lab) (Slid _ "mutable") se])
@@ -1247,7 +1262,7 @@
  (Grammar.Unsafe.clear_entry module_type)
  (Grammar.Unsafe.clear_entry with_constr)
  (Grammar.Unsafe.clear_entry let_binding)
- (Grammar.Unsafe.clear_entry type_declaration)
+ (Grammar.Unsafe.clear_entry type_decl)
  (Grammar.Unsafe.clear_entry class_type)
  (Grammar.Unsafe.clear_entry class_expr)
  (Grammar.Unsafe.clear_entry class_sig_item)
@@ -1264,22 +1279,22 @@ EXTEND
   implem :
     [ [ "#" / se = sexpr ->
           (let (((values n dp) (directive_se se)))
-             (values [(values <:str_item< # $lid:n$ $opt:dp$ >> loc)] True))
+             (values [(values <:str_item< # $lid:n$ $opt:dp$ >> loc)] None))
       | si = str_item / x = SELF ->
           (let* (((values sil stopped) x)
                  (loc (MLast.loc_of_str_item si)))
              (values [(values si loc) . sil] stopped))
-      | EOI -> (values [] False) ] ]
+      | EOI -> (values [] (Some loc)) ] ]
   /
   interf :
     [ [ "#" / se = sexpr ->
           (let (((values n dp) (directive_se se)))
-             (values [(values <:sig_item< # $lid:n$ $opt:dp$ >> loc)] True))
+             (values [(values <:sig_item< # $lid:n$ $opt:dp$ >> loc)] None))
       | si = sig_item / x = SELF ->
           (let* (((values sil stopped) x)
                  (loc (MLast.loc_of_sig_item si)))
              (values [(values si loc) . sil] stopped))
-      | EOI -> (values [] False) ] ]
+      | EOI -> (values [] (Some loc)) ] ]
   /
   top_phrase :
     [ [ "#" / se = sexpr ->
