@@ -1,7 +1,10 @@
-(* camlp5r pa_extend.cmo q_MLast.cmo *)
-(* $Id: pa_sml.ml,v 1.27 2010/02/19 09:06:35 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pa_sml.ml,v 6.4 2010-11-13 07:35:45 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_extend.cmo";
+#load "q_MLast.cmo";
+
 open Pcaml;
 
 value ocaml_records = ref False;
@@ -41,7 +44,9 @@ value get_seq =
   | e -> [e] ]
 ;
 
-value mem_tvar s tpl = List.exists (fun (t, _) -> Pcaml.unvala t = s) tpl;
+value mem_tvar s tpl =
+  List.exists (fun (t, _) -> Pcaml.unvala t = Some s) tpl
+;
 
 value choose_tvar tpl =
   let rec find_alpha v =
@@ -179,19 +184,20 @@ value extract_label_types loc tn tal cdol =
              let new_tn = tn ^ "_" ^ c in
              let loc = MLast.loc_of_ctyp anon_record_type in
              let aux_def =
-               {MLast.tdNam = (loc, <:vala< new_tn >>);
+               {MLast.tdNam = <:vala< (loc, <:vala< new_tn >>) >>;
                 MLast.tdPrm = <:vala< [] >>; MLast.tdPrv = <:vala< False >>;
                 MLast.tdDef = anon_record_type; MLast.tdCon = <:vala< [] >>}
              in
              let tl = [<:ctyp< $lid:new_tn$ >>] in
-             ([(loc, <:vala< c >>, <:vala< tl >>) :: cdl], [aux_def :: aux])
-         | None -> ([(loc, <:vala< c >>, <:vala< tl >>) :: cdl], aux) ])
+             ([(loc, <:vala< c >>, <:vala< tl >>, None) :: cdl],
+                [aux_def :: aux])
+         | None -> ([(loc, <:vala< c >>, <:vala< tl >>, None) :: cdl], aux) ])
       cdol ([], [])
   in
   let td1 =
-    {MLast.tdNam = (loc, <:vala< tn >>); MLast.tdPrm = <:vala< tal >>;
-     MLast.tdPrv = <:vala< False >>; MLast.tdDef = <:ctyp< [ $list:cdl$ ] >>;
-     MLast.tdCon = <:vala< [] >>}
+    {MLast.tdNam = <:vala< (loc, <:vala< tn >>) >>;
+     MLast.tdPrm = <:vala< tal >>; MLast.tdPrv = <:vala< False >>;
+     MLast.tdDef = <:ctyp< [ $list:cdl$ ] >>; MLast.tdCon = <:vala< [] >>}
   in
   [td1 :: aux]
 ;
@@ -346,7 +352,7 @@ EXTEND
     [ [ x = interdec; EOI -> x ] ]
   ;
   interf:
-    [ [ x = LIST1 [ s = sig_item; OPT ";" -> (s, loc) ] -> (x, False) ] ]
+    [ [ x = LIST1 [ s = sig_item; OPT ";" -> (s, loc) ] -> (x, Some loc) ] ]
   ;
   top_phrase:
     [ [ ph = phrase; ";" -> Some ph
@@ -379,9 +385,11 @@ EXTEND
   tyvarseq: [ [ -> not_impl loc "tyvarseq" ] ];
 
   tyvar_pc:
-    [ [ "'"; x1 = V LIDENT -> [(x1, (False, False))]
-      | "'"; x1 = V LIDENT; ","; l = tyvar_pc ->
-          [(x1, (False, False)) :: l] ] ]
+    [ [ "'"; x1 = V tyvar -> [(x1, None)]
+      | "'"; x1 = V tyvar; ","; l = tyvar_pc -> [(x1, None) :: l] ] ]
+  ;
+  tyvar:
+    [ [ v = LIDENT -> Some v ] ]
   ;
   id:
     [ [ x1 = idd -> x1
@@ -639,21 +647,22 @@ EXTEND
   ;
   tb:
     [ [ x1 = tyvars; x2 = idd; "="; x3 = ctyp ->
-          {MLast.tdNam = (loc, <:vala< uncap x2 >>);
+          {MLast.tdNam = <:vala< (loc, <:vala< uncap x2 >>) >>;
            MLast.tdPrm = <:vala< x1 >>; MLast.tdPrv = <:vala< False >>;
            MLast.tdDef = x3; MLast.tdCon = <:vala< [] >>}
       | x1 = tyvars; x2 = idd; "="; x3 = ctyp; "=="; x4 = dbrhs ->
           let x4 =
             List.map
-              (fun (loc, c, tl, _) -> (loc, <:vala< c>>, <:vala< tl >>)) x4
+              (fun (loc, c, tl, _) -> (loc, <:vala< c>>, <:vala< tl >>, None))
+              x4
           in
-          {MLast.tdNam = (loc, <:vala< uncap x2 >>);
+          {MLast.tdNam = <:vala< (loc, <:vala< uncap x2 >>) >>;
            MLast.tdPrm = <:vala< x1 >>; MLast.tdPrv = <:vala< False >>;
            MLast.tdDef = <:ctyp< $x3$ == [ $list:x4$ ] >>;
            MLast.tdCon = <:vala< [] >>} ] ]
   ;
   tyvars:
-    [ [ "'"; x1 = V LIDENT -> [(x1, (False, False))]
+    [ [ "'"; x1 = V tyvar -> [(x1, None)]
       | "("; x1 = tyvar_pc; ")" -> x1
       | -> [] ] ]
   ;
@@ -669,7 +678,7 @@ EXTEND
           List.fold_right (fun td tdl -> td @ tdl) x1 [] ] ]
   ;
   dbrhs:
-    [ [ x1 = LIST1 constr SEP "|"  -> x1
+    [ [ x1 = LIST1 constr SEP "|" -> x1
       | "datatype"; x1 = tycon -> not_impl loc "dbrhs 2" ] ]
   ;
   constr:
@@ -751,12 +760,12 @@ EXTEND
   ;
   tyspec:
     [ [ x1 = tyvars; x2 = idd ->
-          {MLast.tdNam = (loc, <:vala< uncap x2 >>);
+          {MLast.tdNam = <:vala< (loc, <:vala< uncap x2 >>) >>;
            MLast.tdPrm = <:vala< x1 >>; MLast.tdPrv = <:vala< False >>;
            MLast.tdDef = <:ctyp< '$choose_tvar x1$ >>;
            MLast.tdCon = <:vala< [] >>}
       | x1 = tyvars; x2 = idd; "="; x3 = ctyp ->
-          {MLast.tdNam = (loc, <:vala< uncap x2 >>);
+          {MLast.tdNam = <:vala< (loc, <:vala< uncap x2 >>) >>;
            MLast.tdPrm = <:vala< x1 >>; MLast.tdPrv = <:vala< False >>;
            MLast.tdDef = x3; MLast.tdCon = <:vala< [] >>} ] ]
   ;
@@ -962,7 +971,7 @@ EXTEND
           not_impl loc "fctb 2" ] ]
   ;
   interdec:
-    [ [ x = LIST1 [ s = str_item; OPT ";" -> (s, loc) ] -> (x, False)
+    [ [ x = LIST1 [ s = str_item; OPT ";" -> (s, loc) ] -> (x, Some loc)
       | x = expr; OPT ";" -> not_impl loc "interdec 2" ] ]
   ;
 END;
