@@ -1,7 +1,14 @@
-(* camlp5r q_MLast.cmo ./pa_extfun.cmo ./pa_extprint.cmo ./pa_pprintf.cmo *)
-(* $Id: pr_extend.ml,v 1.62 2010/02/19 09:06:35 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pr_extend.ml,v 6.2 2010-09-19 09:56:36 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#directory ".";
+#load "q_MLast.cmo";
+#load "pa_extfun.cmo";
+#load "pa_extprint.cmo";
+#load "pa_pprintf.cmo";
+#load "pa_macro.cmo";
+
 (* heuristic to rebuild the EXTEND statement from the AST *)
 
 open Pretty;
@@ -22,6 +29,12 @@ value not_impl name pc x =
   pprintf pc "\"pr_extend, not impl: %s; %s\"" name (String.escaped desc)
 ;
 
+IFDEF OCAML_VERSION <= OCAML_1_07 THEN
+  value with_ind = Pprintf.with_ind;
+  value with_bef = Pprintf.with_bef;
+  value with_aft = Pprintf.with_aft;
+END;
+
 value bar_before elem pc x = pprintf pc "| %p" elem x;
 value semi_after elem pc x = pprintf pc "%p;" elem x;
 
@@ -31,9 +44,9 @@ type symbol =
   [ Snterm of MLast.expr
   | Snterml of MLast.expr and string
   | Slist0 of symbol
-  | Slist0sep of symbol and symbol
+  | Slist0sep of symbol and symbol and bool
   | Slist1 of symbol
-  | Slist1sep of symbol and symbol
+  | Slist1sep of symbol and symbol and bool
   | Sopt of symbol
   | Sflag of symbol
   | Sself
@@ -131,6 +144,13 @@ value unstring =
   | _ -> assert False ]
 ;
 
+value unbool =
+  fun
+  [ <:expr< True >> -> True
+  | <:expr< False >> -> False
+  | _ -> assert False ]
+;
+
 value untoken =
   fun
   [ <:expr< ($str:x$, $str:y$) >> -> Left (x, y)
@@ -169,15 +189,15 @@ and unsymbol =
   | <:expr< Gramext.Snterml ($uid:_$.Entry.obj ($e$ : $_$), $str:s$) >> ->
       Snterml e s
   | <:expr< Gramext.Slist0 $e$ >> -> Slist0 (unsymbol e)
-  | <:expr< Gramext.Slist0sep $e1$ $e2$ >> ->
-      Slist0sep (unsymbol e1) (unsymbol e2)
-  | <:expr< Gramext.Slist0sep ($e1$, $e2$) >> ->
-      Slist0sep (unsymbol e1) (unsymbol e2)
+  | <:expr< Gramext.Slist0sep $e1$ $e2$ $b$ >> ->
+      Slist0sep (unsymbol e1) (unsymbol e2) (unbool b)
+  | <:expr< Gramext.Slist0sep ($e1$, $e2$, $b$) >> ->
+      Slist0sep (unsymbol e1) (unsymbol e2) (unbool b)
   | <:expr< Gramext.Slist1 $e$ >> -> Slist1 (unsymbol e)
-  | <:expr< Gramext.Slist1sep $e1$ $e2$ >> ->
-      Slist1sep (unsymbol e1) (unsymbol e2)
-  | <:expr< Gramext.Slist1sep ($e1$, $e2$) >> ->
-      Slist1sep (unsymbol e1) (unsymbol e2)
+  | <:expr< Gramext.Slist1sep $e1$ $e2$ $b$ >> ->
+      Slist1sep (unsymbol e1) (unsymbol e2) (unbool b)
+  | <:expr< Gramext.Slist1sep ($e1$, $e2$, $b$) >> ->
+      Slist1sep (unsymbol e1) (unsymbol e2) (unbool b)
   | <:expr< Gramext.Sopt $e$ >> -> Sopt (unsymbol e)
   | <:expr< Gramext.Sflag $e$ >> -> Sflag (unsymbol e)
   | <:expr< Gramext.Sself >> -> Sself
@@ -352,12 +372,14 @@ and symbol pc sy =
   | Snterml e s -> pprintf pc "%p LEVEL \"%s\"" expr e s
   | Slist0 sy ->
       pprintf pc "LIST0@;%p" simple_symbol sy
-  | Slist0sep sy sep ->
-      pprintf pc "LIST0@;%p@ @[SEP@;%p@]" simple_symbol sy simple_symbol sep
+  | Slist0sep sy sep b ->
+      pprintf pc "LIST0@;%p@ @[SEP@;%p%s@]" simple_symbol sy simple_symbol sep
+        (if b then " OPT_SEP" else "")
   | Slist1 sy ->
       pprintf pc "LIST1@;%p" simple_symbol sy
-  | Slist1sep sy sep ->
-      pprintf pc "LIST1@;%p@ @[SEP@;%p@]" simple_symbol sy simple_symbol sep
+  | Slist1sep sy sep b ->
+      pprintf pc "LIST1@;%p@ @[SEP@;%p%s@]" simple_symbol sy simple_symbol sep
+        (if b then " OPT_SEP" else "")
   | Sopt sy ->
       pprintf pc "OPT@;%p" simple_symbol sy
   | Sflag sy ->
@@ -387,7 +409,7 @@ and simple_symbol pc sy =
                pprintf pc "[ %p ]"
                  (vlist2 (rule False) (bar_before (rule False))) rl) ]
   | Stoken (Left ("", _) | Left (_, "")) -> symbol pc sy
-  | Snterml _ _ | Slist0 _ | Slist0sep _ _ | Slist1 _ | Slist1sep _ _ |
+  | Snterml _ _ | Slist0 _ | Slist0sep _ _ _ | Slist1 _ | Slist1sep _ _ _ |
     Sflag _ | Sopt _ ->
       pprintf pc "@[<1>(%p)@]" symbol sy
   | sy -> not_impl "simple_symbol" pc sy ]
@@ -395,10 +417,12 @@ and s_symbol pc =
   fun
   [ Slist0 sy -> pprintf pc "SLIST0@;%p" simple_symbol sy
   | Slist1 sy -> pprintf pc "SLIST1@;%p" simple_symbol sy
-  | Slist0sep sy sep ->
-      pprintf pc "SLIST0@;%p@ @[SEP@;%p@]" simple_symbol sy simple_symbol sep
-  | Slist1sep sy sep ->
-      pprintf pc "SLIST1@;%p@ @[SEP@;%p@]" simple_symbol sy simple_symbol sep
+  | Slist0sep sy sep b ->
+      pprintf pc "SLIST0@;%p@ @[SEP@;%p%s@]" simple_symbol sy simple_symbol
+        sep (if b then " OPT_SEP" else "")
+  | Slist1sep sy sep b ->
+      pprintf pc "SLIST1@;%p@ @[SEP@;%p%s@]" simple_symbol sy simple_symbol
+        sep (if b then " OPT_SEP" else "")
   | Sopt s ->
       let sy =
         match s with
@@ -423,12 +447,12 @@ and check_slist rl =
     match rl with
     [ [([(Some <:patt< a >>, Snterm <:expr< a_list >>)], Some <:expr< a >>);
        ([(Some <:patt< a >>,
-          ((Slist0 _ | Slist1 _ | Slist0sep _ _ | Slist1sep _ _) as s))],
+          ((Slist0 _ | Slist1 _ | Slist0sep _ _ _ | Slist1sep _ _ _) as s))],
           Some <:expr< Qast.List a >>)] ->
         Some s
     | [([(Some <:patt< a >>, Snterm <:expr< a_list2 >>)], Some <:expr< a >>);
        ([(Some <:patt< a >>,
-          ((Slist0 _ | Slist1 _ | Slist0sep _ _ | Slist1sep _ _) as s))],
+          ((Slist0 _ | Slist1 _ | Slist0sep _ _ _ | Slist1sep _ _ _) as s))],
           Some <:expr< Qast.VaVal (Qast.List a) >>)] ->
         Some (Svala [] None s)
 
@@ -462,12 +486,12 @@ and check_slist rl =
               let ls =
                 match (s, ls) with
                 [ (Sflag _, ["flag"; "opt"]) -> []
-                | ((Slist0 _ | Slist0sep _ _), ["list"]) -> []
-                | ((Slist1 _ | Slist1sep _ _), ["list"]) -> []
+                | ((Slist0 _ | Slist0sep _ _ _), ["list"]) -> []
+                | ((Slist1 _ | Slist1sep _ _ _), ["list"]) -> []
                 | (Sopt _, ["opt"]) -> []
                 | (Stoken (Left (s, "")), _) ->
                     if ls = anti_of_tok s then [] else ls
-                | _ -> ls ]                   
+                | _ -> ls ]
               in
               Some (Svala ls None s)
           | [([(Some <:patt< a >>, s)], Some <:expr< Qast.VaVal $_$ >>);
