@@ -1,7 +1,14 @@
-(* camlp5r q_MLast.cmo ./pa_extfun.cmo ./pa_extprint.cmo ./pa_pprintf.cmo *)
-(* $Id: pr_extfun.ml,v 1.21 2010/02/19 09:06:35 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pr_extfun.ml,v 6.1 2010-09-15 16:00:22 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#directory ".";
+#load "q_MLast.cmo";
+#load "pa_extfun.cmo";
+#load "pa_extprint.cmo";
+#load "pa_pprintf.cmo";
+#load "pa_macro.cmo";
+
 (* heuristic to rebuild the extfun statement from the AST *)
 
 open Pretty;
@@ -43,6 +50,10 @@ value rec un_extfun rpel =
   | _ -> raise Not_found ]
 ;
 
+IFDEF OCAML_VERSION <= OCAML_1_07 THEN
+  value with_ind = Pprintf.with_ind;
+END;
+
 value bar_before elem pc x = pprintf pc "| %p" elem x;
 
 value comm_bef pc loc =
