@@ -1,7 +1,13 @@
-(* camlp5r q_MLast.cmo -I . pa_extfun.cmo pa_extprint.cmo pa_pprintf.cmo *)
-(* $Id: pr_extprint.ml,v 1.9 2010/02/19 09:06:35 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pr_extprint.ml,v 6.1 2010-09-15 16:00:22 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#directory ".";
+#load "q_MLast.cmo";
+#load "pa_extfun.cmo";
+#load "pa_extprint.cmo";
+#load "pa_pprintf.cmo";
+
 (* heuristic to rebuild the EXTEND_PRINTER statement from the AST *)
 
 open Pcaml;
@@ -33,7 +39,7 @@ value unrules =
         in
         match (p, wo, e, rules) with
         [ (<:patt< z >>, None, <:expr< next pc z >>, <:expr< [] >>) -> r
-        | _ -> loop [(p, wo, e) ::  r] rules ]
+        | _ -> loop [(p, wo, e) :: r] rules ]
     | <:expr< [] >> -> r
     | _ -> raise Not_found ]
 ;
