@@ -1,10 +1,18 @@
-(* camlp5r pa_macro.cmo q_MLast.cmo ./pa_extprint.cmo ./pa_extfun.cmo ./pa_pprintf.cmo *)
-(* $Id: pr_scheme.ml 1832 2007-12-29 04:26:28Z deraugla $ *)
-(* Copyright (c) INRIA 2007-2008 *)
+(* camlp5r *)
+(* $Id: pr_scheme.ml,v 6.6 2010-11-13 07:35:45 deraugla Exp $ *)
+(* Copyright (c) INRIA 2007-2010 *)
+
+#directory ".";
+#load "pa_macro.cmo";
+#load "q_MLast.cmo";
+#load "pa_extprint.cmo";
+#load "pa_extfun.cmo";
+#load "pa_pprintf.cmo";
 
 open Pretty;
 open Pcaml;
 open Prtools;
+open Versdep;
 
 do {
   Eprinter.clear pr_expr;
@@ -42,6 +50,14 @@ value to_be_renamed = ["cond"; "sum"];
 
 value rename_id s = if List.mem s to_be_renamed then s ^ "#" else s;
 
+IFDEF OCAML_VERSION <= OCAML_1_07 THEN
+  value with_ind_bef = Pprintf.with_ind_bef;
+  value with_ind_bef_aft = Pprintf.with_ind_bef_aft;
+  value with_bef = Pprintf.with_bef;
+  value with_bef_aft = Pprintf.with_bef_aft;
+  value with_aft = Pprintf.with_aft;
+END;
+
 value rec longident pc sl =
   match sl with
   [ [] -> pprintf pc ""
@@ -77,11 +93,10 @@ value rec is_irrefut_patt =
       List.for_all (fun (_, p) -> is_irrefut_patt p) fpl
   | <:patt< ($p$ : $_$) >> -> is_irrefut_patt p
   | <:patt< ($list:pl$) >> -> List.for_all is_irrefut_patt pl
-  | <:patt< ?$_$: ($_$ = $_$) >> -> True
-  | <:patt< ?$_$: ($_$) >> -> True
-  | <:patt< ?$_$ >> -> True
-  | <:patt< ~$_$ >> -> True
-  | <:patt< ~$_$: $_$ >> -> True
+  | <:patt< ?{$_$ = ?{$_$ = $_$}} >> -> True
+  | <:patt< ?{$_$ = $_$} >> -> True
+  | <:patt< ?{$_$} >> -> True
+  | <:patt< ~{$_$} >> -> True
   | _ -> False ]
 ;
 
@@ -96,7 +111,7 @@ pr_expr_fun_args.val :=
 
 value has_cons_with_params vdl =
   List.exists
-    (fun (_, _, tl) ->
+    (fun (_, _, tl, rto) ->
        match tl with
        [ <:vala< [] >> -> False
        | _ -> True ])
@@ -123,15 +138,22 @@ value braceless pc b =
    aft = sprintf ">}%s" pc.aft}
 ;
 
-value type_param pc (s, (pl, mi)) =
-  pprintf pc "%s'%s" (if pl then "+" else if mi then "-" else "")
-    (Pcaml.unvala s)
+value type_param pc (tv, vari) =
+  let tv = Pcaml.unvala tv in
+  pprintf pc "%s%s"
+    (match vari with
+     [ Some True -> "+"
+     | Some False -> "-"
+     | None -> "" ])
+    (match tv with
+     [ Some v -> "'" ^ v
+     | None -> "_" ])
 ;
 
 value type_var pc v = pprintf pc "'%s" v;
 
 value type_decl b pc td =
-  let n = rename_id (Pcaml.unvala (snd td.MLast.tdNam)) in
+  let n = rename_id (Pcaml.unvala (snd (Pcaml.unvala td.MLast.tdNam))) in
   pprintf pc "@[<1>(%s%p@ %p)@]" b
     (fun pc ->
        fun
@@ -239,7 +261,7 @@ value match_assoc pc (p, we, e) =
   plistf 0 (paren pc "") list
 ;
 
-value constr_decl pc (_, c, tl) =
+value constr_decl pc (_, c, tl, rto) =
   let c = Pcaml.unvala c in
   let tl = Pcaml.unvala tl in
   match tl with
@@ -659,7 +681,7 @@ EXTEND_PRINTER
       | <:expr< let $flag:rf$ $list:pel$ in $e$ >> ->
           let b = if rf then "letrec" else "let" in
           let_binding_list pc (b, pel, e)
-      | <:expr< let module $s$ = $me$ in $e$  >> ->
+      | <:expr< let module $uid:s$ = $me$ in $e$ >> ->
           plistbf 0 (paren pc "letmodule")
             [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "");
              (fun pc -> module_expr pc me, "");
@@ -699,7 +721,7 @@ EXTEND_PRINTER
                     aft = sprintf ")%s" pc.aft}
                    e3
                in
-               sprintf "%s\n%s\n%s" s1 s2 s3) 
+               sprintf "%s\n%s\n%s" s1 s2 s3)
      | <:expr< do { $list:el$ } >> ->
           horiz_vertic
             (fun () ->
@@ -776,7 +798,7 @@ EXTEND_PRINTER
                 (fun () -> hlistl curr dot_expr pc (el @ [x]))
                 (fun () ->
                    let el =
-                     List.rev_map (fun e -> (e, "")) [x :: List.rev el]
+                     list_rev_map (fun e -> (e, "")) [x :: List.rev el]
                    in
                    plistl curr dot_expr 0 pc el) ]
       | <:expr< [| $list:el$ |] >> ->
@@ -823,17 +845,17 @@ EXTEND_PRINTER
             [(fun pc -> sprintf "%ssend%s" pc.bef pc.aft, "");
              (fun pc -> curr pc e, "");
              (fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "")]
-      | <:expr< ?$s$ >> ->
+      | <:expr< ?{$lid:s$} >> ->
           plistbf 0 (paren pc "?")
             [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "")]
-      | <:expr< ?$s$: $e$ >> ->
+      | <:expr< ?{$lid:s$ = $e$} >> ->
           plistbf 0 (paren pc "?")
             [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "");
              (fun pc -> curr pc e, "")]
-      | <:expr< ~$s$ >> ->
+      | <:expr< ~{$lid:s$} >> ->
           plistbf 0 (paren pc "~")
             [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "")]
-      | <:expr< ~$s$: $e$ >> ->
+      | <:expr< ~{$lid:s$ = $e$} >> ->
           plistbf 0 (paren pc "~")
             [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "");
              (fun pc -> curr pc e, "")]
@@ -948,7 +970,7 @@ EXTEND_PRINTER
                 (fun () -> hlistl curr dot_patt pc (pl @ [x]))
                 (fun () ->
                    let pl =
-                     List.rev_map (fun p -> (p, "")) [x :: List.rev pl]
+                     list_rev_map (fun p -> (p, "")) [x :: List.rev pl]
                    in
                    plistl curr dot_patt 0 pc pl) ]
       | <:patt< [| $list:pl$ |] >> ->
@@ -981,26 +1003,24 @@ EXTEND_PRINTER
           in
           plist record_binding 0 (brace pc "")
             (List.map (fun fp -> (fp, "")) fpl)
-      | <:patt< ?$s$ >> ->
+      | <:patt< ?{$p$} >> ->
           plistbf 0 (paren pc "?")
-            [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "")]
-      | <:patt< ?$s$: ($p$) >> ->
+            [(fun pc -> pprintf pc "%p" patt p, "")]
+      | <:patt< ?{$lid:s$ = $lid:p$} >> ->
           plistbf 0 (paren pc "?")
-            [(fun pc -> plistbf 0 (paren pc s) [(fun pc -> curr pc p, "")],
-              "")]
-      | <:patt< ? ($lid:s$ = $e$) >> ->
+            [(fun pc -> plistbf 0 (paren pc s) [(fun pc -> p, "")], "")]
+      | <:patt< ?{$lid:s$ = $e$} >> ->
           plistbf 0 (paren pc "?")
             [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "");
              (fun pc -> expr pc e, "")]
-      | <:patt< ?$s$: ($p$ = $e$) >> ->
+      | <:patt< ?{$lid:s$ = ?{$lid:p$ = $e$}} >> ->
           plistbf 0 (paren pc "?")
-            [(fun pc -> plistbf 0 (paren pc s) [(fun pc -> curr pc p, "")],
-              "");
+            [(fun pc -> plistbf 0 (paren pc s) [(fun pc -> p, "")], "");
              (fun pc -> expr pc e, "")]
-      | <:patt< ~$s$ >> ->
+      | <:patt< ~{$lid:s$} >> ->
           plistbf 0 (paren pc "~")
             [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "")]
-      | <:patt< ~$s$: $p$ >> ->
+      | <:patt< ~{$lid:s$ = $p$} >> ->
           plistbf 0 (paren pc "~")
             [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "");
              (fun pc -> curr pc p, "")]
@@ -1223,7 +1243,7 @@ EXTEND_PRINTER
             [(fun pc -> class_expr pc ce, "") :: list]
       | <:class_str_item< initializer $e$ >> ->
           plistb expr 0 (paren pc "initializer") [(e, "")]
-      | <:class_str_item< method $flag:pf$ $s$ = $e$ >> ->
+      | <:class_str_item< method $priv:pf$ $s$ = $e$ >> ->
           let (pl, e) = expr_fun_args e in
           let list =
             let list =
@@ -1380,8 +1400,8 @@ EXTEND_PRINTER
              List.map (fun e -> (fun pc -> expr pc e, "")) el]
       | <:class_expr< $list:sl$ >> ->
           longident pc sl
-      | <:class_expr< $list:sl$ [ $list:ctcl$ ] >> ->
-          not_impl  "CeCon" pc sl
+      | <:class_expr< [ $list:ctcl$ ] $list:sl$ >> ->
+          not_impl "CeCon" pc sl
       | x ->
           not_impl "class_expr" pc x ] ]
   ;
@@ -1442,23 +1462,33 @@ module Buff =
   end
 ;
 
-value apply_printer f ast = do {
-  if Pcaml.input_file.val = "-" then sep.val := Some "\n"
-  else do {
-    let ic = open_in_bin Pcaml.input_file.val in
-    let src =
-      loop 0 where rec loop len =
-        match try Some (input_char ic) with [ End_of_file -> None ] with
-        [ Some c -> loop (Buff.store len c)
-        | None -> Buff.get len ]
-    in
-    Prtools.source.val := src;
-    close_in ic
-  };
+value first_loc_of_ast =
+  fun
+  [ [(_, loc) :: _] -> loc
+  | [] -> Ploc.dummy ]
+;
+
+value apply_printer f (ast, eoi_loc) = do {
+  let loc = first_loc_of_ast ast in
+  let fname = Ploc.file_name loc in
+  let src =
+    if fname = "-" then do { sep.val := Some "\n"; "" }
+    else do {
+      let ic = open_in_bin fname in
+      let src =
+        loop 0 where rec loop len =
+          match try Some (input_char ic) with [ End_of_file -> None ] with
+          [ Some c -> loop (Buff.store len c)
+          | None -> Buff.get len ]
+      in
+      close_in ic;
+      src
+    }
+  in
   let oc =
     match Pcaml.output_file.val with
     [ Some f -> open_out_bin f
-    | None -> do { set_binary_mode_out stdout True; stdout } ]
+    | None -> do { pervasives_set_binary_mode_out stdout True; stdout } ]
   in
   let cleanup () =
     match Pcaml.output_file.val with
@@ -1471,14 +1501,14 @@ value apply_printer f ast = do {
         (fun (first, last_pos) (si, loc) -> do {
            let bp = Ploc.first_pos loc in
            let ep = Ploc.last_pos loc in
-           copy_source Prtools.source.val oc first last_pos bp;
+           copy_source src oc first last_pos bp;
            flush oc;
            output_string oc (f {ind = 0; bef = ""; aft = ""; dang = ""} si);
            (False, ep)
          })
         (True, 0) ast
     in
-    copy_to_end Prtools.source.val oc first last_pos;
+    copy_to_end src oc first last_pos;
     flush oc
   }
   with exn -> do {
