@@ -1,7 +1,9 @@
 (* camlp5r *)
-(* $Id: eprinter.ml,v 1.16 2010/02/19 09:06:37 deraugla Exp $ *)
+(* $Id: eprinter.ml,v 6.1 2010-09-15 16:00:23 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_macro.cmo";
+
 type t 'a =
   { pr_name : string;
     pr_fun : mutable string -> pr_fun 'a;
@@ -33,6 +35,12 @@ value add_lev (lab, extf) levs =
   [lev :: levs]
 ;
 
+IFDEF OCAML_VERSION <= OCAML_1_07 OR COMPATIBLE_WITH_OLD_OCAML THEN
+  value with_pr_rules lev pr_rules =
+    {pr_label = lev.pr_label; pr_rules = pr_rules}
+  ;
+END;
+
 value extend pr pos levs =
   match pos with
   [ None ->
@@ -78,7 +86,8 @@ value pr_fun name pr lab =
     [ [] ->
         fun pc z ->
           failwith
-            (Printf.sprintf "cannot print %s%s; probably a missing case"
+            (Printf.sprintf
+               "cannot print %s%s; a missing case in camlp5; please report"
                name (if lab = "" then "" else " \"" ^ lab ^ "\""))
     | [lev :: levl] ->
         if lab = "" || app || lev.pr_label = lab then
