@@ -1,7 +1,11 @@
-(* camlp5r pa_lexer.cmo *)
-(* $Id: plexer.ml,v 1.108 2010/02/19 09:06:37 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: plexer.ml,v 6.12 2011-02-17 10:14:09 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_lexer.cmo";
+
+open Versdep;
+
 value no_quotations = ref False;
 value error_on_unknown_keywords = ref False;
 
@@ -56,6 +60,16 @@ value misc_punct buf strm =
     match strm with parser []
 ;
 
+value utf8_equiv ctx bp buf strm =
+  if utf8_lexing.val then
+    match strm with lexer
+    [ "→" -> keyword_or_error ctx (bp, $pos) "->"
+    | "≤" -> keyword_or_error ctx (bp, $pos) "<="
+    | "≥" -> keyword_or_error ctx (bp, $pos) ">=" ]
+  else
+    match strm with parser []
+;
+
 value rec ident =
   lexer
   [ [ 'A'-'Z' | 'a'-'z' | '0'-'9' | '_' | ''' | misc_letter ] ident! | ]
@@ -123,11 +137,15 @@ value number =
   | decimal_digits_under end_integer! ]
 ;
 
-value char_aux = lexer [ "'"/ | _ [ "'"/ | _ [ "'"/ | ] ] ];
+value char_after_bslash =
+  lexer
+  [ "'"/
+  | _ [ "'"/ | _ [ "'"/ | ] ] ]
+;
 
 value char ctx bp =
   lexer
-  [ "\\" _ char_aux!
+  [ "\\" _ char_after_bslash!
   | "\\" -> err ctx (bp, $pos) "char not terminated"
   | ?= [ _ '''] _! "'"/ ]
 ;
@@ -152,8 +170,9 @@ value comment ctx bp =
     | "(*" comment! comment!
     | "(" comment!
     | "\"" (string ctx bp)! [ -> $add "\"" ] comment!
-    | "'" (char ctx bp) comment!
-    | "'" comment!
+    | "'*)"
+    | "'*" comment!
+    | "'" (any ctx) comment!
     | (any ctx) comment!
     | -> err ctx (bp, $pos) "comment not terminated" ]
 ;
@@ -172,6 +191,8 @@ value rec quotation ctx bp =
   | -> err ctx (bp, $pos) "quotation not terminated" ]
 ;
 
+value less_expected = "character '<' expected";
+
 value less ctx bp buf strm =
   if no_quotations.val then
     match strm with lexer
@@ -179,8 +200,9 @@ value less ctx bp buf strm =
   else
     match strm with lexer
     [ "<"/ (quotation ctx bp) -> ("QUOTATION", ":" ^ $buf)
-    | ":"/ ident! [ -> $add ":" ]! "<"/ ? "character '<' expected"
-      (quotation ctx bp) ->
+    | ":"/ ident! "<"/ ? less_expected [ -> $add ":" ]! (quotation ctx bp) ->
+        ("QUOTATION", $buf)
+    | ":"/ ident! ":<"/ ? less_expected [ -> $add "@" ]! (quotation ctx bp) ->
         ("QUOTATION", $buf)
     | [ -> $add "<" ] ident2! -> keyword_or_error ctx (bp, $pos) $buf ]
 ;
@@ -196,7 +218,7 @@ value rec antiquot_rest ctx bp =
 value rec antiquot ctx bp =
   lexer
   [ "$"/ -> ":" ^ $buf
-  | [ 'a'-'z' | 'A'-'Z' | '0'-'9' | '_' ] (antiquot ctx bp)!
+  | [ 'a'-'z' | 'A'-'Z' | '0'-'9' | '!' | '_' ] (antiquot ctx bp)!
   | ":" (antiquot_rest ctx bp)! -> $buf
   | "\\"/ (any ctx) (antiquot_rest ctx bp)! -> ":" ^ $buf
   | (any ctx) (antiquot_rest ctx bp)! -> ":" ^ $buf
@@ -208,7 +230,7 @@ value antiloc bp ep s = Printf.sprintf "%d,%d:%s" bp ep s;
 value rec antiquot_loc ctx bp =
   lexer
   [ "$"/ -> antiloc bp $pos (":" ^ $buf)
-  | [ 'a'-'z' | 'A'-'Z' | '0'-'9' | '_' ] (antiquot_loc ctx bp)!
+  | [ 'a'-'z' | 'A'-'Z' | '0'-'9' | '!' | '_' ] (antiquot_loc ctx bp)!
   | ":" (antiquot_rest ctx bp)! -> antiloc bp $pos $buf
   | "\\"/ (any ctx) (antiquot_rest ctx bp)! -> antiloc bp $pos (":" ^ $buf)
   | (any ctx) (antiquot_rest ctx bp)! -> antiloc bp $pos (":" ^ $buf)
@@ -216,7 +238,7 @@ value rec antiquot_loc ctx bp =
 ;
 
 value dollar ctx bp buf strm =
-  if ctx.dollar_for_antiquotation then
+  if not no_quotations.val && ctx.dollar_for_antiquotation then
     ("ANTIQUOT", antiquot ctx bp buf strm)
   else if force_antiquot_loc.val then
     ("ANTIQUOT_LOC", antiquot_loc ctx bp buf strm)
@@ -339,6 +361,7 @@ value next_token_after_spaces ctx bp =
   | "0" [ 'x' | 'X' ] (digits hexa)!
   | "0" [ 'b' | 'B' ] (digits binary)!
   | "0" number!
+  | "'"/ ?= [ '\\' 'a'-'z' 'a'-'z' ] -> keyword_or_error ctx (bp, $pos) "'"
   | "'"/ (char ctx bp) -> ("CHAR", $buf)
   | "'" -> keyword_or_error ctx (bp, $pos) "'"
   | "\""/ (string ctx bp)! -> ("STRING", $buf)
@@ -380,11 +403,14 @@ value next_token_after_spaces ctx bp =
       keyword_or_error ctx (bp, $pos) id
   | ";;" -> keyword_or_error ctx (bp, $pos) ";;"
   | ";" -> keyword_or_error ctx (bp, $pos) ";"
+  | (utf8_equiv ctx bp)
   | misc_punct ident2! -> keyword_or_error ctx (bp, $pos) $buf
   | "\\"/ ident3! -> ("LIDENT", $buf)
   | (any ctx) -> keyword_or_error ctx (bp, $pos) $buf ]
 ;
 
+value get_comment buf strm = $buf;
+
 value rec next_token ctx buf =
   parser bp
   [ [: `('\n' | '\r' as c); s :] ep -> do {
@@ -399,6 +425,7 @@ value rec next_token ctx buf =
       next_token ctx ($add c) s
     }
   | [: `'#' when bp = Plexing.bol_pos.val.val; s :] ->
+      let comm = get_comment buf () in
       if linedir 1 s then do {
         let buf = any_to_nl ($add '#') s in
         incr Plexing.line_nb.val;
@@ -408,7 +435,7 @@ value rec next_token ctx buf =
         next_token ctx buf s
       }
       else
-        let loc = ctx.make_lined_loc (bp, bp + 1) $buf in
+        let loc = ctx.make_lined_loc (bp, bp + 1) comm in
         (keyword_or_error ctx (bp, bp + 1) "#", loc)
   | [: `'(';
        a =
@@ -421,11 +448,12 @@ value rec next_token ctx buf =
          | [: :] ep ->
              let loc = ctx.make_lined_loc (bp, ep) $buf in
              (keyword_or_error ctx (bp, ep) "(", loc) ] ! :] -> a
-  | [: tok = next_token_after_spaces ctx bp $empty :] ep ->
-      let loc = ctx.make_lined_loc (bp, max (bp + 1) ep) $buf in
+  | [: comm = get_comment buf;
+       tok = next_token_after_spaces ctx bp $empty :] ep ->
+      let loc = ctx.make_lined_loc (bp, max (bp + 1) ep) comm in
       (tok, loc)
-  | [: _ = Stream.empty :] ->
-      let loc = ctx.make_lined_loc (bp, bp + 1) $buf in
+  | [: comm = get_comment buf; _ = Stream.empty :] ->
+      let loc = ctx.make_lined_loc (bp, bp + 1) comm in
       (("EOI", ""), loc) ]
 ;
 
@@ -435,7 +463,7 @@ value next_token_fun ctx glexr (cstrm, s_line_nb, s_bol_pos) =
     [ Some (line_nb, bol_pos) -> do {
         s_line_nb.val := line_nb;
         s_bol_pos.val := bol_pos;
-        Plexing.restore_lexing_info.val := None
+        Plexing.restore_lexing_info.val := None;
       }
     | None -> () ];
     Plexing.line_nb.val := s_line_nb;
@@ -478,7 +506,7 @@ value func kwd_table glexr =
        bol_pos.val := Plexing.bol_pos.val.val;
      };
      make_lined_loc loc comm =
-       Ploc.make line_nb.val bol_pos.val loc}
+       Ploc.make_loc Plexing.input_file.val line_nb.val bol_pos.val loc comm}
   in
   Plexing.lexer_func_of_parser (next_token_fun ctx glexr)
 ;
@@ -554,9 +582,9 @@ value error_ident_and_keyword p_con p_prm =
 value using_token kwd_table ident_table (p_con, p_prm) =
   match p_con with
   [ "" ->
-      if not (Hashtbl.mem kwd_table p_prm) then
+      if not (hashtbl_mem kwd_table p_prm) then
         if check_keyword p_prm then
-          if Hashtbl.mem ident_table p_prm then
+          if hashtbl_mem ident_table p_prm then
             error_ident_and_keyword (Hashtbl.find ident_table p_prm) p_prm
           else Hashtbl.add kwd_table p_prm p_prm
         else error_no_respect_rules p_con p_prm
@@ -567,7 +595,7 @@ value using_token kwd_table ident_table (p_con, p_prm) =
         match p_prm.[0] with
         [ 'A'..'Z' -> error_no_respect_rules p_con p_prm
         | _ ->
-            if Hashtbl.mem kwd_table p_prm then
+            if hashtbl_mem kwd_table p_prm then
               error_ident_and_keyword p_con p_prm
             else Hashtbl.add ident_table p_prm p_con ]
   | "UIDENT" ->
@@ -576,7 +604,7 @@ value using_token kwd_table ident_table (p_con, p_prm) =
         match p_prm.[0] with
         [ 'a'..'z' -> error_no_respect_rules p_con p_prm
         | _ ->
-            if Hashtbl.mem kwd_table p_prm then
+            if hashtbl_mem kwd_table p_prm then
               error_ident_and_keyword p_con p_prm
             else Hashtbl.add ident_table p_prm p_con ]
   | "TILDEIDENT" | "TILDEIDENTCOLON" | "QUESTIONIDENT" |
