@@ -1,5 +1,5 @@
 (* camlp5r *)
-(* $Id: plexing.mli,v 1.5 2010/02/19 09:06:37 deraugla Exp $ *)
+(* $Id: plexing.mli,v 6.3 2010-09-29 09:45:05 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
 (** Lexing for Camlp5 grammars.
@@ -67,7 +67,7 @@ and lexer_func 'te = Stream.t char -> (Stream.t 'te * location_function)
       function (see below) for this tokens stream. *)
 
 and location_function = int -> Ploc.t;
-  (**>The type of a function giving the location of a token in the
+  (** The type of a function giving the location of a token in the
       source from the token number in the stream (starting from zero). *)
 
 value lexer_text : pattern -> string;
@@ -87,7 +87,8 @@ value default_match : pattern -> (string * string) -> string;
             Plexing.tok_using = (fun _ -> ());
             Plexing.tok_removing = (fun _ -> ());
             Plexing.tok_match = Plexing.default_match;
-            Plexing.tok_text = Plexing.lexer_text }
+            Plexing.tok_text = Plexing.lexer_text;
+            Plexing.tok_comm = None }
    ]}
    Note that a better [tok_using] function should check the used tokens
    and raise [Plexing.Error] for incorrect ones. The other functions
@@ -120,6 +121,7 @@ value eval_string : Ploc.t -> string -> string;
        return [s] *)
 
 value restore_lexing_info : ref (option (int * int));
+value input_file : ref string;
 value line_nb : ref (ref int);
 value bol_pos : ref (ref int);
    (** Special variables used to reinitialize line numbers and position
