@@ -1,5 +1,5 @@
 (* camlp5r *)
-(* $Id: stdpp.ml,v 1.20 2010/02/19 09:06:37 deraugla Exp $ *)
+(* $Id: stdpp.ml,v 6.4 2010-11-16 16:56:40 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
 type location = Ploc.t;
@@ -7,7 +7,7 @@ type location = Ploc.t;
 exception Exc_located = Ploc.Exc;
 value raise_with_loc = Ploc.raise;
 
-value make_lined_loc = Ploc.make;
+value make_lined_loc lin bol (bp, ep) = Ploc.make_loc "" lin bol (bp, ep) "";
 value make_loc = Ploc.make_unlined;
 value dummy_loc = Ploc.dummy;
 
