@@ -1,10 +1,10 @@
 (* camlp5r *)
-(* $Id: token.ml,v 1.21 2010/02/19 09:06:37 deraugla Exp $ *)
+(* $Id: token.ml,v 6.1 2010-09-15 16:00:23 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
 type pattern = Plexing.pattern;
 
-exception Error = Plexing.Error;
+exception Error of string;
 
 type location = Ploc.t;
 type location_function = int -> location;
