@@ -1,26 +1,19 @@
 (* camlp5r *)
-(* $Id: argl.ml,v 1.2 2007/07/11 12:01:39 deraugla Exp $ *)
+(* $Id: argl.ml,v 6.1 2010-09-15 16:00:24 deraugla Exp $ *)
 
 open Printf;
+open Versdep;
 
 value action_arg s sl =
   fun
-  [ Arg.Unit f -> if s = "" then do { f (); Some sl } else None
-  | Arg.Set r -> if s = "" then do { r.val := True; Some sl } else None
+  [ Arg.Set r -> if s = "" then do { r.val := True; Some sl } else None
   | Arg.Clear r -> if s = "" then do { r.val := False; Some sl } else None
-  | Arg.Rest f -> do { List.iter f [s :: sl]; Some [] }
   | Arg.String f ->
       if s = "" then
         match sl with
         [ [s :: sl] -> do { f s; Some sl }
         | [] -> None ]
       else do { f s; Some sl }
-  | Arg.Set_string r ->
-      if s = "" then
-        match sl with
-        [ [s :: sl] -> do { r.val := s; Some sl }
-        | [] -> None ]
-      else do { r.val := s; Some sl }
   | Arg.Int f ->
       if s = "" then
         match sl with
@@ -31,34 +24,59 @@ value action_arg s sl =
       else
         try do { f (int_of_string s); Some sl } with
         [ Failure "int_of_string" -> None ]
-  | Arg.Set_int r ->
-      if s = "" then
-        match sl with
-        [ [s :: sl] ->
-            try do { r.val := int_of_string s; Some sl } with
-            [ Failure "int_of_string" -> None ]
-        | [] -> None ]
-      else
-        try do { r.val := int_of_string s; Some sl } with
-        [ Failure "int_of_string" -> None ]
   | Arg.Float f ->
       if s = "" then
         match sl with
         [ [s :: sl] -> do { f (float_of_string s); Some sl }
         | [] -> None ]
       else do { f (float_of_string s); Some sl }
-  | Arg.Set_float r ->
-      if s = "" then
-        match sl with
-        [ [s :: sl] -> do { r.val := float_of_string s; Some sl }
-        | [] -> None ]
-      else do { r.val := float_of_string s; Some sl }
-  | Arg.Symbol syms f ->
-      match if s = "" then sl else [s :: sl] with
-      [ [s :: sl] when List.mem s syms -> do { f s; Some sl }
-      | _ -> None ]
-  | Arg.Tuple _ -> failwith "Arg.Tuple not implemented"
-  | Arg.Bool _ -> failwith "Arg.Bool not implemented" ]
+  | a ->
+      match arg_rest a with
+      [ Some f -> do { List.iter f [s :: sl]; Some [] }
+      | None ->
+      match arg_set_string a with
+      [ Some r ->
+          if s = "" then
+            match sl with
+            [ [s :: sl] -> do { r.val := s; Some sl }
+            | [] -> None ]
+          else do { r.val := s; Some sl }
+      | None ->
+      match arg_set_int a with
+      [ Some r ->
+          if s = "" then
+            match sl with
+            [ [s :: sl] ->
+                try do { r.val := int_of_string s; Some sl } with
+                [ Failure "int_of_string" -> None ]
+            | [] -> None ]
+          else
+            try do { r.val := int_of_string s; Some sl } with
+            [ Failure "int_of_string" -> None ]
+      | None ->
+      match arg_set_float a with
+      [ Some r ->
+          if s = "" then
+            match sl with
+            [ [s :: sl] -> do { r.val := float_of_string s; Some sl }
+            | [] -> None ]
+          else do { r.val := float_of_string s; Some sl }
+      | None ->
+      match arg_symbol a with
+      [ Some (syms, f) ->
+          match if s = "" then sl else [s :: sl] with
+          [ [s :: sl] when List.mem s syms -> do { f s; Some sl }
+          | _ -> None ]
+      | None ->
+      match arg_tuple a with
+      [ Some _ -> failwith "Arg.Tuple not implemented"
+      | None ->
+      match arg_bool a with
+      [ Some _ -> failwith "Arg.Bool not implemented"
+      | None ->
+      match a with
+      [ Arg.Unit f -> if s = "" then do { f (); Some sl } else None
+      | _ -> assert False ] ] ] ] ] ] ] ] ]
 ;
 
 value common_start s1 s2 =
@@ -127,12 +145,13 @@ value make_symlist l =
 value print_usage_list l =
   List.iter
     (fun (key, spec, doc) ->
-       match spec with
-       [ Arg.Symbol symbs _ ->
+       match Versdep.arg_symbol spec with
+       [ Some (symbs, _) ->
            let s = make_symlist symbs in
            let synt = key ^ " " ^ s in
            eprintf "  %s %s\n" synt (align_doc synt doc)
-       | _ -> eprintf "  %s %s\n" key (align_doc key doc) ])
+       | None ->
+           eprintf "  %s %s\n" key (align_doc key doc) ])
     l
 ;
 
@@ -163,7 +182,7 @@ Other options:
 
 value parse spec_list anon_fun remaining_args =
   let spec_list =
-    Sort.list (fun (k1, _, _) (k2, _, _) -> k1 >= k2) spec_list
+    list_sort (fun (k1, _, _) (k2, _, _) -> compare k2 k1) spec_list
   in
   parse_aux spec_list anon_fun remaining_args
 ;
