@@ -1,6 +1,5 @@
 (* camlp5r *)
-(* $Id: ast2pt.mli,v 1.8 2010/02/19 09:06:37 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2010 *)
+(* $Id: ast2pt.mli,v 6.1 2010-09-15 16:00:24 deraugla Exp $ *)
 
 (** Conversion between Camlp5 AST into OCaml AST *)
 
@@ -12,13 +11,7 @@ value implem : string -> list MLast.str_item -> Parsetree.structure;
        implementation (ml file), [fname] being the input file name. *)
 value phrase : MLast.str_item -> Parsetree.toplevel_phrase;
    (** [phrase sil] return the OCaml equivalent AST of a toplevel phrase. *)
-
 value mkloc : Ploc.t -> Location.t;
-   (** Convert a Camlp5 location into a OCaml location. *)
-
+   (** Convert a Camlp5 location into an OCaml location. *)
 value fast : ref bool;
    (** Flag to generate fast (unsafe) access to arrays. Default: False. *)
-value no_constructors_arity : ref bool;
-   (** Flag to generate nodes telling that constructor arity is not taken
-       into account in the AST (e.g. True for normal syntax, False for
-       revised syntax). Default: False. *)
