@@ -1,7 +1,9 @@
-(* camlp5r q_MLast.cmo *)
-(* $Id: exparser.ml,v 1.14 2010/02/19 09:06:37 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: exparser.ml,v 6.3 2010-10-01 13:12:19 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "q_MLast.cmo";
+
 type spat_comp =
   [ SpTrm of MLast.loc and MLast.patt and MLast.v (option MLast.expr)
   | SpNtr of MLast.loc and MLast.patt and MLast.expr
@@ -81,9 +83,8 @@ and no_raising_failure_fun =
 ;
 
 value rec subst v e =
-  let loc = MLast.loc_of_expr e in
   match e with
-  [ <:expr< $lid:x$ >> ->
+  [ <:expr:< $lid:x$ >> ->
       let x = if x = v then strm_n else x in
       <:expr< $lid:x$ >>
   | <:expr< $uid:_$ >> -> e
@@ -91,10 +92,10 @@ value rec subst v e =
   | <:expr< $chr:_$ >> -> e
   | <:expr< $str:_$ >> -> e
   | <:expr< $_$ . $_$ >> -> e
-  | <:expr< let $flag:rf$ $list:pel$ in $e$ >> ->
+  | <:expr:< let $flag:rf$ $list:pel$ in $e$ >> ->
       <:expr< let $flag:rf$ $list:List.map (subst_pe v) pel$ in $subst v e$ >>
-  | <:expr< $e1$ $e2$ >> -> <:expr< $subst v e1$ $subst v e2$ >>
-  | <:expr< ( $list:el$ ) >> -> <:expr< ( $list:List.map (subst v) el$ ) >>
+  | <:expr:< $e1$ $e2$ >> -> <:expr< $subst v e1$ $subst v e2$ >>
+  | <:expr:< ( $list:el$ ) >> -> <:expr< ( $list:List.map (subst v) el$ ) >>
   | _ -> raise Not_found ]
 and subst_pe v (p, e) =
   match p with
@@ -113,11 +114,7 @@ value rec perhaps_bound s =
 
 value wildcard_if_not_bound p e =
   match p with
-  [ <:patt< $lid:s$ >> ->
-      if perhaps_bound s e then p
-      else
-        let loc = MLast.loc_of_patt p in
-        <:patt< _ >>
+  [ <:patt:< $lid:s$ >> -> if perhaps_bound s e then p else <:patt< _ >>
   | _ -> p ]
 ;
 
@@ -356,11 +353,15 @@ value cparser loc bpo pc =
   let pc = left_factorize pc in
   let e = parser_cases loc pc in
   let e =
+    let loc = Ploc.with_comment loc "" in
     match bpo with
     [ Some bp -> <:expr< let $bp$ = Stream.count $lid:strm_n$ in $e$ >>
     | None -> e ]
   in
-  let p = <:patt< ($lid:strm_n$ : Stream.t _) >> in
+  let p =
+    let loc = Ploc.with_comment loc "" in
+    <:patt< ($lid:strm_n$ : Stream.t _) >>
+  in
   <:expr< fun $p$ -> $e$ >>
 ;
 
@@ -373,8 +374,10 @@ value rec is_not_bound s =
 
 value cparser_match loc me bpo pc =
   let pc = left_factorize pc in
-  let pc = parser_cases loc pc in
+  let iloc = Ploc.with_comment loc "" in
+  let pc = parser_cases iloc pc in
   let e =
+    let loc = iloc in
     match bpo with
     [ Some bp -> <:expr< let $bp$ = Stream.count $lid:strm_n$ in $pc$ >>
     | None -> pc ]
@@ -383,6 +386,7 @@ value cparser_match loc me bpo pc =
   [ <:expr< $lid:x$ >> when x = strm_n -> e
   | _ ->
       let p =
+        let loc = iloc in
         if is_not_bound strm_n e then <:patt< _ >>
         else <:patt< $lid:strm_n$ >>
       in
