@@ -1,7 +1,9 @@
-(* camlp5r pa_macro.cmo *)
-(* $Id: mLast.mli,v 1.55 2010/02/19 09:06:37 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: mLast.mli,v 6.17 2011-02-17 09:17:07 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_macro.cmo";
+
 (* Module [MLast]: abstract syntax tree.
 
    This is undocumented because the AST is not supposed to be used
@@ -19,35 +21,7 @@ END;
 
 type v 'a = V 'a;
 
-type ctyp =
-  [ TyAcc of loc and ctyp and ctyp
-  | TyAli of loc and ctyp and ctyp
-  | TyAny of loc
-  | TyApp of loc and ctyp and ctyp
-  | TyArr of loc and ctyp and ctyp
-  | TyCls of loc and V (list string)
-  | TyLab of loc and V string and ctyp
-  | TyLid of loc and V string
-  | TyMan of loc and ctyp and ctyp
-  | TyObj of loc and V (list (string * ctyp)) and V bool
-  | TyOlb of loc and V string and ctyp
-  | TyPol of loc and V (list string) and ctyp
-  | TyQuo of loc and V string
-  | TyRec of loc and V (list (loc * string * bool * ctyp))
-  | TySum of loc and V (list (loc * V string * V (list ctyp)))
-  | TyTup of loc and V (list ctyp)
-  | TyUid of loc and V string
-  | TyVrn of loc and V (list poly_variant) and
-      option (option (V (list string)))
-  | IFDEF STRICT THEN
-      TyXtr of loc and string and option (V ctyp)
-    END ]
-and poly_variant =
-  [ PvTag of V string and V bool and V (list ctyp)
-  | PvInh of ctyp ]
-;
-
-type type_var = (V string * (bool * bool));
+type type_var = (V (option string) * option bool);
 
 type class_infos 'a =
   { ciLoc : loc;
@@ -57,7 +31,33 @@ type class_infos 'a =
     ciExp : 'a }
 ;
 
-type patt =
+type ctyp =
+  [ TyAcc of loc and ctyp and ctyp
+  | TyAli of loc and ctyp and ctyp
+  | TyAny of loc
+  | TyApp of loc and ctyp and ctyp
+  | TyArr of loc and ctyp and ctyp
+  | TyCls of loc and V (list string)
+  | TyLab of loc and V string and ctyp
+  | TyLid of loc and V string
+  | TyMan of loc and ctyp and V bool and ctyp
+  | TyObj of loc and V (list (string * ctyp)) and V bool
+  | TyOlb of loc and V string and ctyp
+  | TyPck of loc and module_type
+  | TyPol of loc and V (list string) and ctyp
+  | TyPot of loc and V (list string) and ctyp
+  | TyQuo of loc and V string
+  | TyRec of loc and V (list (loc * string * bool * ctyp))
+  | TySum of loc and V (list (loc * V string * V (list ctyp) * option ctyp))
+  | TyTup of loc and V (list ctyp)
+  | TyUid of loc and V string
+  | TyVrn of loc and V (list poly_variant) and
+      option (option (V (list string)))
+  | TyXtr of loc and string and option (V ctyp) ]
+and poly_variant =
+  [ PvTag of loc and V string and V bool and V (list ctyp)
+  | PvInh of loc and ctyp ]
+and patt =
   [ PaAcc of loc and patt and patt
   | PaAli of loc and patt and patt
   | PaAnt of loc and patt
@@ -65,24 +65,24 @@ type patt =
   | PaApp of loc and patt and patt
   | PaArr of loc and V (list patt)
   | PaChr of loc and V string
-  | PaInt of loc and V string and string
   | PaFlo of loc and V string
-  | PaLab of loc and V string and option patt
+  | PaInt of loc and V string and string
+  | PaLab of loc and V (list (patt * V (option patt)))
   | PaLaz of loc and patt
   | PaLid of loc and V string
-  | PaOlb of loc and V string and option (patt * V (option expr))
+  | PaNty of loc and V string
+  | PaOlb of loc and patt and V (option expr)
   | PaOrp of loc and patt and patt
-  | PaRng of loc and patt and patt
   | PaRec of loc and V (list (patt * patt))
+  | PaRng of loc and patt and patt
   | PaStr of loc and V string
   | PaTup of loc and V (list patt)
   | PaTyc of loc and patt and ctyp
   | PaTyp of loc and V (list string)
   | PaUid of loc and V string
+  | PaUnp of loc and V string and option module_type
   | PaVrn of loc and V string
-  | IFDEF STRICT THEN
-      PaXtr of loc and string and option (V patt)
-    END ]
+  | PaXtr of loc and string and option (V patt) ]
 and expr =
   [ ExAcc of loc and expr and expr
   | ExAnt of loc and expr
@@ -99,7 +99,7 @@ and expr =
   | ExFun of loc and V (list (patt * V (option expr) * expr))
   | ExIfe of loc and expr and expr and expr
   | ExInt of loc and V string and string
-  | ExLab of loc and V string and option expr
+  | ExLab of loc and V (list (patt * V (option expr)))
   | ExLaz of loc and expr
   | ExLet of loc and V bool and V (list (patt * expr)) and expr
   | ExLid of loc and V string
@@ -107,8 +107,9 @@ and expr =
   | ExMat of loc and expr and V (list (patt * V (option expr) * expr))
   | ExNew of loc and V (list string)
   | ExObj of loc and V (option patt) and V (list class_str_item)
-  | ExOlb of loc and V string and option expr
+  | ExOlb of loc and patt and V (option expr)
   | ExOvr of loc and V (list (string * expr))
+  | ExPck of loc and module_expr and option module_type
   | ExRec of loc and V (list (patt * expr)) and option expr
   | ExSeq of loc and V (list expr)
   | ExSnd of loc and expr and V string
@@ -120,9 +121,7 @@ and expr =
   | ExUid of loc and V string
   | ExVrn of loc and V string
   | ExWhi of loc and expr and V (list expr)
-  | IFDEF STRICT THEN
-      ExXtr of loc and string and option (V expr)
-    END ]
+  | ExXtr of loc and string and option (V expr) ]
 and module_type =
   [ MtAcc of loc and module_type and module_type
   | MtApp of loc and module_type and module_type
@@ -130,11 +129,10 @@ and module_type =
   | MtLid of loc and V string
   | MtQuo of loc and V string
   | MtSig of loc and V (list sig_item)
+  | MtTyo of loc and module_expr
   | MtUid of loc and V string
   | MtWit of loc and module_type and V (list with_constr)
-  | IFDEF STRICT THEN
-      MtXtr of loc and string and option (V module_type)
-    END ]
+  | MtXtr of loc and string and option (V module_type) ]
 and sig_item =
   [ SgCls of loc and V (list (class_infos class_type))
   | SgClt of loc and V (list (class_infos class_type))
@@ -147,14 +145,14 @@ and sig_item =
   | SgMty of loc and V string and module_type
   | SgOpn of loc and V (list string)
   | SgTyp of loc and V (list type_decl)
-  | SgUse of loc and string and list (sig_item * loc)
+  | SgUse of loc and V string and V (list (sig_item * loc))
   | SgVal of loc and V string and ctyp
-  | IFDEF STRICT THEN
-      SgXtr of loc and string and option (V sig_item)
-    END ]
+  | SgXtr of loc and string and option (V sig_item) ]
 and with_constr =
-  [ WcTyp of loc and V (list string) and V (list type_var) and V bool and ctyp
-  | WcMod of loc and V (list string) and module_expr ]
+  [ WcMod of loc and V (list string) and module_expr
+  | WcMos of loc and V (list string) and module_expr
+  | WcTyp of loc and V (list string) and V (list type_var) and V bool and ctyp
+  | WcTys of loc and V (list string) and V (list type_var) and ctyp ]
 and module_expr =
   [ MeAcc of loc and module_expr and module_expr
   | MeApp of loc and module_expr and module_expr
@@ -162,9 +160,8 @@ and module_expr =
   | MeStr of loc and V (list str_item)
   | MeTyc of loc and module_expr and module_type
   | MeUid of loc and V string
-  | IFDEF STRICT THEN
-      MeXtr of loc and string and option (V module_expr)
-    END ]
+  | MeUnp of loc and expr and option module_type
+  | MeXtr of loc and string and option (V module_expr) ]
 and str_item =
   [ StCls of loc and V (list (class_infos class_expr))
   | StClt of loc and V (list (class_infos class_type))
@@ -178,31 +175,30 @@ and str_item =
   | StMty of loc and V string and module_type
   | StOpn of loc and V (list string)
   | StTyp of loc and V (list type_decl)
-  | StUse of loc and string and list (str_item * loc)
+  | StUse of loc and V string and V (list (str_item * loc))
   | StVal of loc and V bool and V (list (patt * expr))
-  | IFDEF STRICT THEN
-      StXtr of loc and string and option (V str_item)
-    END ]
+  | StXtr of loc and string and option (V str_item) ]
 and type_decl =
-  { tdNam : (loc * V string);
+  { tdNam : V (loc * V string);
     tdPrm : V (list type_var);
     tdPrv : V bool;
     tdDef : ctyp;
     tdCon : V (list (ctyp * ctyp)) }
 and class_type =
-  [ CtCon of loc and V (list string) and V (list ctyp)
+  [ CtAcc of loc and class_type and class_type
+  | CtApp of loc and class_type and class_type
+  | CtCon of loc and class_type and V (list ctyp)
   | CtFun of loc and ctyp and class_type
+  | CtIde of loc and V string
   | CtSig of loc and V (option ctyp) and V (list class_sig_item)
-  | IFDEF STRICT THEN
-      CtXtr of loc and string and option (V class_type)
-    END ]
+  | CtXtr of loc and string and option (V class_type) ]
 and class_sig_item =
   [ CgCtr of loc and ctyp and ctyp
   | CgDcl of loc and V (list class_sig_item)
   | CgInh of loc and class_type
-  | CgMth of loc and V string and V bool and ctyp
-  | CgVal of loc and V string and V bool and ctyp
-  | CgVir of loc and V string and V bool and ctyp ]
+  | CgMth of loc and V bool and V string and ctyp
+  | CgVal of loc and V bool and V string and ctyp
+  | CgVir of loc and V bool and V string and ctyp ]
 and class_expr =
   [ CeApp of loc and class_expr and expr
   | CeCon of loc and V (list string) and V (list ctyp)
@@ -210,17 +206,17 @@ and class_expr =
   | CeLet of loc and V bool and V (list (patt * expr)) and class_expr
   | CeStr of loc and V (option patt) and V (list class_str_item)
   | CeTyc of loc and class_expr and class_type
-  | IFDEF STRICT THEN
-      CeXtr of loc and string and option (V class_expr)
-    END ]
+  | CeXtr of loc and string and option (V class_expr) ]
 and class_str_item =
   [ CrCtr of loc and ctyp and ctyp
   | CrDcl of loc and V (list class_str_item)
   | CrInh of loc and class_expr and V (option string)
   | CrIni of loc and expr
-  | CrMth of loc and V string and V bool and expr and V (option ctyp)
-  | CrVal of loc and V string and V bool and expr
-  | CrVir of loc and V string and V bool and ctyp ]
+  | CrMth of loc and V bool and V bool and V string and V (option ctyp) and
+      expr
+  | CrVal of loc and V bool and V bool and V string and expr
+  | CrVav of loc and V bool and V string and ctyp
+  | CrVir of loc and V bool and V string and ctyp ]
 ;
 
 external loc_of_ctyp : ctyp -> loc = "%field0";
