@@ -1,8 +1,13 @@
-(* camlp5r pa_macro.cmo pa_extend.cmo *)
-(* $Id: pcaml.ml,v 1.71 2010/04/28 17:05:07 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pcaml.ml,v 6.18 2011-03-15 12:37:26 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
-value version = "5.14";
+#load "pa_macro.cmo";
+#load "pa_extend.cmo";
+
+open Printf;
+
+value version = "6.02.2";
 value syntax_name = ref "";
 
 value gram =
@@ -13,10 +18,14 @@ value gram =
      Plexing.tok_comm = None}
 ;
 
+type status = option Ploc.t;
+
 value interf = Grammar.Entry.create gram "interf";
 value implem = Grammar.Entry.create gram "implem";
 value top_phrase = Grammar.Entry.create gram "top_phrase";
 value use_file = Grammar.Entry.create gram "use_file";
+value signature = Grammar.Entry.create gram "signature";
+value structure = Grammar.Entry.create gram "structure";
 value sig_item = Grammar.Entry.create gram "sig_item";
 value str_item = Grammar.Entry.create gram "str_item";
 value module_type = Grammar.Entry.create gram "module_type";
@@ -26,7 +35,7 @@ value patt = Grammar.Entry.create gram "patt";
 value ipatt = Grammar.Entry.create gram "ipatt";
 value ctyp = Grammar.Entry.create gram "ctyp";
 value let_binding = Grammar.Entry.create gram "let_binding";
-value type_declaration = Grammar.Entry.create gram "type_declaration";
+value type_decl = Grammar.Entry.create gram "type_declaration";
 value match_case = Grammar.Entry.create gram "match_case";
 value constructor_declaration =
   Grammar.Entry.create gram "constructor_declaration";
@@ -51,12 +60,12 @@ value rec skip_to_eol cs =
 ;
 value sync = ref skip_to_eol;
 
-value input_file = ref "";
+value input_file = Plexing.input_file;
 value output_file = ref None;
 
 value warning_default_function loc txt = do {
   let (bp, ep) = (Ploc.first_pos loc, Ploc.last_pos loc) in
-  Printf.eprintf "<W> loc %d %d: %s\n" bp ep txt;
+  eprintf "<W> loc %d %d: %s\n" bp ep txt;
   flush stderr
 };
 
@@ -85,8 +94,11 @@ value quotation_location () =
 value expand_quotation gloc expander shift name str = do {
   let new_warning =
     let warn = warning.val in
-    let shift = Ploc.first_pos gloc + shift in
-    fun loc txt -> warn (Ploc.shift shift loc) txt
+    fun loc txt ->
+      let bp = Ploc.first_pos loc in
+      let ep = Ploc.last_pos loc in
+      let loc = Ploc.sub gloc (shift + bp) (ep - bp) in
+      warn loc txt
   in
   let restore =
     let old_warning = warning.val in
@@ -111,10 +123,11 @@ value expand_quotation gloc expander shift name str = do {
                Ploc.make_unlined
                  (shift + Ploc.first_pos loc, shift + Ploc.last_pos loc)
              else
-               Ploc.make (gloc_line_nb + loc_line_nb - 1)
+               Ploc.make_loc (Ploc.file_name loc)
+                 (gloc_line_nb + loc_line_nb - 1)
                  (if loc_line_nb = 1 then Ploc.bol_pos gloc
                   else shift + Ploc.bol_pos loc)
-                 (shift + Ploc.first_pos loc, shift + Ploc.last_pos loc)
+                 (shift + Ploc.first_pos loc, shift + Ploc.last_pos loc) ""
            in
            raise (Ploc.Exc loc exc1)
        | exc ->
@@ -143,10 +156,21 @@ value parse_quotation_result entry loc shift name str =
 ;
 
 value handle_quotation loc proj proj2 in_expr entry reloc (name, str) =
+  let (name, locate) =
+    let len = String.length name in
+    if len = 0 then (name, False)
+    else if name.[len-1] = ':' then (String.sub name 0 (len - 1), False)
+    else if name.[len-1] = '@' then (String.sub name 0 (len - 1), True)
+    else (name, False)
+  in
   let shift =
     match name with
     [ "" -> String.length "<<"
-    | _ -> String.length "<:" + String.length name + String.length "<" ]
+    | _ ->
+        if locate then
+          String.length "<:" + String.length name + String.length ":<"
+        else
+          String.length "<:" + String.length name + String.length "<" ]
   in
   let expander =
     try Quotation.find name with exc ->
@@ -159,9 +183,20 @@ value handle_quotation loc proj proj2 in_expr entry reloc (name, str) =
         let new_str = expand_quotation loc (f in_expr) shift name str in
         parse_quotation_result entry loc shift name new_str
     | Quotation.ExAst fe_fp ->
+        let str = if locate then "@" ^ str else str in
         expand_quotation loc (proj fe_fp) shift name str ]
   in
-  reloc (fun _ -> loc) shift ast
+  let floc =
+    let evaluated = ref None in
+    fun _ ->
+      match evaluated.val with
+      [ Some loc -> loc
+      | None -> do {
+          evaluated.val := Some (Ploc.with_comment loc "");
+          loc
+        } ]
+  in
+  reloc floc shift ast
 ;
 
 value expr_eoi = Grammar.Entry.create gram "expr_eoi";
@@ -197,19 +232,20 @@ value find_line loc str =
     else find (succ i) line (succ col)
 ;
 
-value loc_fmt =
+value string_of_loc fname line bp ep =
   match Sys.os_type with
   [ "MacOS" ->
-      format_of_string "File \"%s\"; line %d; characters %d to %d\n### "
+      sprintf "File \"%s\"; line %d; characters %d to %d\n### " fname line
+        bp ep
   | _ ->
-      format_of_string "File \"%s\", line %d, characters %d-%d:\n" ]
+      sprintf "File \"%s\", line %d, characters %d-%d:\n" fname line bp ep ]
 ;
 
 value report_quotation_error name ctx = do {
   let name = if name = "" then Quotation.default.val else name in
   Format.print_flush ();
   Format.open_hovbox 2;
-  Printf.eprintf "While %s \"%s\":"
+  eprintf "While %s \"%s\":"
     (match ctx with
      [ Finding -> "finding quotation"
      | Expanding -> "expanding quotation"
@@ -219,7 +255,7 @@ value report_quotation_error name ctx = do {
   [ ParsingResult loc str ->
       match quotation_dump_file.val with
       [ Some dump_file -> do {
-          Printf.eprintf " dumping result...\n";
+          eprintf " dumping result...\n";
           flush stderr;
           try do {
             let (line, c1, c2) = find_line loc str in
@@ -228,24 +264,23 @@ value report_quotation_error name ctx = do {
             output_string oc "\n";
             flush oc;
             close_out oc;
-            Printf.eprintf loc_fmt dump_file line c1 c2;
+            eprintf "%s" (string_of_loc dump_file line c1 c2);
             flush stderr
           }
           with _ -> do {
-            Printf.eprintf "Error while dumping result in file \"%s\""
-              dump_file;
-            Printf.eprintf "; dump aborted.\n";
+            eprintf "Error while dumping result in file \"%s\"" dump_file;
+            eprintf "; dump aborted.\n";
             flush stderr
           }
         }
       | None -> do {
           if input_file.val = "" then
-            Printf.eprintf
+            eprintf
               "\n(consider setting variable Pcaml.quotation_dump_file)\n"
-          else Printf.eprintf " (consider using option -QD)\n";
+          else eprintf " (consider using option -QD)\n";
           flush stderr
         } ]
-  | _ -> do { Printf.eprintf "\n"; flush stderr } ]
+  | _ -> do { eprintf "\n"; flush stderr } ]
 };
 
 value print_format str = do {
@@ -301,8 +336,12 @@ value print_exn =
       else print_format ("Parse error: " ^ str)
   | Stream.Failure -> Format.print_string "Parse failure"
   | Plexing.Error str -> do {
-      Format.print_string "Lexing error: ";
-      Format.print_string str
+      Format.print_string "Lexing error";
+      if str <> "" then do {
+        Format.print_string ": ";
+        Format.print_string str
+      }
+      else Format.print_string ".";
     }
   | Failure str -> do {
       Format.print_string "Failure: ";
@@ -356,7 +395,7 @@ value report_error exn =
   | e -> print_exn exn ]
 ;
 
-value no_constructors_arity = Ast2pt.no_constructors_arity;
+value no_constructors_arity = Prtools.no_constructors_arity;
 
 value arg_spec_list_ref = ref [];
 value arg_spec_list () = arg_spec_list_ref.val;
@@ -385,6 +424,7 @@ value pr_expr_fun_args = ref Extfun.empty;
 
 value flag_comments_in_phrases = ref True;
 value flag_equilibrate_cases = ref False;
+value flag_compatible_old_versions_of_ocaml = ref False;
 
 value inter_phrases = ref None;
 
@@ -449,9 +489,9 @@ add_option "-mode"
 add_option "-pmode"
   (Arg.Unit
      (fun () -> do {
-        if strict_mode.val then Printf.eprintf "strict\n"
-        else Printf.eprintf "transitional\n";
+        if strict_mode.val then eprintf "strict\n"
+        else eprintf "transitional\n";
         flush stderr;
         exit 0
       }))
-  "Print the current mode and exit";
+  "Print the current mode and exit.";
