@@ -1,7 +1,9 @@
-(* camlp5r pa_macro.cmo *)
-(* $Id: pcaml.mli,v 1.31 2010/02/19 09:06:38 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pcaml.mli,v 6.4 2010-11-12 23:24:00 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_macro.cmo";
+
 (** Language grammar, entries and printers.
 
    Hold variables to be set by language syntax extensions. Some of them
@@ -11,10 +13,12 @@ value syntax_name : ref string;
 
 (** {6 Parsers} *)
 
+type status = option Ploc.t;
+
 value parse_interf :
-  ref (Stream.t char -> (list (MLast.sig_item * MLast.loc) * bool));
+  ref (Stream.t char -> (list (MLast.sig_item * MLast.loc) * status));
 value parse_implem :
-  ref (Stream.t char -> (list (MLast.str_item * MLast.loc) * bool));
+  ref (Stream.t char -> (list (MLast.str_item * MLast.loc) * status));
    (** Called when parsing an interface (mli file) or an implementation
        (ml file) to build the syntax tree; the returned list contains the
        phrases (signature items or structure items) and their locations;
@@ -30,12 +34,14 @@ value parse_implem :
 value gram : Grammar.g;
    (** Grammar variable of the OCaml language *)
 
-value interf : Grammar.Entry.e (list (MLast.sig_item * MLast.loc) * bool);
-value implem : Grammar.Entry.e (list (MLast.str_item * MLast.loc) * bool);
+value interf : Grammar.Entry.e (list (MLast.sig_item * MLast.loc) * status);
+value implem : Grammar.Entry.e (list (MLast.str_item * MLast.loc) * status);
 value top_phrase : Grammar.Entry.e (option MLast.str_item);
 value use_file : Grammar.Entry.e (list MLast.str_item * bool);
 value module_type : Grammar.Entry.e MLast.module_type;
 value module_expr : Grammar.Entry.e MLast.module_expr;
+value signature : Grammar.Entry.e (MLast.v (list MLast.sig_item));
+value structure : Grammar.Entry.e (MLast.v (list MLast.str_item));
 value sig_item : Grammar.Entry.e MLast.sig_item;
 value str_item : Grammar.Entry.e MLast.str_item;
 value expr : Grammar.Entry.e MLast.expr;
@@ -43,11 +49,13 @@ value patt : Grammar.Entry.e MLast.patt;
 value ipatt : Grammar.Entry.e MLast.patt;
 value ctyp : Grammar.Entry.e MLast.ctyp;
 value let_binding : Grammar.Entry.e (MLast.patt * MLast.expr);
-value type_declaration : Grammar.Entry.e MLast.type_decl;
+value type_decl : Grammar.Entry.e MLast.type_decl;
 value match_case :
   Grammar.Entry.e (MLast.patt * MLast.v (option MLast.expr) * MLast.expr);
 value constructor_declaration :
-  Grammar.Entry.e (MLast.loc * MLast.v string * MLast.v (list MLast.ctyp));
+  Grammar.Entry.e
+    (MLast.loc * MLast.v string * MLast.v (list MLast.ctyp) *
+     option MLast.ctyp);
 value label_declaration :
   Grammar.Entry.e (MLast.loc * string * bool * MLast.ctyp);
 value with_constr : Grammar.Entry.e MLast.with_constr;
@@ -84,8 +92,10 @@ value handle_patt_quotation : MLast.loc -> (string * string) -> MLast.patt;
 
 (** {6 Printers} *)
 
-value print_interf : ref (list (MLast.sig_item * MLast.loc) -> unit);
-value print_implem : ref (list (MLast.str_item * MLast.loc) -> unit);
+value print_interf :
+  ref ((list (MLast.sig_item * MLast.loc) * MLast.loc) -> unit);
+value print_implem :
+  ref ((list (MLast.str_item * MLast.loc) * MLast.loc) -> unit);
 
 value pr_expr : Eprinter.t MLast.expr;
 value pr_patt : Eprinter.t MLast.patt;
@@ -161,3 +171,4 @@ value expr_reloc :
 value rename_id : ref (string -> string);
 value flag_comments_in_phrases : ref bool;
 value flag_equilibrate_cases : ref bool;
+value flag_compatible_old_versions_of_ocaml : ref bool;
