@@ -1,7 +1,10 @@
-(* camlp5r q_MLast.cmo *)
-(* $Id: prtools.ml,v 1.17 2010/02/19 09:06:38 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: prtools.ml,v 6.8 2010-10-07 13:06:40 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "q_MLast.cmo";
+#load "pa_macro.cmo";
+
 open Pretty;
 
 type pr_context =
@@ -11,6 +14,14 @@ type pr_context =
 
 type pr_fun 'a = pr_context -> 'a -> string;
 
+IFDEF OCAML_VERSION <= OCAML_1_07 OR COMPATIBLE_WITH_OLD_OCAML THEN
+  value with_ind_bef = Pprintf.with_ind_bef;
+  value with_bef = Pprintf.with_bef;
+  value with_bef_aft_dang = Pprintf.with_bef_aft_dang;
+  value with_aft = Pprintf.with_aft;
+  value with_aft_dang = Pprintf.with_aft_dang;
+END;
+
 value tab ind = String.make ind ' ';
 
 (* horizontal list *)
@@ -93,6 +104,13 @@ value rec vlistf pc fl =
         (vlistf {(pc) with bef = tab pc.ind} fl) ]
 ;
 
+value hvlistl elem eleml pc xl =
+  if Pretty.horizontally () then
+    hlistl elem eleml pc xl
+  else
+    vlistl elem eleml pc xl
+;
+
 value rise_string ind sh b s =
   (* hack for "plistl" (below): if s is a "string" (i.e. starting with
      double-quote) which contains newlines, attempt to concat its first
@@ -351,94 +369,17 @@ module Buff =
   end
 ;
 
-value rev_extract_comment strm =
-  let rec find_comm len =
-    parser
-    [ [: `' '; a = find_comm (Buff.store len ' ') ! :] -> a
-    | [: `'\t'; a = find_comm (Buff.mstore len (String.make 8 ' ')) ! :] -> a
-    | [: `'\n'; a = find_comm (Buff.store len '\n') ! :] -> a
-    | [: `')'; a = find_star_bef_rparen (Buff.store len ')') ! :] -> a
-    | [: :] -> 0 ]
-  and find_star_bef_rparen len =
-    parser
-    [ [: `'*'; a = insert (Buff.store len '*') ! :] -> a
-    | [: :] -> 0 ]
-  and insert len =
-    parser
-    [ [: `')'; a = find_star_bef_rparen_in_comm (Buff.store len ')') ! :] -> a
-    | [: `'*'; a = find_lparen_aft_star (Buff.store len '*') ! :] -> a
-    | [: `'"'; a = insert_string (Buff.store len '"') ! :] -> a
-    | [: `x; a = insert (Buff.store len x) ! :] -> a
-    | [: :] -> len ]
-  and insert_string len =
-    parser
-    [ [: `'"'; a = insert (Buff.store len '"') ! :] -> a
-    | [: `x; a = insert_string (Buff.store len x) ! :] -> a
-    | [: :] -> len ]
-  and find_star_bef_rparen_in_comm len =
-    parser
-    [ [: `'*'; len = insert (Buff.store len '*'); a = insert len ! :] -> a
-    | [: a = insert len :] -> a ]
-  and find_lparen_aft_star len =
-    parser
-    [ [: `'('; a = while_space (Buff.store len '(') :] -> a
-    | [: a = insert len :] -> a ]
-  and while_space len =
-    parser
-    [ [: `' '; a = while_space (Buff.store len ' ') ! :] -> a
-    | [: `'\t'; a = while_space (Buff.mstore len (String.make 8 ' ')) :] -> a
-    | [: `'\n'; a = while_space (Buff.store len '\n') ! :] -> a
-    | [: `')'; a = find_star_bef_rparen_again len ! :] -> a
-    | [: :] -> len ]
-  and find_star_bef_rparen_again len =
-    parser
-    [ [: `'*'; a = insert (Buff.mstore len ")*") ! :] -> a
-    | [: :] -> len ]
-  in
-  let len = find_comm 0 strm in
-  let s = Buff.get len in
-  loop (len - 1) 0 0 where rec loop i nl_bef ind_bef =
-    if i <= 0 then ("", 0, 0)
-    else if s.[i] = '\n' then loop (i - 1) (nl_bef + 1) ind_bef
-    else if s.[i] = ' ' then loop (i - 1) nl_bef (ind_bef + 1)
+value comment_info s =
+  loop 0 0 0 where rec loop i nl_bef ind_bef =
+    if i >= String.length s then ("", 0, 0)
+    else if s.[i] = '\n' then loop (i + 1) (nl_bef + 1) 0
+    else if s.[i] = ' ' then loop (i + 1) nl_bef (ind_bef + 1)
     else do {
-      let s = String.sub s 0 (i + 1) in
-      for i = 0 to String.length s / 2 - 1 do {
-        let t = s.[i] in
-        s.[i] := s.[String.length s - i - 1];
-        s.[String.length s - i - 1] := t;
-      };
+      let s = String.sub s i (String.length s - i) in
       (s, nl_bef, ind_bef)
     }
 ;
 
-value source = ref "";
-value comm_min_pos = ref 0;
-
-value set_comm_min_pos bp = comm_min_pos.val := bp;
-
-value rev_read_comment_in_file bp ep =
-  let strm =
-    Stream.from
-      (fun i ->
-         let j = bp - i - 1 in
-         if j < comm_min_pos.val || j >= String.length source.val then None
-         else Some source.val.[j])
-  in
-  let (s, nl_bef, ind_bef) = rev_extract_comment strm in
-  if s = "" then
-    (* heuristic to find the possible comment before 'begin' or left
-       parenthesis *)
-    loop 0 where rec loop i =
-      match strm with parser
-      [ [: `'(' when i = 0 :] -> rev_extract_comment strm
-      | [: `c when c = "begin".[4-i] :] ->
-          if i = String.length "begin" - 1 then rev_extract_comment strm
-          else loop (i + 1)
-      | [: :] -> (s, nl_bef, ind_bef) ]
-  else (s, nl_bef, ind_bef)
-;
-
 value adjust_comment_indentation ind s nl_bef ind_bef =
   if s = "" then ""
   else
@@ -473,55 +414,100 @@ value adjust_comment_indentation ind s nl_bef ind_bef =
         loop olen i
 ;
 
+value eight_chars = String.make 8 ' ';
+value expand_tabs s =
+  loop 0 0 where rec loop len i =
+    if i = String.length s then Buff.get len
+    else if s.[i] = '\t' then loop (Buff.mstore len eight_chars) (i + 1)
+    else loop (Buff.store len s.[i]) (i + 1)
+;
+
 value comm_bef ind loc =
-  let bp = Ploc.first_pos loc in
-  let ep = Ploc.last_pos loc in
-  let (s, nl_bef, ind_bef) = rev_read_comment_in_file bp ep in
+  let s = expand_tabs (Ploc.comment loc) in
+  let (s, nl_bef, ind_bef) = comment_info s in
   adjust_comment_indentation ind s nl_bef ind_bef
 ;
 
-(* For pretty printing improvement:
-   - if e is a "sequence" or a "let..in sequence", get "the list of its
-     expressions", which is flattened (merging sequences inside sequences
-     and changing "let..in do {e1; .. en}" into "do {let..in e1; .. en}",
-     and return Some "the resulting expression list".
-   - otherwise return None *)
-value flatten_sequence e =
-  let rec get_sequence =
+value no_constructors_arity = ref False;
+
+value expand_module_prefix m =
+  loop where rec loop rev_lel =
     fun
-    [ <:expr< do { $list:el$ } >> -> Some el
-    | <:expr< let $flag:rf$ $list:pel$ in $e$ >> as se ->
-        match get_sequence e with
-        [ Some [e :: el] ->
-            let e =
-              let loc =
-                let loc1 = MLast.loc_of_expr se in
-                let loc2 = MLast.loc_of_expr e in
-                Ploc.encl loc1 loc2
-              in
-              <:expr< let $flag:rf$ $list:pel$ in $e$ >>
-            in
-            Some [e :: el]
-        | None | _ -> None ]
-    | _ -> None ]
-  in
-  match get_sequence e with
-  [ Some el ->
-      let rec list_of_sequence =
-        fun
-        [ [e :: el] ->
-            match get_sequence e with
-            [ Some el1 -> list_of_sequence (el1 @ el)
-            | None -> [e :: list_of_sequence el] ]
-        | [] -> [] ]
-      in
-      Some (list_of_sequence el)
-  | None -> None ]
+    [ [(p, e) :: rest] -> do {
+        let p =
+          match p with
+          [ <:patt< $uid:_$.$_$ >> -> p
+          | _ ->
+              let loc = MLast.loc_of_patt p in
+              <:patt< $uid:m$.$p$ >> ]
+        in
+        loop [(p, e) :: rev_lel] rest
+      }
+  | [] -> List.rev rev_lel ]
 ;
 
-value hvlistl elem eleml pc xl =
-  if Pretty.horizontally () then
-    hlistl elem eleml pc xl
-  else
-    vlistl elem eleml pc xl
+value do_split_or_patterns_with_bindings pel =
+  loop [] pel where rec loop rev_pel =
+    fun
+    [ [(p, wo, e) :: pel] ->
+        let (rev_pel, pel) =
+          let (p, as_opt) =
+            match p with
+            [ MLast.PaAli loc p1 p2 -> (p1, fun p -> MLast.PaAli loc p p2)
+            | _ -> (p, fun p -> p) ]
+          in
+          match p with
+          [ MLast.PaOrp loc p1 p2 ->
+              let has_bindings =
+                loop p2 where rec loop =
+                  fun
+                  [ MLast.PaLid _ _ -> True
+                  | MLast.PaApp _ p1 p2 -> loop p1 || loop p2
+                  | _ -> False ]
+              in
+              if has_bindings then
+                let pl =
+                  loop [] p where rec loop pl =
+                    fun
+                    [ MLast.PaOrp loc p1 p2 -> loop [p2 :: pl] p1
+                    | p -> [p :: pl] ]
+                in
+                let rev_pel =
+                  List.fold_left
+                    (fun rev_pel p -> [(as_opt p, wo, e) :: rev_pel]) rev_pel
+                    pl
+                in
+                (rev_pel, pel)
+              else ([(as_opt p, wo, e) :: rev_pel], pel)
+          | _ -> ([(as_opt p, wo, e) :: rev_pel], pel) ]
+        in
+        loop rev_pel pel
+    | [] -> List.rev rev_pel ]
+;
+
+value record_without_with loc e lel =
+  try
+    let (m, name) =
+      let (m, sl) =
+        List.fold_right
+          (fun (p, _) (m, sl) ->
+             match p with
+              [ <:patt< $lid:lab$ >> -> (m, [lab :: sl])
+              | <:patt< $uid:m$.$lid:lab$ >> -> (m, [lab :: sl])
+              | _ -> raise Exit ])
+          lel ("", [])
+      in
+      (m, String.concat "_" ["with" :: sl])
+    in
+    let f =
+      let f = <:expr< $lid:name$ >> in
+      if m = "" then f else <:expr< $uid:m$.$f$ >>
+    in
+    let e =
+      List.fold_left (fun e1 (_, e2) -> <:expr< $e1$ $e2$ >>)
+        <:expr< $f$ $e$ >> lel
+    in
+    Some e
+  with
+  [ Exit -> None ]
 ;
