@@ -1,5 +1,5 @@
 (* camlp5r *)
-(* $Id: prtools.mli,v 1.13 2010/02/19 09:06:38 deraugla Exp $ *)
+(* $Id: prtools.mli,v 6.3 2010-10-07 13:06:40 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
 type pr_context =
@@ -7,6 +7,8 @@ type pr_context =
     { ind : int; bef : string; aft : string; dang : string }
 ;
 
+type pr_fun 'a = Pprintf.pr_fun 'a;
+
 (* comments *)
 
 value comm_bef : int -> MLast.loc -> string;
@@ -14,17 +16,8 @@ value comm_bef : int -> MLast.loc -> string;
        given location [loc]. May be reindented using [ind]. Returns the
        empty string if no comment found. *)
 
-value source : ref string;
-   (** The initial source string, which must be set by the pretty printing
-       kit. Used by [comm_bef] above. *)
-value set_comm_min_pos : int -> unit;
-   (** Set the minimum position of the source where comments can be found,
-       (to prevent possible duplication of comments). *)
-
 (* meta functions to treat lists *)
 
-type pr_fun 'a = pr_context -> 'a -> string;
-
 value hlist : pr_fun 'a -> pr_fun (list 'a);
    (** horizontal list
        [hlist elem pc e] returns the horizontally pretty printed string
@@ -89,12 +82,17 @@ value hvlistl : pr_fun 'a -> pr_fun 'a -> pr_fun (list 'a);
 
 value tab : int -> string;
 
-value flatten_sequence : MLast.expr -> option (list MLast.expr);
-   (** [flatten_sequence e]. If [e] is an expression representing a sequence,
-       return the list of expressions of the sequence. If some of these
-       expressions are already sequences, they are expanded in the list.
-       If that list contains expressions of the form let..in sequence, this
-       sub-sequence is also flattened with the let..in spplies only to the
-       first expression of the sequence. If [e] is a let..in sequence, it
-       works the same way. If [e] is not a sequence nor a let..in sequence,
-       return None. *)
+value expand_module_prefix :
+  string -> list (MLast.patt * 'a) -> list (MLast.patt * 'a) ->
+    list (MLast.patt * 'a);
+
+value do_split_or_patterns_with_bindings :
+  list (MLast.patt * 'a * 'b) -> list (MLast.patt * 'a * 'b);
+
+value record_without_with :
+  Ploc.t -> MLast.expr -> list (MLast.patt * MLast.expr) -> option MLast.expr;
+
+value no_constructors_arity : ref bool;
+   (** Flag to generate nodes telling that constructor arity is not taken
+       into account in the AST (e.g. True for normal syntax, False for
+       revised syntax). Default: False. *)
