@@ -1,11 +1,15 @@
-(* camlp5r pa_macro.cmo pa_extend.cmo q_MLast.cmo *)
-(* $Id: pa_extend.ml,v 1.105 2010/02/19 09:06:38 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pa_extend.ml,v 6.5 2010-11-12 23:24:00 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_macro.cmo";
+#load "pa_extend.cmo";
+#load "q_MLast.cmo";
+
 value split_ext = ref False;
 
 Pcaml.add_option "-split_ext" (Arg.Set split_ext)
-  "Split EXTEND by functions to turn around a PowerPC problem.";
+  "Split EXTEND by using functions.";
 
 type loc = Ploc.t;
 
@@ -33,7 +37,8 @@ and a_psymbol 'e 'p =
 and a_symbol 'e 'p =
   [ ASflag of loc and a_symbol 'e 'p
   | ASkeyw of loc and a_string 'e
-  | ASlist of loc and lmin_len and a_symbol 'e 'p and option (a_symbol 'e 'p)
+  | ASlist of loc and lmin_len and a_symbol 'e 'p and
+      option (a_symbol 'e 'p * bool)
   | ASnext of loc
   | ASnterm of loc and (string * 'e) and option string
   | ASopt of loc and a_symbol 'e 'p
@@ -67,7 +72,7 @@ type styp =
 type text 'e 'p =
   [ TXfacto of loc and text 'e 'p
   | TXmeta of loc and string and list (text 'e 'p) and 'e and styp
-  | TXlist of loc and lmin_len and text 'e 'p and option (text 'e 'p)
+  | TXlist of loc and lmin_len and text 'e 'p and option (text 'e 'p * bool)
   | TXnext of loc
   | TXnterm of loc and name 'e and option string
   | TXopt of loc and text 'e 'p
@@ -233,6 +238,8 @@ module MetaAction =
           <:expr< MLast.ExAcc $mloc$ $mexpr e1$ $mexpr e2$ >>
       | MLast.ExApp loc e1 e2 ->
           <:expr< MLast.ExApp $mloc$ $mexpr e1$ $mexpr e2$ >>
+      | MLast.ExAsr loc e ->
+          <:expr< MLast.ExAsr $mloc$ $mexpr e$ >>
       | MLast.ExChr loc s -> <:expr< MLast.ExChr $mloc$ $mvala mstring s$ >>
       | MLast.ExFun loc pwel ->
           <:expr< MLast.ExFun $mloc$ $mvala (mlist mpwe) pwel$ >>
@@ -368,6 +375,10 @@ value anti_of_tok =
   | s -> [] ]
 ;
 
+value is_not_translated_function f =
+  f = "warning_deprecated_since_6_00"
+;
+
 value quot_expr psl e =
   loop e where rec loop e =
     let loc = MLast.loc_of_expr e in
@@ -390,7 +401,7 @@ value quot_expr psl e =
     | <:expr< Qast.VaVal $_$ >> -> e
     | <:expr< [] >> -> <:expr< Qast.List [] >>
     | <:expr< [$e$] >> -> <:expr< Qast.List [$loop e$] >>
-    | <:expr< [$e1$ :: $e2$] >> -> <:expr< Qast.Cons $loop e1$  $loop e2$ >>
+    | <:expr< [$e1$ :: $e2$] >> -> <:expr< Qast.Cons $loop e1$ $loop e2$ >>
     | <:expr< $_$ $_$ >> ->
         let (f, al) = expr_fa [] e in
         match f with
@@ -404,9 +415,11 @@ value quot_expr psl e =
             let al = List.map loop al in
             <:expr< Qast.Node $str:m ^ "." ^ c$ $mklistexp loc al$ >>
         | <:expr< $lid:f$ >> ->
-            let al = List.map loop al in
-            List.fold_left (fun f e -> <:expr< $f$ $e$ >>) <:expr< $lid:f$ >>
-              al
+            if is_not_translated_function f then e
+            else
+              let al = List.map loop al in
+              List.fold_left (fun f e -> <:expr< $f$ $e$ >>)
+                <:expr< $lid:f$ >> al
         | _ -> e ]
     | <:expr< {$list:pel$} >> ->
         try
@@ -425,7 +438,8 @@ value quot_expr psl e =
           <:expr< Qast.Record $mklistexp loc lel$>>
         with
         [ Not_found -> e ]
-    | <:expr< $lid:s$ >> -> if s = Ploc.name.val then <:expr< Qast.Loc >> else e
+    | <:expr< $lid:s$ >> ->
+        if s = Ploc.name.val then <:expr< Qast.Loc >> else e
     | <:expr< MLast.$uid:s$ >> -> <:expr< Qast.Node $str:s$ [] >>
     | <:expr< $uid:m$.$uid:s$ >> -> <:expr< Qast.Node $str:m ^ "." ^ s$ [] >>
     | <:expr< $uid:s$ >> -> <:expr< Qast.Node $str:s$ [] >>
@@ -549,12 +563,14 @@ value rec make_expr gmod tvar =
       match (min, ts) with
       [ (LML_0, None) -> <:expr< Gramext.Slist0 $txt$ >>
       | (LML_1, None) -> <:expr< Gramext.Slist1 $txt$ >>
-      | (LML_0, Some s) ->
+      | (LML_0, Some (s, b)) ->
           let x = make_expr gmod tvar s in
-          <:expr< Gramext.Slist0sep $txt$ $x$ >>
-      | (LML_1, Some s) ->
+          let b = if b then <:expr< True >> else <:expr< False >> in
+          <:expr< Gramext.Slist0sep $txt$ $x$ $b$ >>
+      | (LML_1, Some (s, b)) ->
           let x = make_expr gmod tvar s in
-          <:expr< Gramext.Slist1sep $txt$ $x$ >> ]
+          let b = if b then <:expr< True >> else <:expr< False >> in
+          <:expr< Gramext.Slist1sep $txt$ $x$ $b$ >> ]
   | TXnext loc -> <:expr< Gramext.Snext >>
   | TXnterm loc n lev ->
       match lev with
@@ -612,7 +628,7 @@ value mk_name2 (i, e) =
 value slist loc min sep symb =
   let t =
     match sep with
-    [ Some s -> Some s.text
+    [ Some (s, b) -> Some (s.text, b)
     | None -> None ]
   in
   TXlist loc min symb.text t
@@ -745,10 +761,10 @@ value rec symbol_of_a =
       {used = []; text = text; styp = STlid loc "string"}
   | ASlist loc min s sep ->
       let s = symbol_of_a s in
-      let sep = option_map symbol_of_a sep in
+      let sep = option_map (fun (sep, b) -> (symbol_of_a sep, b)) sep in
       let used =
         match sep with
-        [ Some symb -> symb.used @ s.used
+        [ Some (symb, _) -> symb.used @ s.used
         | None -> s.used ]
       in
       let text = slist loc min sep s in
@@ -868,6 +884,7 @@ value text_of_entry loc gmod e =
     let loc = e.name.loc in
     <:expr< ($x.expr$ : $uid:gmod$.Entry.e '$x.tvar$) >>
   in
+  let loc = Ploc.with_comment loc "" in
   let pos =
     match e.pos with
     [ Some pos -> <:expr< Some $pos$ >>
@@ -955,8 +972,10 @@ value let_in_of_extend loc gmod functor_version gl el args =
 value text_of_extend loc gmod gl el f =
   let el = List.map entry_of_a el in
   let gl = option_map (List.map mk_name2) gl in
+  let iloc = Ploc.with_comment loc "" in
   if split_ext.val then
     let args =
+      let loc = iloc in
       List.map
         (fun e ->
            let (ent, pos, txt) = text_of_entry e.name.loc gmod e in
@@ -965,19 +984,26 @@ value text_of_extend loc gmod gl el f =
            <:expr< let aux () = $f$ [$e$] in aux () >>)
         el
     in
-    let args = <:expr< do { $list:args$ } >> in
+    let args =
+      let loc = iloc in
+      <:expr< do { $list:args$ } >>
+    in
     let_in_of_extend loc gmod False gl el args
   else
     let args =
+      let loc = iloc in
       List.fold_right
         (fun e el ->
            let (ent, pos, txt) = text_of_entry e.name.loc gmod e in
            let ent = <:expr< $uid:gmod$.Entry.obj $ent$ >> in
-           let e = <:expr< ($ent$, $pos$, $txt$) >> in
+           let e =
+             let loc = e.name.loc in
+             <:expr< ($ent$, $pos$, $txt$) >>
+           in
            <:expr< [$e$ :: $el$] >>)
         el <:expr< [] >>
     in
-    let args = let_in_of_extend loc gmod False gl el args in
+    let args = let_in_of_extend iloc gmod False gl el args in
     <:expr< $f$ $args$ >>
 ;
 
@@ -1094,13 +1120,15 @@ EXTEND
       | s = symbol ->
           {ap_loc = loc; ap_patt = None; ap_symb = s} ] ]
   ;
+  sep_opt_sep:
+    [ [ sep = UIDENT "SEP"; t = symbol; b = FLAG [ UIDENT "OPT_SEP" ] ->
+          (t, b) ] ]
+  ;
   symbol:
     [ "top" NONA
-      [ UIDENT "LIST0"; s = SELF;
-        sep = OPT [ UIDENT "SEP"; t = symbol -> t ] ->
+      [ UIDENT "LIST0"; s = SELF; sep = OPT sep_opt_sep ->
           ASlist loc LML_0 s sep
-      | UIDENT "LIST1"; s = SELF;
-        sep = OPT [ UIDENT "SEP"; t = symbol -> t ] ->
+      | UIDENT "LIST1"; s = SELF; sep = OPT sep_opt_sep ->
           ASlist loc LML_1 s sep
       | UIDENT "OPT"; s = SELF ->
           ASopt loc s
