@@ -1,7 +1,10 @@
-(* camlp5r pa_extend.cmo q_MLast.cmo *)
-(* $Id: pa_extend_m.ml,v 1.34 2010/02/19 09:06:38 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pa_extend_m.ml,v 6.2 2010-09-19 09:56:37 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_extend.cmo";
+#load "q_MLast.cmo";
+
 open Pa_extend;
 
 EXTEND
@@ -9,10 +12,10 @@ EXTEND
   symbol: LEVEL "top"
     [ NONA
       [ UIDENT "SLIST0"; s = SELF;
-        sep = OPT [ UIDENT "SEP"; t = symbol -> t ] ->
+        sep = OPT [ UIDENT "SEP"; t = symbol -> (t, False) ] ->
           ASquot loc (ASlist loc LML_0 s sep)
       | UIDENT "SLIST1"; s = SELF;
-        sep = OPT [ UIDENT "SEP"; t = symbol -> t ] ->
+        sep = OPT [ UIDENT "SEP"; t = symbol -> (t, False) ] ->
           ASquot loc (ASlist loc LML_1 s sep)
       | UIDENT "SOPT"; s = SELF ->
           ASquot loc (ASopt loc s)
@@ -33,6 +36,6 @@ EXTEND
           ASvala2 loc s al oe ] ]
   ;
   name:
-    [ [ i = LIDENT  -> (i, <:expr< $lid:i$ >>) ] ]
+    [ [ i = LIDENT -> (i, <:expr< $lid:i$ >>) ] ]
   ;
 END;
