@@ -1,7 +1,10 @@
-(* camlp5r pa_extend.cmo q_MLast.cmo *)
-(* $Id: pa_r.ml,v 1.121 2010/02/19 09:06:38 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pa_r.ml,v 6.35 2011-02-17 10:14:09 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_extend.cmo";
+#load "q_MLast.cmo";
+
 open Pcaml;
 
 Pcaml.syntax_name.val := "Revised";
@@ -10,6 +13,7 @@ Pcaml.no_constructors_arity.val := False;
 do {
   let odfa = Plexer.dollar_for_antiquotation.val in
   Plexer.dollar_for_antiquotation.val := False;
+  Plexer.utf8_lexing.val := True;
   Grammar.Unsafe.gram_reinit gram (Plexer.gmake ());
   Plexer.dollar_for_antiquotation.val := odfa;
   Grammar.Unsafe.clear_entry interf;
@@ -20,12 +24,14 @@ do {
   Grammar.Unsafe.clear_entry module_expr;
   Grammar.Unsafe.clear_entry sig_item;
   Grammar.Unsafe.clear_entry str_item;
+  Grammar.Unsafe.clear_entry signature;
+  Grammar.Unsafe.clear_entry structure;
   Grammar.Unsafe.clear_entry expr;
   Grammar.Unsafe.clear_entry patt;
   Grammar.Unsafe.clear_entry ipatt;
   Grammar.Unsafe.clear_entry ctyp;
   Grammar.Unsafe.clear_entry let_binding;
-  Grammar.Unsafe.clear_entry type_declaration;
+  Grammar.Unsafe.clear_entry type_decl;
   Grammar.Unsafe.clear_entry constructor_declaration;
   Grammar.Unsafe.clear_entry label_declaration;
   Grammar.Unsafe.clear_entry match_case;
@@ -40,6 +46,10 @@ do {
 Pcaml.parse_interf.val := Grammar.Entry.parse interf;
 Pcaml.parse_implem.val := Grammar.Entry.parse implem;
 
+Pcaml.add_option "-ignloaddir"
+  (Arg.Unit (fun _ -> add_directive "load" (fun _ -> ())))
+  "Ignore the #load directives in the input file.";
+
 value mksequence2 loc =
   fun
   [ <:vala< [e] >> -> e
@@ -67,20 +77,6 @@ value neg_string n =
   else "-" ^ n
 ;
 
-value mkumin loc f arg =
-  match arg with
-  [ <:expr< $int:n$ >> -> <:expr< $int:neg_string n$ >>
-  | <:expr< $flo:n$ >> -> <:expr< $flo:neg_string n$ >>
-  | _ ->
-      let f = "~" ^ f in
-      <:expr< $lid:f$ $arg$ >> ]
-;
-
-value mkuminpat loc f is_int n =
-  if is_int then <:patt< $int:neg_string n$ >>
-  else <:patt< $flo:neg_string n$ >>
-;
-
 value mklistexp loc last =
   loop True where rec loop top =
     fun
@@ -114,33 +110,100 @@ value mktuppat loc p pl = <:patt< ($list:[p::pl]$) >>;
 value mktuptyp loc t tl = <:ctyp< ( $list:[t::tl]$ ) >>;
 
 value mklabdecl loc i mf t = (loc, i, mf, t);
-
 value mkident i : string = i;
 
+value rec generalized_type_of_type =
+  fun
+  [ <:ctyp< $t1$ -> $t2$ >> ->
+      let (tl, rt) = generalized_type_of_type t2 in
+      ([t1 :: tl], rt)
+  | t ->
+      ([], t) ]
+;
+
+value start_with s s_ini =
+  let len = String.length s_ini in
+  String.length s >= len && String.sub s 0 len = s_ini
+;
+
+value greek_tab =
+  ["α"; "β"; "γ"; "δ"; "ε"; "ζ"; "η"; "θ"; "ι"; "κ"; "λ"; "μ"; "ν"; "ξ";
+   "ο"; "π"; "ρ"; "σ"; "τ"; "υ"; "φ"; "χ"; "ψ"; "ω"]
+;
+value index_tab = [""; "₁"; "₂"; "₃"; "₄"; "₅"; "₆"; "₇"; "₈"; "₉"];
+value ascii_of_greek s =
+  loop 0 greek_tab where rec loop i =
+    fun
+    [ [g :: gl] -> do {
+        if start_with s g then do {
+          let c1 = Char.chr (Char.code 'a' + i) in
+          let glen = String.length g in
+          let rest = String.sub s glen (String.length s - glen) in
+          loop 0 index_tab where rec loop i =
+            fun
+            [ [k :: kl] -> do {
+                if rest = k then do {
+                  let s2 = if i = 0 then "" else string_of_int i in
+                  String.make 1 c1 ^ s2
+                }
+                else loop (i + 1) kl
+              }
+            | [] -> s ]
+        }
+        else loop (i + 1) gl
+      }
+    | [] -> s ]
+;
+
+(* should be added in lib/plexer.ml, perhaps, as a new token GREEK? *)
+value greek_token =
+  Grammar.Entry.of_parser gram "greek_token"
+    (parser
+       [: `("LIDENT", x) when List.exists (start_with x) greek_tab :] ->
+          ascii_of_greek x)
+;
+
+value warned = ref False;
+value warning_deprecated_since_6_00 loc =
+  if not warned.val then do {
+    Pcaml.warning.val loc "syntax deprecated since version 6.00";
+    warned.val := True
+  }
+  else ()
+;
+
 EXTEND
-  GLOBAL: sig_item str_item ctyp patt expr module_type module_expr class_type
-    class_expr class_sig_item class_str_item let_binding type_declaration
-    constructor_declaration label_declaration match_case ipatt with_constr
-    poly_variant;
+  GLOBAL: sig_item str_item ctyp patt expr module_type module_expr
+    signature structure class_type class_expr class_sig_item class_str_item
+    let_binding type_decl constructor_declaration label_declaration
+    match_case ipatt with_constr poly_variant;
   module_expr:
     [ [ "functor"; "("; i = V UIDENT "uid" ""; ":"; t = module_type; ")";
         "->"; me = SELF ->
           <:module_expr< functor ( $_uid:i$ : $t$ ) -> $me$ >>
-      | "struct"; st = V (LIST0 [ s = str_item; ";" -> s ]); "end" ->
+      | "struct"; st = structure; "end" ->
           <:module_expr< struct $_list:st$ end >> ]
     | [ me1 = SELF; me2 = SELF -> <:module_expr< $me1$ $me2$ >> ]
     | [ me1 = SELF; "."; me2 = SELF -> <:module_expr< $me1$ . $me2$ >> ]
     | "simple"
       [ i = V UIDENT -> <:module_expr< $_uid:i$ >>
+      | "("; "value"; e = expr; ":"; mt = module_type; ")" ->
+         <:module_expr< (value $e$ : $mt$) >>
+      | "("; "value"; e = expr; ")" ->
+         <:module_expr< (value $e$) >>
       | "("; me = SELF; ":"; mt = module_type; ")" ->
           <:module_expr< ( $me$ : $mt$ ) >>
       | "("; me = SELF; ")" -> <:module_expr< $me$ >> ] ]
   ;
+  structure:
+    [ [ st = V (LIST0 [ s = str_item; ";" -> s ]) -> st ] ]
+  ;
   str_item:
     [ "top"
       [ "declare"; st = V (LIST0 [ s = str_item; ";" -> s ]); "end" ->
           <:str_item< declare $_list:st$ end >>
-      | "exception"; (_, c, tl) = constructor_declaration; b = rebind_exn ->
+      | "exception"; (_, c, tl, _) = constructor_declaration;
+        b = rebind_exn ->
           <:str_item< exception $_uid:c$ of $_list:tl$ = $_:b$ >>
       | "external"; i = V LIDENT "lid" ""; ":"; t = ctyp; "=";
         pd = V (LIST1 STRING) ->
@@ -148,15 +211,17 @@ EXTEND
       | "include"; me = module_expr -> <:str_item< include $me$ >>
       | "module"; r = V (FLAG "rec"); l = V (LIST1 mod_binding SEP "and") ->
           <:str_item< module $_flag:r$ $_list:l$ >>
-      | "module"; "type"; i = V UIDENT "uid" ""; "="; mt = module_type ->
+      | "module"; "type"; i = V UIDENT "uid" ""; mt = mod_type_fun_binding ->
           <:str_item< module type $_uid:i$ = $mt$ >>
       | "open"; i = V mod_ident "list" "" -> <:str_item< open $_:i$ >>
-      | "type"; tdl = V (LIST1 type_declaration SEP "and") ->
+      | "type"; tdl = V (LIST1 type_decl SEP "and") ->
           <:str_item< type $_list:tdl$ >>
       | "value"; r = V (FLAG "rec"); l = V (LIST1 let_binding SEP "and") ->
           <:str_item< value $_flag:r$ $_list:l$ >>
       | "#"; n = V LIDENT "lid" ""; dp = V (OPT expr) ->
           <:str_item< # $_lid:n$ $_opt:dp$ >>
+      | "#"; s = V STRING; sil = V (LIST0 [ si = str_item -> (si, loc) ]) ->
+          <:str_item< # $_str:s$ $_list:sil$ >>
       | e = expr -> <:str_item< $exp:e$ >> ] ]
   ;
   rebind_exn:
@@ -174,14 +239,22 @@ EXTEND
           <:module_expr< ( $me$ : $mt$ ) >>
       | "="; me = module_expr -> <:module_expr< $me$ >> ] ]
   ;
+  mod_type_fun_binding:
+    [ [ "("; m = V UIDENT; ":"; mt1 = module_type; ")"; mt2 = SELF ->
+          <:module_type< functor ( $_uid:m$ : $mt1$ ) -> $mt2$ >>
+      | "="; mt = module_type ->
+          <:module_type< $mt$ >> ] ]
+  ;
   module_type:
     [ [ "functor"; "("; i = V UIDENT "uid" ""; ":"; t = SELF; ")"; "->";
         mt = SELF ->
           <:module_type< functor ( $_uid:i$ : $t$ ) -> $mt$ >> ]
     | [ mt = SELF; "with"; wcl = V (LIST1 with_constr SEP "and") ->
           <:module_type< $mt$ with $_list:wcl$ >> ]
-    | [ "sig"; sg = V (LIST0 [ s = sig_item; ";" -> s ]); "end" ->
-          <:module_type< sig $_list:sg$ end >> ]
+    | [ "sig"; sg = signature; "end" ->
+          <:module_type< sig $_list:sg$ end >>
+      | "module"; "type"; "of"; me = module_expr ->
+          <:module_type< module type of $me$ >> ]
     | [ m1 = SELF; m2 = SELF -> <:module_type< $m1$ $m2$ >> ]
     | [ m1 = SELF; "."; m2 = SELF -> <:module_type< $m1$ . $m2$ >> ]
     | "simple"
@@ -190,11 +263,14 @@ EXTEND
       | "'"; i = V ident "" -> <:module_type< ' $_:i$ >>
       | "("; mt = SELF; ")" -> <:module_type< $mt$ >> ] ]
   ;
+  signature:
+    [ [ st = V (LIST0 [ s = sig_item; ";" -> s ]) -> st ] ]
+  ;
   sig_item:
     [ "top"
       [ "declare"; st = V (LIST0 [ s = sig_item; ";" -> s ]); "end" ->
           <:sig_item< declare $_list:st$ end >>
-      | "exception"; (_, c, tl) = constructor_declaration ->
+      | "exception"; (_, c, tl, _) = constructor_declaration ->
           <:sig_item< exception $_uid:c$ of $_list:tl$ >>
       | "external"; i = V LIDENT "lid" ""; ":"; t = ctyp; "=";
         pd = V (LIST1 STRING) ->
@@ -206,12 +282,14 @@ EXTEND
       | "module"; "type"; i = V UIDENT "uid" ""; "="; mt = module_type ->
           <:sig_item< module type $_uid:i$ = $mt$ >>
       | "open"; i = V mod_ident "list" "" -> <:sig_item< open $_:i$ >>
-      | "type"; tdl = V (LIST1 type_declaration SEP "and") ->
+      | "type"; tdl = V (LIST1 type_decl SEP "and") ->
           <:sig_item< type $_list:tdl$ >>
       | "value"; i = V LIDENT "lid" ""; ":"; t = ctyp ->
           <:sig_item< value $_lid:i$ : $t$ >>
       | "#"; n = V LIDENT "lid" ""; dp = V (OPT expr) ->
-          <:sig_item< # $_lid:n$ $_opt:dp$ >> ] ]
+          <:sig_item< # $_lid:n$ $_opt:dp$ >>
+      | "#"; s = V STRING; sil = V (LIST0 [ si = sig_item -> (si, loc) ]) ->
+          <:sig_item< # $_str:s$ $_list:sil$ >> ] ]
   ;
   mod_decl_binding:
     [ [ i = V UIDENT; mt = module_declaration -> (i, mt) ] ]
@@ -226,8 +304,13 @@ EXTEND
     [ [ "type"; i = V mod_ident "list" ""; tpl = V (LIST0 type_parameter);
         "="; pf = V (FLAG "private"); t = ctyp ->
           <:with_constr< type $_:i$ $_list:tpl$ = $_flag:pf$ $t$ >>
+      | "type"; i = V mod_ident "list" ""; tpl = V (LIST0 type_parameter);
+        ":="; t = ctyp ->
+          <:with_constr< type $_:i$ $_list:tpl$ := $t$ >>
       | "module"; i = V mod_ident "list" ""; "="; me = module_expr ->
-          <:with_constr< module $_:i$ = $me$ >> ] ]
+          <:with_constr< module $_:i$ = $me$ >>
+      | "module"; i = V mod_ident "list" ""; ":="; me = module_expr ->
+          <:with_constr< module $_:i$ := $me$ >> ] ]
   ;
   expr:
     [ "top" RIGHTA
@@ -246,8 +329,8 @@ EXTEND
           <:expr< match $e$ with $p1$ -> $e1$ >>
       | "try"; e = SELF; "with"; "["; l = V (LIST0 match_case SEP "|"); "]" ->
           <:expr< try $e$ with [ $_list:l$ ] >>
-      | "try"; e = SELF; "with"; p1 = ipatt; "->"; e1 = SELF ->
-          <:expr< try $e$ with $p1$ -> $e1$ >>
+      | "try"; e = SELF; "with"; mc = match_case ->
+          <:expr< try $e$ with [ $list:[mc]$ ] >>
       | "if"; e1 = SELF; "then"; e2 = SELF; "else"; e3 = SELF ->
           <:expr< if $e1$ then $e2$ else $e3$ >>
       | "do"; "{"; seq = V sequence "list"; "}" -> mksequence2 loc seq
@@ -297,8 +380,8 @@ EXTEND
       | e1 = SELF; "lsl"; e2 = SELF -> <:expr< $e1$ lsl $e2$ >>
       | e1 = SELF; "lsr"; e2 = SELF -> <:expr< $e1$ lsr $e2$ >> ]
     | "unary minus" NONA
-      [ "-"; e = SELF -> mkumin loc "-" e
-      | "-."; e = SELF -> mkumin loc "-." e ]
+      [ "-"; e = SELF -> <:expr< - $e$ >>
+      | "-."; e = SELF -> <:expr< -. $e$ >> ]
     | "apply" LEFTA
       [ e1 = SELF; e2 = SELF -> <:expr< $e1$ $e2$ >>
       | "assert"; e = SELF -> <:expr< assert $e$ >>
@@ -332,6 +415,10 @@ EXTEND
         "}" ->
           <:expr< { ($e$) with $_list:lel$ } >>
       | "("; ")" -> <:expr< () >>
+      | "("; "module"; me = module_expr; ":"; mt = module_type; ")" ->
+          <:expr< (module $me$ : $mt$) >>
+      | "("; "module"; me = module_expr; ")" ->
+          <:expr< (module $me$) >>
       | "("; e = SELF; ":"; t = ctyp; ")" -> <:expr< ($e$ : $t$) >>
       | "("; e = SELF; ","; el = LIST1 expr SEP ","; ")" -> mktupexp loc e el
       | "("; e = SELF; ")" -> <:expr< $e$ >>
@@ -349,6 +436,9 @@ EXTEND
       [ "let"; rf = V (FLAG "rec"); l = V (LIST1 let_binding SEP "and");
         "in"; el = SELF ->
           [<:expr< let $_flag:rf$ $_list:l$ in $mksequence loc el$ >>]
+      | "let"; "module"; m = V UIDENT; mb = mod_fun_binding; "in";
+        el = SELF ->
+          [<:expr< let module $_uid:m$ = $mb$ in $mksequence loc el$ >>]
       | e = expr; ";"; el = SELF -> [e :: el]
       | e = expr; ";" -> [e]
       | e = expr -> [e] ] ]
@@ -401,8 +491,11 @@ EXTEND
       | s = V FLOAT -> <:patt< $_flo:s$ >>
       | s = V STRING -> <:patt< $_str:s$ >>
       | s = V CHAR -> <:patt< $_chr:s$ >>
-      | "-"; s = INT -> mkuminpat loc "-" True s
-      | "-"; s = FLOAT -> mkuminpat loc "-" False s
+      | "-"; s = INT -> <:patt< $int:neg_string s$ >>
+      | "-"; s = INT_l -> <:patt< $int32:neg_string s$ >>
+      | "-"; s = INT_L -> <:patt< $int64:neg_string s$ >>
+      | "-"; s = INT_n -> <:patt< $nativeint:neg_string s$ >>
+      | "-"; s = FLOAT -> <:patt< $flo:neg_string s$ >>
       | "["; "]" -> <:patt< [] >>
       | "["; pl = LIST1 patt SEP ";"; last = cons_patt_opt; "]" ->
           mklistpat loc last pl
@@ -418,6 +511,11 @@ EXTEND
       | p = patt; ","; pl = LIST1 patt SEP "," -> mktuppat loc p pl
       | p = patt -> <:patt< $p$ >>
       | pl = V (LIST1 patt SEP ",") -> <:patt< ($_list:pl$) >>
+      | "type"; s = V LIDENT -> <:patt< (type $_lid:s$) >>
+      | "module"; s = V UIDENT; ":"; mt = module_type ->
+          <:patt< (module $_uid:s$ : $mt$) >>
+      | "module"; s = V UIDENT ->
+          <:patt< (module $_uid:s$) >>
       | -> <:patt< () >> ] ]
   ;
   cons_patt_opt:
@@ -447,16 +545,20 @@ EXTEND
       | p = ipatt; ","; pl = LIST1 ipatt SEP "," -> mktuppat loc p pl
       | p = ipatt -> <:patt< $p$ >>
       | pl = V (LIST1 ipatt SEP ",") -> <:patt< ( $_list:pl$) >>
+      | "type"; s = V LIDENT -> <:patt< (type $_lid:s$) >>
+      | "module"; s  = V UIDENT; ":"; mt = module_type ->
+          <:patt< (module $_uid:s$ : $mt$) >>
+      | "module"; s = V UIDENT ->
+          <:patt< (module $_uid:s$) >>
       | -> <:patt< () >> ] ]
   ;
   label_ipatt:
     [ [ i = patt_label_ident; "="; p = ipatt -> (i, p) ] ]
   ;
-  type_declaration:
-    [ [ n = type_patt; tpl = V (LIST0 type_parameter); "=";
-        pf = V (FLAG "private"); tk = ctyp; cl = V (LIST0 constrain) ->
-          {MLast.tdNam = n; MLast.tdPrm = tpl; MLast.tdPrv = pf;
-           MLast.tdDef = tk; MLast.tdCon = cl} ] ]
+  type_decl:
+    [ [ n = V type_patt "tp"; tpl = V (LIST0 type_parameter); "=";
+        pf = V (FLAG "private") "priv"; tk = ctyp; cl = V (LIST0 constrain) ->
+          <:type_decl< $_tp:n$ $_list:tpl$ = $_priv:pf$ $tk$ $_list:cl$ >> ] ]
   ;
   type_patt:
     [ [ n = V LIDENT -> (loc, n) ] ]
@@ -465,20 +567,28 @@ EXTEND
     [ [ "constraint"; t1 = ctyp; "="; t2 = ctyp -> (t1, t2) ] ]
   ;
   type_parameter:
-    [ [ "'"; i = V ident "" -> (i, (False, False))
-      | "+"; "'"; i = V ident "" -> (i, (True, False))
-      | "-"; "'"; i = V ident "" -> (i, (False, True)) ] ]
+    [ [ "+"; p = V simple_type_parameter -> (p, Some True)
+      | "-"; p = V simple_type_parameter -> (p, Some False)
+      | p = V simple_type_parameter -> (p, None) ] ]
+  ;
+  simple_type_parameter:
+    [ [ "'"; i = ident -> Some i
+      | i = greek_token -> Some i
+      | "_" -> None ] ]
   ;
   ctyp:
     [ "top"
       LEFTA
-      [ t1 = SELF; "=="; t2 = SELF -> <:ctyp< $t1$ == $t2$ >> ]
+      [ t1 = SELF; "=="; pf = V (FLAG "private") "priv"; t2 = SELF ->
+          <:ctyp< $t1$ == $_priv:pf$ $t2$ >> ]
     | "as"
       LEFTA
       [ t1 = SELF; "as"; t2 = SELF -> <:ctyp< $t1$ as $t2$ >> ]
     | LEFTA
       [ "!"; pl = V (LIST1 typevar); "."; t = ctyp ->
-          <:ctyp< ! $_list:pl$ . $t$ >> ]
+          <:ctyp< ! $_list:pl$ . $t$ >>
+      | "type"; pl = V (LIST1 LIDENT); "."; t = ctyp ->
+          <:ctyp< type $_list:pl$ . $t$ >> ]
     | "arrow" RIGHTA
       [ t1 = SELF; "->"; t2 = SELF -> <:ctyp< $t1$ -> $t2$ >> ]
     | "apply" LEFTA
@@ -487,9 +597,11 @@ EXTEND
       [ t1 = SELF; "."; t2 = SELF -> <:ctyp< $t1$ . $t2$ >> ]
     | "simple"
       [ "'"; i = V ident "" -> <:ctyp< '$_:i$ >>
+      | i = greek_token -> <:ctyp< '$i$ >>
       | "_" -> <:ctyp< _ >>
       | i = V LIDENT -> <:ctyp< $_lid:i$ >>
       | i = V UIDENT -> <:ctyp< $_uid:i$ >>
+      | "module"; mt = module_type -> <:ctyp< module $mt$ >>
       | "("; t = SELF; "*"; tl = LIST1 ctyp SEP "*"; ")" -> mktuptyp loc t tl
       | "("; t = SELF; ")" -> <:ctyp< $t$ >>
       | "("; tl = V (LIST1 ctyp SEP "*"); ")" -> <:ctyp< ( $_list:tl$ ) >>
@@ -500,9 +612,12 @@ EXTEND
   ;
   constructor_declaration:
     [ [ ci = V UIDENT "uid" ""; "of"; cal = V (LIST1 ctyp SEP "and") ->
-          (loc, ci, cal)
+          (loc, ci, cal, None)
+      | ci = V UIDENT "uid" ""; ":"; t = ctyp ->
+          let (tl, rt) = generalized_type_of_type t in
+          (loc, ci, <:vala< tl >>, Some rt)
       | ci = V UIDENT "uid" "" ->
-          (loc, ci, <:vala< [] >>) ] ]
+          (loc, ci, <:vala< [] >>, None) ] ]
   ;
   label_declaration:
     [ [ i = LIDENT; ":"; mf = FLAG "mutable"; t = ctyp ->
@@ -562,14 +677,14 @@ EXTEND
       [ ce = SELF; e = expr LEVEL "label" ->
           <:class_expr< $ce$ $e$ >> ]
     | "simple"
-      [ ci = V class_longident "list"; "["; ctcl = V (LIST1 ctyp SEP ",");
-        "]" ->
-          <:class_expr< $_list:ci$ [ $_list:ctcl$ ] >>
-      | ci = V class_longident "list" ->
+      [ ci = V class_longident "list" ->
           <:class_expr< $_list:ci$ >>
       | "object"; cspo = V (OPT class_self_patt); cf = class_structure;
         "end" ->
           <:class_expr< object $_opt:cspo$ $_list:cf$ end >>
+      | "["; ctcl = V (LIST1 ctyp SEP ","); "]";
+        ci = V class_longident "list" ->
+          <:class_expr< [ $_list:ctcl$ ] $_list:ci$ >>
       | "("; ce = SELF; ":"; ct = class_type; ")" ->
           <:class_expr< ($ce$ : $ct$) >>
       | "("; ce = SELF; ")" -> ce ] ]
@@ -586,15 +701,19 @@ EXTEND
           <:class_str_item< declare $_list:st$ end >>
       | "inherit"; ce = class_expr; pb = V (OPT as_lident) ->
           <:class_str_item< inherit $ce$ $_opt:pb$ >>
-      | "value"; mf = V (FLAG "mutable"); lab = V label "lid" "";
-        e = cvalue_binding ->
-          <:class_str_item< value $_flag:mf$ $_lid:lab$ = $e$ >>
-      | "method"; "virtual"; pf = V (FLAG "private"); l = V label "lid" "";
+      | "value"; ovf = V (FLAG "!") "!"; mf = V (FLAG "mutable");
+        lab = V lident "lid" ""; e = cvalue_binding ->
+          <:class_str_item< value $_!:ovf$ $_flag:mf$ $_lid:lab$ = $e$ >>
+      | "value"; "virtual"; mf = V (FLAG "mutable");
+        lab = V lident "lid" ""; ":"; t = ctyp ->
+          <:class_str_item< value virtual $_flag:mf$ $_lid:lab$ : $t$ >>
+      | "method"; "virtual"; pf = V (FLAG "private"); l = V lident "lid" "";
         ":"; t = ctyp ->
           <:class_str_item< method virtual $_flag:pf$ $_lid:l$ : $t$ >>
-      | "method"; pf = V (FLAG "private"); l = V label "lid" "";
-        topt = V (OPT polyt); e = fun_binding ->
-          <:class_str_item< method $_flag:pf$ $_lid:l$ $_opt:topt$ = $e$ >>
+      | "method"; ovf = V (FLAG "!") "!"; pf = V (FLAG "private") "priv";
+        l = V lident "lid" ""; topt = V (OPT polyt); e = fun_binding ->
+          <:class_str_item<
+            method $_!:ovf$ $_priv:pf$ $_lid:l$ $_opt:topt$ = $e$ >>
       | "type"; t1 = ctyp; "="; t2 = ctyp ->
           <:class_str_item< type $t1$ = $t2$ >>
       | "initializer"; se = expr -> <:class_str_item< initializer $se$ >> ] ]
@@ -612,18 +731,26 @@ EXTEND
           <:expr< ($e$ : $t$ :> $t2$) >>
       | ":>"; t = ctyp; "="; e = expr -> <:expr< ($e$ :> $t$) >> ] ]
   ;
-  label:
+  lident:
     [ [ i = LIDENT -> mkident i ] ]
   ;
   class_type:
-    [ [ "["; t = ctyp; "]"; "->"; ct = SELF ->
+    [ "top" RIGHTA
+      [ "["; t = ctyp; "]"; "->"; ct = SELF ->
           <:class_type< [ $t$ ] -> $ct$ >>
-      | id = V clty_longident "list"; "["; tl = V (LIST1 ctyp SEP ","); "]" ->
-          <:class_type< $_list:id$ [ $_list:tl$ ] >>
-      | id = V clty_longident "list" -> <:class_type< $_list:id$ >>
       | "object"; cst = V (OPT class_self_type);
         csf = V (LIST0 [ csf = class_sig_item; ";" -> csf ]); "end" ->
-          <:class_type< object $_opt:cst$ $_list:csf$ end >> ] ]
+          <:class_type< object $_opt:cst$ $_list:csf$ end >>
+      | ct = SELF; "["; tl = V (LIST1 ctyp SEP ","); "]" ->
+          <:class_type< $ct$ [ $_list:tl$ ] >> ]
+    | "apply"
+      [ ct1 = SELF; ct2 = SELF -> <:class_type< $ct1$ $ct2$ >> ]
+    | "dot"
+      [ ct1 = SELF; "."; ct2 = SELF -> <:class_type< $ct1$ . $ct2$ >> ]
+    | "simple"
+      [ i = V LIDENT "id" -> <:class_type< $_id:i$ >>
+      | i = V UIDENT "id" -> <:class_type< $_id:i$ >>
+      | "("; ct = SELF; ")" -> ct ] ]
   ;
   class_self_type:
     [ [ "("; t = ctyp; ")" -> t ] ]
@@ -632,13 +759,13 @@ EXTEND
     [ [ "declare"; st = V (LIST0 [ s = class_sig_item; ";" -> s ]); "end" ->
           <:class_sig_item< declare $_list:st$ end >>
       | "inherit"; cs = class_type -> <:class_sig_item< inherit $cs$ >>
-      | "value"; mf = V (FLAG "mutable"); l = V label "lid" ""; ":";
+      | "value"; mf = V (FLAG "mutable"); l = V lident "lid" ""; ":";
         t = ctyp ->
           <:class_sig_item< value $_flag:mf$ $_lid:l$ : $t$ >>
-      | "method"; "virtual"; pf = V (FLAG "private"); l = V label "lid" "";
+      | "method"; "virtual"; pf = V (FLAG "private"); l = V lident "lid" "";
         ":"; t = ctyp ->
           <:class_sig_item< method virtual $_flag:pf$ $_lid:l$ : $t$ >>
-      | "method"; pf = V (FLAG "private"); l = V label "lid" ""; ":";
+      | "method"; pf = V (FLAG "private"); l = V lident "lid" ""; ":";
         t = ctyp ->
           <:class_sig_item< method $_flag:pf$ $_lid:l$ : $t$ >>
       | "type"; t1 = ctyp; "="; t2 = ctyp ->
@@ -664,7 +791,7 @@ EXTEND
           <:expr< object $_opt:cspo$ $_list:cf$ end >> ] ]
   ;
   expr: LEVEL "."
-    [ [ e = SELF; "#"; lab = V label "lid" "" ->
+    [ [ e = SELF; "#"; lab = V lident "lid" "" ->
           <:expr< $e$ # $_lid:lab$ >> ] ]
   ;
   expr: LEVEL "simple"
@@ -675,7 +802,7 @@ EXTEND
           <:expr< {< $_list:fel$ >} >> ] ]
   ;
   field_expr:
-    [ [ l = label; "="; e = expr -> (l, e) ] ]
+    [ [ l = lident; "="; e = expr -> (l, e) ] ]
   ;
   ctyp: LEVEL "simple"
     [ [ "#"; id = V class_longident "list" -> <:ctyp< # $_list:id$ >>
@@ -688,10 +815,6 @@ EXTEND
   typevar:
     [ [ "'"; i = ident -> i ] ]
   ;
-  clty_longident:
-    [ [ m = UIDENT; "."; l = SELF -> [mkident m :: l]
-      | i = LIDENT -> [mkident i] ] ]
-  ;
   class_longident:
     [ [ m = UIDENT; "."; l = SELF -> [mkident m :: l]
       | i = LIDENT -> [mkident i] ] ]
@@ -729,44 +852,92 @@ EXTEND
   patt: LEVEL "simple"
     [ [ "`"; s = V ident "" -> <:patt< ` $_:s$ >>
       | "#"; sl = V mod_ident "list" "" -> <:patt< # $_:sl$ >>
-      | i = V TILDEIDENTCOLON; p = SELF -> <:patt< ~$_:i$: $p$ >>
-      | i = V TILDEIDENT -> <:patt< ~$_:i$ >>
-      | i = V QUESTIONIDENTCOLON; "("; p = patt_tcon; eo = V (OPT eq_expr);
-        ")" ->
-          <:patt< ?$_:i$: ($p$ $_opt:eo$) >>
-      | i = V QUESTIONIDENT ->
-          <:patt< ?$_:i$ >>
-      | "?"; "("; p = patt_tcon; eo = V (OPT eq_expr); ")" ->
-          <:patt< ? ($p$ $_opt:eo$) >> ] ]
+      | "~"; "{"; lppo = V (LIST1 patt_tcon_opt_eq_patt SEP ";"); "}" ->
+          <:patt< ~{$_list:lppo$} >>
+      | "?"; "{"; p = patt_tcon; eo = V (OPT [ "="; e = expr -> e ]); "}" ->
+          <:patt< ?{$p$ $_opt:eo$ } >>
+      | i = V TILDEIDENTCOLON; p = SELF ->
+          let _ = warning_deprecated_since_6_00 loc in
+          <:patt< ~{$_lid:i$ = $p$} >>
+      | i = V TILDEIDENT ->
+          let _ = warning_deprecated_since_6_00 loc in
+          <:patt< ~{$_lid:i$} >>
+      | p = patt_option_label ->
+          let _ = warning_deprecated_since_6_00 loc in
+          p ] ]
+  ;
+  patt_tcon_opt_eq_patt:
+    [ [ p = patt_tcon; po = V (OPT [ "="; p = patt -> p ]) -> (p, po) ] ]
   ;
   patt_tcon:
-    [ [ p = patt; ":"; t = ctyp -> <:patt< ($p$ : $t$) >>
-      | p = patt -> p ] ]
+    [ [ p = patt -> p
+      | p = patt; ":"; t = ctyp -> <:patt< ($p$ : $t$) >> ] ]
   ;
   ipatt:
-    [ [ i = V TILDEIDENTCOLON; p = SELF -> <:patt< ~$_:i$: $p$ >>
-      | i = V TILDEIDENT -> <:patt< ~$_:i$ >>
-      | i = V QUESTIONIDENTCOLON; "("; p = ipatt_tcon;
-        eo = V (OPT eq_expr); ")" ->
-          <:patt< ?$_:i$: ($p$ $_opt:eo$) >>
-      | i = V QUESTIONIDENT ->
-          <:patt< ?$_:i$ >>
-      | "?"; "("; p = ipatt_tcon; eo = V (OPT eq_expr); ")" ->
-          <:patt< ? ($p$ $_opt:eo$) >> ] ]
+    [ [ "~"; "{"; lppo = V (LIST1 ipatt_tcon_opt_eq_patt SEP ";"); "}" ->
+          <:patt< ~{$_list:lppo$} >>
+      | "?"; "{"; p = ipatt_tcon; eo = V (OPT [ "="; e = expr -> e ]); "}" ->
+          <:patt< ?{$p$ $_opt:eo$ } >>
+
+      | i = V TILDEIDENTCOLON; p = SELF ->
+          let _ = warning_deprecated_since_6_00 loc in
+          <:patt< ~{$_lid:i$ = $p$} >>
+      | i = V TILDEIDENT ->
+          let _ = warning_deprecated_since_6_00 loc in
+          <:patt< ~{$_lid:i$} >>
+      | p = patt_option_label ->
+          let _ = warning_deprecated_since_6_00 loc in
+          p ] ]
+  ;
+  ipatt_tcon_opt_eq_patt:
+    [ [ p = ipatt_tcon; po = V (OPT [ "="; p = patt -> p ]) -> (p, po) ] ]
   ;
   ipatt_tcon:
-    [ [ p = ipatt; ":"; t = ctyp -> <:patt< ($p$ : $t$) >>
-      | p = ipatt -> p ] ]
+    [ [ p = ipatt -> p
+      | p = ipatt; ":"; t = ctyp -> <:patt< ($p$ : $t$) >> ] ]
   ;
-  eq_expr:
-    [ [ "="; e = expr -> e ] ]
+  patt_option_label:
+    [ [ i = V QUESTIONIDENTCOLON; "("; j = V LIDENT; ":"; t = ctyp; "=";
+        e = expr; ")" ->
+          <:patt< ?{$_lid:i$ : $t$ = ?{$_lid:j$ = $e$}} >>
+      | i = V QUESTIONIDENTCOLON; "("; j = V LIDENT; ":"; t = ctyp; ")" ->
+          <:patt< ?{$_lid:i$ : $t$ = ?{$_lid:j$}} >>
+      | i = V QUESTIONIDENTCOLON; "("; j = V LIDENT; "="; e = expr; ")" ->
+          <:patt< ?{$_lid:i$ = ?{$_lid:j$ = $e$}} >>
+      | i = V QUESTIONIDENTCOLON; "("; j = V LIDENT; ")" ->
+          <:patt< ?{$_lid:i$ = $_lid:j$} >>
+      | i = V QUESTIONIDENT ->
+          <:patt< ?{$_lid:i$} >>
+      | "?"; "("; i = V LIDENT; ":"; t = ctyp; "="; e = expr; ")" ->
+          <:patt< ?{$_lid:i$ : $t$ = $e$} >>
+      | "?"; "("; i = V LIDENT; ":"; t = ctyp; ")" ->
+          <:patt< ?{$_lid:i$ : $t$} >>
+      | "?"; "("; i = V LIDENT; "="; e = expr; ")" ->
+          <:patt< ?{$_lid:i$ = $e$} >>
+      | "?"; "("; i = V LIDENT; ")" ->
+          <:patt< ?{$_lid:i$} >> ] ]
   ;
   expr: AFTER "apply"
     [ "label" NONA
-      [ i = V TILDEIDENTCOLON; e = SELF -> <:expr< ~$_:i$: $e$ >>
-      | i = V TILDEIDENT -> <:expr< ~$_:i$ >>
-      | i = V QUESTIONIDENTCOLON; e = SELF -> <:expr< ?$_:i$: $e$ >>
-      | i = V QUESTIONIDENT -> <:expr< ?$_:i$ >> ] ]
+      [ "~"; "{"; lpeo = V (LIST1 ipatt_tcon_fun_binding SEP ";"); "}" ->
+          <:expr< ~{$_list:lpeo$ } >>
+      | "?"; "{"; p = ipatt_tcon; eo = V (OPT fun_binding); "}" ->
+          <:expr< ?{$p$ $_opt:eo$ } >>
+      | i = V TILDEIDENTCOLON; e = SELF ->
+          let _ = warning_deprecated_since_6_00 loc in
+          <:expr< ~{$_lid:i$ = $e$} >>
+      | i = V TILDEIDENT ->
+          let _ = warning_deprecated_since_6_00 loc in
+          <:expr< ~{$_lid:i$} >>
+      | i = V QUESTIONIDENTCOLON; e = SELF ->
+          let _ = warning_deprecated_since_6_00 loc in
+          <:expr< ?{$_lid:i$ = $e$} >>
+      | i = V QUESTIONIDENT ->
+          let _ = warning_deprecated_since_6_00 loc in
+          <:expr< ?{$_lid:i$} >> ] ]
+  ;
+  ipatt_tcon_fun_binding:
+    [ [ p = ipatt_tcon; eo = V (OPT fun_binding) -> (p, eo) ] ]
   ;
   expr: LEVEL "simple"
     [ [ "`"; s = V ident "" -> <:expr< ` $_:s$ >> ] ]
@@ -777,22 +948,35 @@ EXTEND
   ;
 END;
 
+value quotation_content s = do {
+  loop 0 where rec loop i =
+    if i = String.length s then ("", s)
+    else if s.[i] = ':' || s.[i] = '@' then
+      let i = i + 1 in
+      (String.sub s 0 i, String.sub s i (String.length s - i))
+    else loop (i + 1)
+};
+
 EXTEND
   GLOBAL: interf implem use_file top_phrase expr patt;
   interf:
     [ [ "#"; n = V LIDENT; dp = OPT expr; ";" ->
-          ([(<:sig_item< # $_lid:n$ $opt:dp$ >>, loc)], True)
-      | si = sig_item_semi; (sil, stopped) = SELF -> ([si :: sil], stopped)
-      | EOI -> ([], False) ] ]
+          ([(<:sig_item< # $_lid:n$ $opt:dp$ >>, loc)], None)
+      | si = sig_item_semi; (sil, stopped) = SELF ->
+          ([si :: sil], stopped)
+      | EOI ->
+          ([], Some loc) ] ]
   ;
   sig_item_semi:
     [ [ si = sig_item; ";" -> (si, loc) ] ]
   ;
   implem:
     [ [ "#"; n = V LIDENT; dp = OPT expr; ";" ->
-          ([(<:str_item< # $_lid:n$ $opt:dp$ >>, loc)], True)
-      | si = str_item_semi; (sil, stopped) = SELF -> ([si :: sil], stopped)
-      | EOI -> ([], False) ] ]
+          ([(<:str_item< # $_lid:n$ $opt:dp$ >>, loc)], None)
+      | si = str_item_semi; (sil, stopped) = SELF ->
+          ([si :: sil], stopped)
+      | EOI ->
+          ([], Some loc) ] ]
   ;
   str_item_semi:
     [ [ si = str_item; ";" -> (si, loc) ] ]
@@ -814,26 +998,12 @@ EXTEND
   ;
   expr: LEVEL "simple"
     [ [ x = QUOTATION ->
-          let x =
-            try
-              let i = String.index x ':' in
-              (String.sub x 0 i,
-               String.sub x (i + 1) (String.length x - i - 1))
-            with
-            [ Not_found -> ("", x) ]
-          in
-          Pcaml.handle_expr_quotation loc x ] ]
+          let con = quotation_content x in
+          Pcaml.handle_expr_quotation loc con ] ]
   ;
   patt: LEVEL "simple"
     [ [ x = QUOTATION ->
-          let x =
-            try
-              let i = String.index x ':' in
-              (String.sub x 0 i,
-               String.sub x (i + 1) (String.length x - i - 1))
-            with
-            [ Not_found -> ("", x) ]
-          in
-          Pcaml.handle_patt_quotation loc x ] ]
+          let con = quotation_content x in
+          Pcaml.handle_patt_quotation loc con ] ]
   ;
 END;
