@@ -1,7 +1,10 @@
-(* camlp5r pa_extend.cmo q_MLast.cmo *)
-(* $Id: pa_rp.ml,v 1.16 2010/02/19 09:06:38 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: pa_rp.ml,v 6.1 2010-09-15 16:00:25 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_extend.cmo";
+#load "q_MLast.cmo";
+
 open Exparser;
 open Pcaml;
 
