@@ -1,15 +1,24 @@
 (* camlp5r *)
-(* $Id: pr_dump.ml,v 1.8 2010/02/19 09:06:38 deraugla Exp $ *)
+(* $Id: pr_dump.ml,v 6.3 2010-09-30 16:18:19 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+open Versdep;
+
 value open_out_file () =
   match Pcaml.output_file.val with
   [ Some f -> open_out_bin f
-  | None -> do { set_binary_mode_out stdout True; stdout } ]
+  | None -> do { pervasives_set_binary_mode_out stdout True; stdout } ]
 ;
 
-value interf ast = do {
-  let fname = Pcaml.input_file.val in
+value first_loc_of_ast =
+  fun
+  [ [(_, loc) :: _] -> loc
+  | [] -> Ploc.dummy ]
+;
+
+value interf (ast, eoi_loc) = do {
+  let loc = first_loc_of_ast ast in
+  let fname = Ploc.file_name loc in
   let pt = Ast2pt.interf fname (List.map fst ast) in
   let oc = open_out_file () in
   output_string oc Pconfig.ast_intf_magic_number;
@@ -21,8 +30,9 @@ value interf ast = do {
   | None -> () ]
 };
 
-value implem ast = do {
-  let fname = Pcaml.input_file.val in
+value implem (ast, eoi_loc) = do {
+  let loc = first_loc_of_ast ast in
+  let fname = Ploc.file_name loc in
   let pt = Ast2pt.implem fname (List.map fst ast) in
   let oc = open_out_file () in
   output_string oc Pconfig.ast_impl_magic_number;
