@@ -1,7 +1,10 @@
-(* camlp5r pa_macro.cmo q_MLast.cmo *)
-(* $Id: camlp5_top.ml,v 1.7 2010/02/19 09:06:41 deraugla Exp $ *)
+(* camlp5r *)
+(* $Id: camlp5_top.ml,v 6.1 2010-09-15 16:00:49 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+#load "pa_macro.cmo";
+#load "q_MLast.cmo";
+
 open Parsetree;
 open Lexing;
 
@@ -47,7 +50,12 @@ value highlight_locations lb loc1 loc2 =
 value print_location lb loc =
   if String.length Toploop.input_name.val = 0 then
     highlight_locations lb loc (-1, -1)
-  else Toploop.print_location Format.err_formatter (Ast2pt.mkloc loc)
+  else
+    IFDEF OCAML_VERSION <= OCAML_2_99 THEN
+      Toploop.print_location (Ast2pt.mkloc loc)
+    ELSE
+      Toploop.print_location Format.err_formatter (Ast2pt.mkloc loc)
+    END
 ;
 
 value wrap f shfn lb =
@@ -96,7 +104,7 @@ value first_phrase = ref True;
 value toplevel_phrase cs = do {
   if Sys.interactive.val && first_phrase.val then do {
     first_phrase.val := False;
-    Printf.eprintf "\tCamlp5 Parsing version %s\n\n" Pcaml.version;
+    Printf.eprintf "\tCamlp5 parsing version %s\n\n" Pcaml.version;
     flush stderr
   }
   else ();
@@ -107,7 +115,9 @@ value toplevel_phrase cs = do {
 
 Pcaml.add_directive "load"
   (fun
-   [ Some <:expr< $str:s$ >> -> Topdirs.dir_load Format.std_formatter s
+   [ Some <:expr< $str:s$ >> ->
+       IFDEF OCAML_VERSION <= OCAML_2_99 THEN Topdirs.dir_load s
+       ELSE Topdirs.dir_load Format.std_formatter s END
    | Some _ | None -> raise Not_found ]);
 
 Pcaml.add_directive "directory"
@@ -170,14 +180,14 @@ Toploop.parse_use_file.val :=
   wrap use_file (fun lb -> lb.lex_curr_pos - lb.lex_start_pos)
 ;
 
-IFDEF
-  OCAML_3_08_0 OR OCAML_3_08_1 OR OCAML_3_08_2 OR OCAML_3_08_3 OR OCAML_3_08_4
-THEN
-  DEFINE OCAML_3_08
-END;
-
 Pcaml.warning.val :=
   fun loc txt ->
-    Toploop.print_warning (Ast2pt.mkloc loc) Format.err_formatter
-      (IFDEF OCAML_3_08 THEN Warnings.Other txt
-       ELSE Warnings.Camlp4 txt END);
+    IFDEF OCAML_VERSION <= OCAML_2_00 THEN
+      Toploop.print_warning (Ast2pt.mkloc loc) txt
+    ELSIFDEF OCAML_VERSION <= OCAML_2_99 THEN
+      Toploop.print_warning (Ast2pt.mkloc loc) (Warnings.Other txt)
+    ELSE
+      Toploop.print_warning (Ast2pt.mkloc loc) Format.err_formatter
+        (IFDEF OCAML_VERSION <= OCAML_3_08_4 THEN Warnings.Other txt
+         ELSE Warnings.Camlp4 txt END)
+    END;
