@@ -1,29 +1,12 @@
 (* camlp5r pa_macro.cmo *)
-(* $Id: rprint.ml,v 1.28 2010/02/19 09:06:41 deraugla Exp $ *)
+(* $Id: rprint.ml,v 6.12 2010-11-23 15:48:13 deraugla Exp $ *)
 (* Copyright (c) INRIA 2007-2010 *)
 
+IFDEF OCAML_VERSION >= OCAML_3_03 THEN
+
 open Format;
 open Outcometree;
 
-IFDEF
-  OCAML_3_08_0 OR OCAML_3_08_1 OR OCAML_3_08_2 OR OCAML_3_08_3 OR OCAML_3_08_4
-THEN
-  DEFINE OCAML_3_08
-END;
-
-IFDEF OCAML_3_11 OR OCAML_3_11_0 OR OCAML_3_11_1 OR OCAML_3_11_2 OR
-  OCAML_3_12_0
-THEN
-  DEFINE AFTER_OCAML_3_11
-END;
-
-IFDEF
-  OCAML_3_10 OR OCAML_3_10_0 OR OCAML_3_10_1 OR OCAML_3_10_2 OR
-  OCAML_3_10_3 OR AFTER_OCAML_3_11 
-THEN
-  DEFINE AFTER_OCAML_3_10
-END;
-
 exception Ellipsis;
 value cautious f ppf arg =
   try f ppf arg with [ Ellipsis -> fprintf ppf "..." ]
@@ -62,9 +45,15 @@ value print_out_value ppf tree =
   and print_simple_tree ppf =
     fun
     [ Oval_int i -> fprintf ppf "%i" i
-    | Oval_int32 i -> fprintf ppf "%lil" i
-    | Oval_int64 i -> fprintf ppf "%LiL" i
-    | Oval_nativeint i -> fprintf ppf "%nin" i
+    | IFDEF OCAML_VERSION >= OCAML_3_07 THEN
+        Oval_int32 i -> fprintf ppf "%lil" i
+      END
+    | IFDEF OCAML_VERSION >= OCAML_3_07 THEN
+        Oval_int64 i -> fprintf ppf "%LiL" i
+      END
+    | IFDEF OCAML_VERSION >= OCAML_3_07 THEN
+        Oval_nativeint i -> fprintf ppf "%nin" i
+      END
     | Oval_float f -> fprintf ppf "%.12g" f
     | Oval_char c -> fprintf ppf "'%s'" (Char.escaped c)
     | Oval_string s ->
@@ -128,17 +117,57 @@ value pr_present =
   print_list (fun ppf s -> fprintf ppf "`%s" s) (fun ppf -> fprintf ppf "@ ")
 ;
 
+value default_lang =
+  try Sys.getenv "LC_ALL" with
+  [ Not_found ->
+      try Sys.getenv "LC_MESSAGES" with
+      [ Not_found -> try Sys.getenv "LANG" with [ Not_found -> "" ] ] ]
+;
+
+value utf8 =
+  let s = default_lang in
+  let utf8_str = "utf-8" in
+  let slen = String.length s in
+  let ulen = String.length utf8_str in
+  slen >= ulen &&
+  String.lowercase (String.sub s (slen - ulen) ulen) = utf8_str
+;
+
+(* Type variables in Greek *)
+
+value greek_tab =
+  [| "α"; "β"; "γ"; "δ"; "ε"; "ζ"; "η"; "θ"; "ι"; "κ"; "λ"; "μ"; "ν"; "ξ";
+     "ο"; "π"; "ρ"; "σ"; "τ"; "υ"; "φ"; "χ"; "ψ"; "ω" |]
+;
+value index_tab = [| ""; "₁"; "₂"; "₃"; "₄"; "₅"; "₆"; "₇"; "₈"; "₉" |];
+
+value try_greek s = do {
+  if utf8 then do {
+    if String.length s = 1 then do {
+      let c = Char.code s.[0] - Char.code 'a' in
+      let g = greek_tab.(c mod Array.length greek_tab) in
+      let n = c / Array.length greek_tab in
+      if n < Array.length index_tab then ("", g ^ index_tab.(n))
+      else ("'", s)
+    }
+    else ("'", s)
+  }
+  else ("'", s)
+};
+
 (* Types *)
 
 value rec print_out_type ppf =
   fun
-  [ Otyp_alias ty s -> fprintf ppf "@[%a as '%s@]" print_out_type ty s
+  [ Otyp_alias ty s ->
+      let (q, s) = try_greek s in
+      fprintf ppf "@[%a as %s%s@]" print_out_type ty q s
   | ty -> print_out_type_1 ppf ty ]
 and print_out_type_1 ppf =
   fun
   [ Otyp_arrow lab ty1 ty2 ->
-      fprintf ppf "@[%s%a ->@ %a@]" (if lab <> "" then lab ^ ":" else "")
-        print_out_type_2 ty1 print_out_type_1 ty2
+      fprintf ppf "@[%s%a %s@ %a@]" (if lab <> "" then lab ^ ":" else "")
+        print_out_type_2 ty1 (if utf8 then "→" else "->") print_out_type_1 ty2
   | ty -> print_out_type_2 ppf ty ]
 and print_out_type_2 ppf =
   fun
@@ -148,7 +177,9 @@ and print_out_type_2 ppf =
   | ty -> print_simple_out_type ppf ty ]
 and print_simple_out_type ppf =
   fun
-  [ Otyp_var ng s -> fprintf ppf "'%s%s" (if ng then "_" else "") s
+  [ Otyp_var ng s ->
+      let (q, s) = try_greek s in
+      fprintf ppf "%s%s%s" q (if ng then "_" else "") s
   | Otyp_constr id [] -> fprintf ppf "@[%a@]" print_ident id
   | Otyp_tuple tyl ->
       fprintf ppf "@[<1>(%a)@]" (print_typlist print_out_type " *") tyl
@@ -167,8 +198,8 @@ and print_simple_out_type ppf =
         | Ovar_name id tyl ->
             fprintf ppf "@[%a%a@]" print_typargs tyl print_ident id ]
       in
-      fprintf ppf "%s[|%s@[<hv>@[<hv>%a@]%a|]@]" (if non_gen then "_" else "")
-        (if closed then if tags = None then " " else "< "
+      fprintf ppf "%s[%s@[<hv>@[<hv>%a@]%a ]@]" (if non_gen then "_" else "")
+        (if closed then if tags = None then "= " else "< "
          else if tags = None then "> "
          else "? ")
         print_fields poly_variants
@@ -181,37 +212,64 @@ and print_simple_out_type ppf =
   | Otyp_manifest ty1 ty2 ->
       fprintf ppf "@[<2>%a ==@ %a@]" print_out_type ty1 print_out_type ty2
   | Otyp_abstract -> fprintf ppf "'abstract"
-  | Otyp_alias _ _ | Otyp_poly _ _
-  | Otyp_arrow _ _ _ | Otyp_constr _ [_ :: _] as ty ->
+  | Otyp_alias _ _ | Otyp_arrow _ _ _ | Otyp_constr _ [_ :: _] as ty ->
       fprintf ppf "@[<1>(%a)@]" print_out_type ty
-  | x ->
-      IFDEF OCAML_3_08 THEN
-        match x with
-        [ Otyp_sum constrs _ ->
-            fprintf ppf "@[<hv>[ %a ]@]"
-              (print_list print_out_constr (fun ppf -> fprintf ppf "@ | "))
-              constrs
-        | Otyp_record lbls _ ->
-            fprintf ppf "@[<hv 2>{ %a }@]"
-              (print_list print_out_label (fun ppf -> fprintf ppf ";@ ")) lbls
-        | _ -> assert False ]
-      ELSE
-        match x with
-        [ Otyp_sum constrs ->
-            fprintf ppf "@[<hv>[ %a ]@]"
-              (print_list print_out_constr (fun ppf -> fprintf ppf "@ | "))
-              constrs
-        | Otyp_record lbls ->
-            fprintf ppf "@[<hv 2>{ %a }@]"
-              (print_list print_out_label (fun ppf -> fprintf ppf ";@ ")) lbls
-        | _ -> assert False ]
-      END ]
+  | IFDEF OCAML_VERSION >= OCAML_3_05 THEN
+      Otyp_poly _ _ as ty ->
+        fprintf ppf "@[<1>(%a)@]" print_out_type ty
+    END
+  | IFDEF OCAML_VERSION >= OCAML_3_12_0 THEN
+      Otyp_module p n tyl -> do {
+        fprintf ppf "@[<1>(module %s" p;
+        let first = ref True in
+        List.iter2
+          (fun s t ->
+             let sep =
+               if first.val then do { first.val := False; "with" } else "and"
+             in
+             fprintf ppf " %s type %s = %a" sep s print_out_type t)
+          n tyl;
+        fprintf ppf ")@]"
+      }
+    END
+  | IFDEF OCAML_VERSION = OCAML_3_13_0_gadt THEN
+      Otyp_sum constrs ->
+        fprintf ppf "@[<hv>[ %a ]@]"
+          (print_list print_out_constr_gadt_opt
+             (fun ppf -> fprintf ppf "@ | "))
+          constrs
+    ELSIFDEF OCAML_VERSION > OCAML_3_06 AND OCAML_VERSION <= OCAML_3_08_4 THEN
+      Otyp_sum constrs _ ->
+        fprintf ppf "@[<hv>[ %a ]@]"
+          (print_list print_out_constr (fun ppf -> fprintf ppf "@ | "))
+          constrs
+    ELSE
+      Otyp_sum constrs ->
+        fprintf ppf "@[<hv>[ %a ]@]"
+          (print_list print_out_constr (fun ppf -> fprintf ppf "@ | "))
+          constrs
+    END
+  | IFDEF OCAML_VERSION > OCAML_3_06 AND OCAML_VERSION <= OCAML_3_08_4 THEN
+      Otyp_record lbls _ ->
+        fprintf ppf "@[<hv 2>{ %a }@]"
+          (print_list print_out_label (fun ppf -> fprintf ppf ";@ ")) lbls
+    ELSE
+      Otyp_record lbls ->
+        fprintf ppf "@[<hv 2>{ %a }@]"
+          (print_list print_out_label (fun ppf -> fprintf ppf ";@ ")) lbls
+    END ]
 and print_out_constr ppf (name, tyl) =
   match tyl with
   [ [] -> fprintf ppf "%s" name
   | _ ->
       fprintf ppf "@[<2>%s of@ %a@]" name
         (print_typlist print_out_type " and") tyl ]
+and print_out_constr_gadt_opt ppf (name, tyl, rto) =
+  match rto with
+  [ Some rt ->
+      let t = List.fold_right (fun t1 t2 -> Otyp_arrow "" t1 t2) tyl rt in
+      fprintf ppf "%s : %a" name print_out_type t
+  | None -> print_out_constr ppf (name, tyl) ]
 and print_out_label ppf (name, mut, arg) =
   fprintf ppf "@[<2>%s :@ %s%a@]" name (if mut then "mutable " else "")
     print_out_type arg
@@ -274,16 +332,16 @@ value rec print_out_class_type ppf =
         [ [] -> ()
         | tyl ->
             fprintf ppf "@[<1>[%a]@]@ "
-              (print_typlist Toploop.print_out_type.val ",") tyl ]
+              (print_typlist print_out_type ",") tyl ]
       in
       fprintf ppf "@[%a%a@]" pr_tyl tyl print_ident id
   | Octy_fun lab ty cty ->
       fprintf ppf "@[%s[ %a ] ->@ %a@]" (if lab <> "" then lab ^ ":" else "")
-        Toploop.print_out_type.val ty print_out_class_type cty
+        print_out_type ty print_out_class_type cty
   | Octy_signature self_ty csil ->
       let pr_param ppf =
         fun
-        [ Some ty -> fprintf ppf "@ @[(%a)@]" Toploop.print_out_type.val ty
+        [ Some ty -> fprintf ppf "@ @[(%a)@]" print_out_type ty
         | None -> () ]
       in
       fprintf ppf "@[<hv 2>@[<2>object%a@]@ %a@;<1 -2>end@]" pr_param self_ty
@@ -292,21 +350,21 @@ value rec print_out_class_type ppf =
 and print_out_class_sig_item ppf =
   fun
   [ Ocsg_constraint ty1 ty2 ->
-      fprintf ppf "@[<2>type %a =@ %a;@]" Toploop.print_out_type.val ty1
-        Toploop.print_out_type.val ty2
+      fprintf ppf "@[<2>type %a =@ %a;@]" print_out_type ty1
+        print_out_type ty2
   | Ocsg_method name priv virt ty ->
       fprintf ppf "@[<2>method %s%s%s :@ %a;@]"
         (if priv then "private " else "") (if virt then "virtual " else "")
-        name Toploop.print_out_type.val ty
+        name print_out_type ty
   | x ->
-      IFDEF AFTER_OCAML_3_10 THEN
+      IFDEF OCAML_VERSION >= OCAML_3_10 THEN
         failwith "Rprint.print_out_class_sig_item: not impl"
       ELSE
         match x with
         [ Ocsg_value name mut ty ->
             fprintf ppf "@[<2>value %s%s :@ %a;@]"
                (if mut then "mutable " else "") name
-               Toploop.print_out_type.val ty
+               print_out_type ty
         | _ -> assert False ]
       END ]
 ;
@@ -315,8 +373,7 @@ value rec print_out_module_type ppf =
   fun
   [ Omty_ident id -> fprintf ppf "%a" print_ident id
   | Omty_signature sg ->
-      fprintf ppf "@[<hv 2>sig@ %a@;<1 -2>end@]"
-        Toploop.print_out_signature.val sg
+      fprintf ppf "@[<hv 2>sig@ %a@;<1 -2>end@]" print_out_signature sg
   | Omty_functor name mty_arg mty_res ->
       fprintf ppf "@[<2>functor@ (%s : %a) ->@ %a@]" name
         print_out_module_type mty_arg print_out_module_type mty_res
@@ -324,9 +381,9 @@ value rec print_out_module_type ppf =
 and print_out_signature ppf =
   fun
   [ [] -> ()
-  | [item] -> fprintf ppf "%a;" Toploop.print_out_sig_item.val item
+  | [item] -> fprintf ppf "%a;" print_out_sig_item item
   | [item :: items] ->
-      fprintf ppf "%a;@ %a" Toploop.print_out_sig_item.val item
+      fprintf ppf "%a;@ %a" print_out_sig_item item
         print_out_signature items ]
 and print_out_sig_item ppf =
   fun
@@ -335,14 +392,25 @@ and print_out_sig_item ppf =
   | Osig_modtype name Omty_abstract ->
       fprintf ppf "@[<2>module type %s@]" name
   | Osig_modtype name mty ->
-      fprintf ppf "@[<2>module type %s =@ %a@]" name
-        Toploop.print_out_module_type.val mty
-  | Osig_module name mty _ ->
-      fprintf ppf "@[<2>module %s :@ %a@]" name
-        Toploop.print_out_module_type.val mty
-  | Osig_type td rs ->
-      print_out_type_decl (if rs = Orec_next then "and" else "type" )
-        ppf td
+      fprintf ppf "@[<2>module type %s =@ %a@]" name print_out_module_type mty
+  | IFDEF OCAML_VERSION <= OCAML_3_07 THEN
+      Osig_module name mty ->
+        fprintf ppf "@[<2>module %s :@ %a@]" name print_out_module_type mty
+    ELSE
+      Osig_module name mty _ ->
+        fprintf ppf "@[<2>module %s :@ %a@]" name
+          print_out_module_type mty
+    END
+  | IFDEF OCAML_VERSION <= OCAML_3_07 THEN
+      Osig_type tdl -> do {
+        print_out_type_decl "type" ppf (List.hd tdl);
+        List.iter (print_out_type_decl "and" ppf) (List.tl tdl);
+      }
+    ELSE
+      Osig_type td rs ->
+        print_out_type_decl (if rs = Orec_next then "and" else "type" )
+          ppf td
+    END
   | Osig_value name ty prims ->
       let kwd = if prims = [] then "value" else "external" in
       let pr_prims ppf =
@@ -355,25 +423,25 @@ and print_out_sig_item ppf =
             } ]
       in
       fprintf ppf "@[<2>%s %a :@ %a%a@]" kwd value_ident name
-        Toploop.print_out_type.val ty pr_prims prims
+        print_out_type ty pr_prims prims
   | x ->
-      IFDEF OCAML_3_08 THEN
+      IFDEF OCAML_VERSION <= OCAML_3_08_4 THEN
         failwith "Rprint.print_out_sig_item: not implemented case"
       ELSE
         match x with
         [ Osig_class vir_flag name params clt _ ->
             fprintf ppf "@[<2>class%s@ %a%s@ :@ %a@]"
               (if vir_flag then " virtual" else "") print_out_class_params
-              params name Toploop.print_out_class_type.val clt
+              params name print_out_class_type clt
         | Osig_class_type vir_flag name params clt _ ->
             fprintf ppf "@[<2>class type%s@ %a%s@ =@ %a@]"
               (if vir_flag then " virtual" else "") print_out_class_params
-              params name Toploop.print_out_class_type.val clt
+              params name print_out_class_type clt
         | _ -> assert False ]
       END ]
 and print_out_type_decl kwd ppf x =
   let (name, args, ty, priv, constraints) =
-    IFDEF OCAML_3_08 THEN
+    IFDEF OCAML_VERSION <= OCAML_3_08_4 THEN
       let (name, args, ty, priv) = x in
       (name, args, ty, priv, [])
     ELSE
@@ -381,13 +449,14 @@ and print_out_type_decl kwd ppf x =
     END
   in
   let constrain ppf (ty, ty') =
-    fprintf ppf "@ @[<2>constraint %a =@ %a@]" Toploop.print_out_type.val ty
-      Toploop.print_out_type.val ty'
+    fprintf ppf "@ @[<2>constraint %a =@ %a@]" print_out_type ty
+      print_out_type ty'
   in
   let print_constraints ppf params = List.iter (constrain ppf) params in
   let type_parameter ppf (ty, (co, cn)) =
-    fprintf ppf "%s'%s" (if not cn then "+" else if not co then "-" else "")
-      ty
+    let (q, ty) = try_greek ty in
+    fprintf ppf "%s%s%s" (if not cn then "+" else if not co then "-" else "")
+      q ty
   in
   let type_defined ppf =
     match args with
@@ -398,7 +467,7 @@ and print_out_type_decl kwd ppf x =
           (print_list type_parameter (fun ppf -> fprintf ppf "@ ")) args ]
   in
   fprintf ppf "@[<2>@[<hv 2>@[%s %t@] =@ %a@]%a@]" kwd type_defined
-    Toploop.print_out_type.val ty print_constraints constraints
+    print_out_type ty print_constraints constraints
 ;
 
 (* Phrases *)
@@ -410,7 +479,7 @@ value print_out_exception ppf exn outv =
   | Stack_overflow ->
       fprintf ppf "Stack overflow during evaluation (looping recursion?).@."
   | _ ->
-      fprintf ppf "@[Exception:@ %a.@]@." Toploop.print_out_value.val outv ]
+      fprintf ppf "@[Exception:@ %a.@]@." print_out_value outv ]
 ;
 
 value rec print_items ppf =
@@ -420,9 +489,9 @@ value rec print_items ppf =
       do {
         match valopt with
         [ Some v ->
-            fprintf ppf "@[<2>%a =@ %a@]" Toploop.print_out_sig_item.val tree
-              Toploop.print_out_value.val v
-        | None -> fprintf ppf "@[%a@]" Toploop.print_out_sig_item.val tree ];
+            fprintf ppf "@[<2>%a =@ %a@]" print_out_sig_item tree
+              print_out_value v
+        | None -> fprintf ppf "@[%a@]" print_out_sig_item tree ];
         if items <> [] then fprintf ppf "@ %a" print_items items else ()
       } ]
 ;
@@ -430,8 +499,8 @@ value rec print_items ppf =
 value print_out_phrase ppf =
   fun
   [ Ophr_eval outv ty ->
-      fprintf ppf "@[- : %a@ =@ %a@]@." Toploop.print_out_type.val ty
-        Toploop.print_out_value.val outv
+      fprintf ppf "@[- : %a@ =@ %a@]@." print_out_type ty print_out_value
+        outv
   | Ophr_signature [] -> ()
   | Ophr_signature items -> fprintf ppf "@[<v>%a@]@." print_items items
   | Ophr_exception (exn, outv) -> print_out_exception ppf exn outv ]
@@ -439,8 +508,12 @@ value print_out_phrase ppf =
 
 Toploop.print_out_value.val := print_out_value;
 Toploop.print_out_type.val := print_out_type;
-Toploop.print_out_class_type.val := print_out_class_type;
-Toploop.print_out_module_type.val := print_out_module_type;
+IFDEF OCAML_VERSION >= OCAML_3_05 THEN
+  Toploop.print_out_class_type.val := print_out_class_type;
+  Toploop.print_out_module_type.val := print_out_module_type;
+  Toploop.print_out_signature.val := print_out_signature;
+END;
 Toploop.print_out_sig_item.val := print_out_sig_item;
-Toploop.print_out_signature.val := print_out_signature;
 Toploop.print_out_phrase.val := print_out_phrase;
+
+END;
